function disp_info = find_pairs_and_displacements(img_info, resultsdir)


imgs = img_info.seg_cell_name;

savedir = [resultsdir filesep 'displacements_v2'];

if ~exist(savedir, 'dir')
    mkdir(savedir)
end

[~,~,movie_inds] = unique([ img_info.collection_num', img_info.replicate_num',img_info.movie_num'], 'rows');

savefile_master = [resultsdir filesep 'displacement_info_v2.mat'];

if ~exist(savefile_master, 'file')

    %save the state of the rng
    state = rng;
    
    %shuffle the rng
    rng('shuffle')
    
    %shuffle the image indices
    img_inds = randperm(length(imgs)-1);
    
    for k = 1:length(img_inds)
          
        i = img_inds(k);
        
        disp(['Finding displacements for image ' num2str(i) filesep num2str(length(imgs)-1)]);


        savefile = [savedir filesep 'imdat_' num2str(i)];
        
        if movie_inds(i) == movie_inds(i+1) && ~exist([savefile '.mat'], 'file')

            [can_start, final_name, final_exists, temp_name] = chunk_start(savefile);

            if can_start && ~final_exists

                load(imgs{i});
                im1 = seg_cell;
                
                load(imgs{i+1});
                im2 = seg_cell;

                [im1_regions, im2_regions] = find_bipartite_pairs(im1, im2);

                displacements = find_displacements(im1, im2, im1_regions, im2_regions);

                impath = imgs{i};

                save(savefile, 'im1_regions', 'im2_regions', 'displacements', 'impath')
            end
        end
    end


    for i = 1:(length(imgs)-1)

        savefile = [savedir filesep 'imdat_' num2str(i) '.mat'];
        if exist(savefile, 'file')
            load(savefile)
            im1_regions_all{i} = im1_regions;
            im2_regions_all{i} = im2_regions;
            displacements_all{i} = displacements;
            impath_all{i} = impath;
            img_num(i) = i;
        end
    end
    im1_regions = im1_regions_all;
    im2_regions = im2_regions_all;
    displacements = displacements_all;
    impath = impath_all;

    save(savefile_master, 'im1_regions', 'im2_regions', 'displacements', 'impath', 'movie_inds', 'img_num')
    
    %reset the rng
    rng(state)
else
    load(savefile_master)
end

disp_info.obj1_objnums = im1_regions;
disp_info.obj2_objnums = im2_regions;


disp_info.displacements = displacements;
disp_info.impath = impath;

disp_info.savedir = savedir;
disp_info.savefile_master = savefile_master;
disp_info.movie_inds = movie_inds;

disp_info.img_info = img_info;
    
    