function object_info = get_objects(img_info, disp_info, resultsdir, param)

obj_save_dir = [resultsdir filesep 'objects_v3'];
if ~exist(obj_save_dir, 'dir')
    mkdir(obj_save_dir)
end

collectionnum = param.collectionnum;
nframes = param.nframes;
ntransitions = param.ntransitions;

obj_save_file = [resultsdir filesep 'object_info_v3.mat'];
if ~exist(obj_save_file, 'file')

    %load all the data into a big matrix
    imgnums = cellfun(@(x,y) ones(size(x)).*y, disp_info.obj1_objnums, num2cell(1:length(disp_info.obj1_objnums)), 'UniformOutput', false);
    framenums = cellfun(@(x,y) ones(size(x)).*y, disp_info.obj1_objnums, num2cell(img_info.frame_num(1:end-1)), 'UniformOutput', false);
    movienums = cellfun(@(x,y) ones(size(x)).*y, disp_info.obj1_objnums, num2cell(img_info.movie_num(1:end-1)), 'UniformOutput', false);
    replicatenums = cellfun(@(x,y) ones(size(x)).*y, disp_info.obj1_objnums, num2cell(img_info.replicate_num(1:end-1)), 'UniformOutput', false);
    collectionnums = cellfun(@(x,y) ones(size(x)).*y, disp_info.obj1_objnums, num2cell(img_info.collection_num(1:end-1)), 'UniformOutput', false);
    
    movieids = cellfun(@(x,y) ones(size(x)).*y, disp_info.obj1_objnums, num2cell(disp_info.movie_inds(1:end-1)'), 'UniformOutput', false);
    
    %source obj, target obj, img num, frame num, movie num, collection
    obj1_nums = vertcat(disp_info.obj1_objnums{:});
    obj2_nums = vertcat(disp_info.obj2_objnums{:});
    imgnums = vertcat(imgnums{:});
    framenums = vertcat(framenums{:});
    movienums = vertcat(movienums{:});
    collectionnums = vertcat(collectionnums{:});
    replicatenums = vertcat(replicatenums{:});
    
    movieids = vertcat(movieids{:});
   
    %get all the images from a particular collection, less than or equal to a particular number of frames
    keepinds = all([collectionnums == collectionnum, framenums <= nframes],2);
    
    %truncate all the data
    obj1_nums = obj1_nums(keepinds);
    obj2_nums = obj2_nums(keepinds);
    imgnums = imgnums(keepinds);
    framenums = framenums(keepinds);
    movienums = movienums(keepinds);
    collectionnums = collectionnums(keepinds);
    replicatenums = replicatenums(keepinds);
    
    movieids = movieids(keepinds);
    
    [~, ~, objind] = unique([imgnums, obj1_nums; imgnums+1, obj2_nums], 'rows');
    objind = [objind(1:(length(objind)/2)), objind((length(objind)/2+1):end)];
    
    
    %add 250 transitions from each [movienum replicatenum] combination
    [~, ~, movieind] = unique([movienums, replicatenums], 'rows');
    umovies = unique(movieind);
    
    %for each movie
    for i = 1:length(umovies)
        %find the transitions in that movie
        transitioninds = find(movieind == umovies(i));
        
        transitions{i} = transitioninds(randsample(length(transitioninds), ntransitions));
        objids{i} = objind(transitions{i},:);
    end
    
    transitions = vertcat(transitions{:});
    objids = vertcat(objids{:});
    
    
    uobjids = unique(objids(:));
    
    %for each unique object id
    for i = 1:length(uobjids)
        disp(['Cropping image ' num2str(i) filesep num2str(length(uobjids))])
        
        obj_cell_save_file{i} = [obj_save_dir filesep 'obj_' num2str(i) '_cell.png'];
        obj_dna_save_file{i} = [obj_save_dir filesep 'obj_' num2str(i) '_dna.png'];
        
        
        if ~exist(obj_cell_save_file{i}, 'file')

            %figure out what image it's in and cut it out
            transinds = find(any(objids == uobjids(i),2));
            transition_ind = transitions(transinds(1));

            %if it is in the source image
            if objids(transinds(1),1) == uobjids(i)
                imgnum = imgnums(transition_ind,1);
                objnum = obj1_nums(transition_ind);
            %if it is in the target image
            else
                imgnum = imgnums(transition_ind,1)+1;
                objnum = obj2_nums(transition_ind);
            end

            load(img_info.seg_cell_name{imgnum});
            load(img_info.seg_dna_name{imgnum});
            
            seg_cell = seg_cell == objnum;
            seg_dna = seg_dna == objnum;

            [crop_cell, cr] = cropImg(seg_cell, 5);
            crop_dna = seg_dna(cr(1):cr(2), cr(3):cr(4));
            
            
            imwrite(crop_cell, obj_cell_save_file{i}, 'png')
            imwrite(crop_dna, obj_dna_save_file{i}, 'png')
        end
    end

    save(obj_save_file);
else
    load(obj_save_file);
end

for i = 1:length(uobjids)
    %figure out what image it's in and cut it out
    transinds = find(any(objids == uobjids(i),2));
    transition_ind = transitions(transinds(1));

    %if it is in the source image
      if objids(transinds(1),1) == uobjids(i)
        imgnum = imgnums(transition_ind,1);
        objnum = obj1_nums(transition_ind);
    %if it is in the target image
    else
        imgnum = imgnums(transition_ind,1)+1;
        objnum = obj2_nums(transition_ind);
    end
    objnums(i) = objnum;
    imgnums(i) = imgnum;

end

object_info.objnums = objnums;
object_info.imgnums = imgnums;

object_info.img_cell_file = obj_cell_save_file;
object_info.img_dna_file = obj_dna_save_file;

object_info.obj_save_file = obj_save_file;
object_info.uobjids = uobjids;


object_info.transitions = transitions;
object_info.objids = objids;

object_info.keepinds = keepinds;

object_info.obj1_nums = obj1_nums;
object_info.obj2_nums = obj2_nums;
object_info.imgnums = imgnums;
object_info.framenums = framenums;
object_info.conditionnums = movienums;
object_info.collectionnums = collectionnums;
object_info.replicatenums = replicatenums;

object_info.umovieid = movieind;

object_info.movieid = movieids;


%%% how to get the indexing into the main lists

[~, loc] = ismember(object_info.uobjids, object_info.objids);
[transition_ind ,b] = ind2sub(size(object_info.objids), loc);

object_info.obj_umovieid = object_info.umovieid(object_info.transitions(transition_ind));
% object_info.obj_movieid = object_info.movieid(object_info.transitions(transition_ind));
object_info.obj_condition = object_info.conditionnums(object_info.transitions(transition_ind));

[~, object_info.transition_obj_inds] = ismember(object_info.objids, object_info.uobjids);

condition_names = cell(1, length(unique(object_info.conditionnums)));

for i = 1:length(condition_names)
    file_inds = find((img_info.collection_num == unique(collectionnums)) &(img_info.movie_num == i));
    
    filename = img_info.img_path{file_inds(1)};
    
    %assume the protein name starts after the last '_' in the string
    underscore_inds = regexp(filename, '_');
    underscore_ind = underscore_inds(end);
    
    filesep_inds = regexp(filename, filesep);
    filesep_ind = find(filesep_inds > underscore_ind);
    filesep_ind = filesep_inds(filesep_ind(1));
    
    condition_names{i} = filename((underscore_ind+1):(filesep_ind-1));
end

object_info.condition_names = condition_names;