% Generates Figure 2, Table 2, Figure 5B
% from G. R. Johnson, T. E. Buck,D. P. Sullivan, G. K. Rohde, and 
% R. F. Murphy (2015) Joint modeling of cell and nuclear shape variation. 
% Molecular Biology of the Cell,
% http://murphylab.web.cmu.edu/software/2015_MBoC_Cell_And_Nuclear_Shape/index.html
%
% Author: Gregory R. Johnson (gj@cs.cmu.edu)
% Created: May 21, 2015
% Nov 2, 2015 - added header block - gj@cs.cmu.edu

disp('Performing setup')

if ~exist('param', 'var')
    param = [];
end

param = ml_initparam(param, struct('build', false));

script_alon_setup_v3

disp(['Running ' mfilename])

% seg_inds = intens_keep;
% suffix = '_G1';

seg_inds = ones(size(intens_keep));
suffix = '';


saveparent = [resultsdir filesep mfilename suffix];
% saveparent = '/home/gj/projects/shapeSpace/script_test_cell_nuc_pred';

if ~exist(saveparent, 'dir')
    mkdir(saveparent)
end

positions_cell_file = [saveparent filesep 'positions_cell.mat'];
positions_nuc_file = [saveparent filesep 'positions_nuc.mat'];

embed_best_dir = [resultsdir filesep 'embed_best_v2'];

if ~exist(positions_cell_file, 'file')
    load([savedir filesep 'cellModel_cell.mat'])
    p_cell_all = model.cellShapeModel.positions;
    clear model
    
    save(positions_cell_file, 'p_cell_all')
else
    load(positions_cell_file)
end

if ~exist(positions_nuc_file, 'file')

    load([savedir filesep 'cellModel_nuc.mat'])
    p_nuc_all = model.cellShapeModel.positions;
    clear model
    
    save(positions_nuc_file, 'p_nuc_all')
else
    load(positions_nuc_file)
end

ucond = unique(object_info.obj_condition);

rng('shuffle')
perm_inds = randperm(length(ucond));
ucond = ucond(perm_inds);
% ucond = [6,1,2,3,4,5,7];


niter = 100;


if param.build
    alldatfile = [saveparent filesep 'alldat_build.mat'];
else
    alldatfile = [saveparent filesep 'alldat.mat'];
end

nuc2cell_pvals = nan(size(p_cell_all,1), 1);
cell2nuc_pvals = nan(size(p_cell_all,1), 1);

if ~exist(alldatfile, 'file') %|| param.build == true
    for i = 1:length(ucond)
        disp(['ucond ' num2str(ucond(i))])

        inds = (object_info.obj_condition == ucond(i)) & seg_inds';
        cvinds = object_info.obj_umovieid(inds);

        p_cell = p_cell_all(inds,:);
        p_nuc = p_nuc_all(inds,:);

        
        savedir_cond = [saveparent filesep 'd' num2str(ucond(i))];
        
        param.method = 'gauss';
        
        
         param.name = 'nuc2cell';
        savedir_iter = [savedir_cond filesep param.name];
        
        param.normfunc = @(x) x;
        nuc2cell{ucond(i)} = kernel_pred_iter(p_nuc, p_cell, cvinds, niter, savedir_iter, param);
        
        nuc2cell_pvals(inds) = nuc2cell{ucond(i)}.y_pval;
        
         param.name = 'cell2nuc';
        savedir_iter = [savedir_cond filesep param.name];
        
        param.normfunc = @(x) x;
        cell2nuc{ucond(i)} = kernel_pred_iter(p_cell, p_nuc, cvinds, niter, savedir_iter, param);
        
        cell2nuc_pvals(inds) = cell2nuc{ucond(i)}.y_pval;

    end
    
    save(alldatfile, 'cell2nuc', 'nuc2cell', 'cell2nuc_pvals', 'nuc2cell_pvals')

else
    load(alldatfile)
end

ucond = sort(ucond)';

save_pred_dat(cell2nuc, nuc2cell, cell2nuc_pvals, nuc2cell_pvals, ucond, niter, object_info, saveparent);




nuc_pred_pos = nan(size(p_cell_all));
cell_pred_pos = nan(size(p_nuc_all));

nuc2cell_dist_all = nan(size(p_cell_all,1), 1);
cell2nuc_dist_all = nan(size(p_cell_all,1), 1);

cell2nuc_dist = cell2nuc;
nuc2cell_dist = nuc2cell;

for i = 1:length(ucond)
    inds = (object_info.obj_condition == ucond(i)) & seg_inds';
   
    [cell2nuc_dist_pval, cell2nuc_pred_norm_err{i}] = pred_pval(p_nuc_all(inds,:), cell2nuc{ucond(i)}.y_pred);
    [nuc2cell_dist_pval, nuc2cell_pred_norm_err{i}] = pred_pval(p_cell_all(inds,:), nuc2cell{ucond(i)}.y_pred);
    
    [cell2nuc_dist_pval_model, cell2nuc_model_err, cell2nuc_model_err_cb] = pred_pval_all(p_nuc_all(inds,:), cell2nuc{ucond(i)}.y_pred);    
    [nuc2cell_dist_pval_model, nuc2cell_model_err, nuc2cell_model_err_cb] = pred_pval_all(p_cell_all(inds,:), nuc2cell{ucond(i)}.y_pred);
 
    cell2nuc_dist_all(inds) = cell2nuc_pred_norm_err{i};
    nuc2cell_dist_all(inds) = nuc2cell_pred_norm_err{i};
    
    cell2nuc_dist{i}.y_pval = cell2nuc_dist_pval;
    cell2nuc_dist{i}.y_norm_err = cell2nuc_pred_norm_err{i};
    cell2nuc_dist{i}.y_model_pval = cell2nuc_dist_pval_model;
    cell2nuc_dist{i}.y_model_err = cell2nuc_model_err;
    cell2nuc_dist{i}.y_model_err_cb = cell2nuc_model_err_cb;

    nuc2cell_dist{i}.y_pval = nuc2cell_dist_pval;
    nuc2cell_dist{i}.y_norm_err = nuc2cell_pred_norm_err{i};
    nuc2cell_dist{i}.y_model_pval = nuc2cell_dist_pval_model;
    nuc2cell_dist{i}.y_model_err = nuc2cell_model_err;
    nuc2cell_dist{i}.y_model_err_cb = nuc2cell_model_err_cb;
    
    
end
save_pred_dat2(cell2nuc_dist, nuc2cell_dist, cell2nuc_dist_all, nuc2cell_dist_all, object_info.condition_names, object_info, [saveparent filesep 'pval-dist'], 'pval-dist');

% diff_err = cell2nuc_pred_dist_all_err./nuc2cell_pred_dist_all_err;
% lb = (cell2nuc_pred_dist_all_err_cb(:,1)./nuc2cell_pred_dist_all_err_cb(:,2))';
% hb = (cell2nuc_pred_dist_all_err_cb(:,2)./nuc2cell_pred_dist_all_err_cb(:,1))';
% 
% figure, subplot(1,2,1)
% plot([cell2nuc_pred_dist_all_err', cell2nuc_pred_dist_all_err']', nuc2cell_pred_dist_all_err_cb')
% hold on, scatter(cell2nuc_pred_dist_all_err, nuc2cell_pred_dist_all_err)
% plot(cell2nuc_pred_dist_all_err_cb', [nuc2cell_pred_dist_all_err', nuc2cell_pred_dist_all_err']')
% xlabel('normalized nuclear shape prediction error')
% ylabel('normalized cell shape prediction error')
% axis equal
% 
% subplot(1,2,2)
% errorbar(ucond, diff_err, diff_err-lb, diff_err-hb, 'rx')



%%%% synthesize each predicted shape

cellmodel = load([savedir filesep 'cellModel_cell.mat']);
nucmodel = load([savedir filesep 'cellModel_nuc.mat']);

nuc2cell_pvals_jaccard = nan(size(p_cell_all,1), 1);
cell2nuc_pvals_jaccard = nan(size(p_cell_all,1), 1);

nuc2cell_index = nuc2cell;
cell2nuc_index = cell2nuc;

for i = 1:length(ucond)
    inds = find((object_info.obj_condition == ucond(i)) & seg_inds');
    
    [nuc2cell_framefiles_pred{i}, ~ ] = shape_generator_pred_struct(cellmodel.model, nuc2cell{i}, [saveparent filesep 'synth_nuc2cell' filesep 'cond' num2str(i)], param);
    [cell2nuc_framefiles_pred{i}, ~ ] = shape_generator_pred_struct(nucmodel.model, cell2nuc{i}, [saveparent filesep 'synth_cell2nuc' filesep 'cond' num2str(i)], param);

    index_save_file = [saveparent filesep 'index_save_file' num2str(i) '.mat'];
    
    if ~exist(index_save_file, 'file')
        [nuc2cell_index_pval2, index_pred_cell, index_dists_cell] = index_sig2(cellmodel.model, inds, nuc2cell_framefiles_pred{i}, @jaccard, [saveparent filesep 'index_save_cell' filesep 'cond' num2str(i)]);
        [cell2nuc_index_pval2, index_pred_nuc, index_dists_nuc] = index_sig2(nucmodel.model, inds, cell2nuc_framefiles_pred{i}, @jaccard, [saveparent filesep 'index_save_nuc' filesep 'cond' num2str(i)]);
        
        if ~any(isnan(index_dists_cell(:))) && ~any(isnan(index_dists_nuc(:)))
            save(index_save_file, 'nuc2cell_index_pval2', 'index_pred_cell', 'index_dists_cell', ...
                                'cell2nuc_index_pval2', 'index_pred_nuc', 'index_dists_nuc')
        else
            continue
        end
    else
        load(index_save_file)
    end
    
    index_dists_cell = 1 - index_dists_cell;
    index_pred_cell = 1 - index_pred_cell;
    
    index_dists_nuc = 1 - index_dists_nuc;
    index_pred_nuc = 1 - index_pred_nuc;
    
    
    [nuc2cell_jaccard_pval, nuc2cell_jaccard_norm_err] = pval_dist(index_pred_cell, index_dists_cell );
    [cell2nuc_jaccard_pval, cell2nuc_jaccard_norm_err] = pval_dist(index_pred_nuc, index_dists_nuc);

    [nuc2cell_jaccard_pval_model, nuc2cell_jaccard_model_err, nuc2cell_jaccard_model_err_cb] = pval_dist_all(index_pred_cell, index_dists_cell );
    [cell2nuc_jaccard_pval_model, cell2nuc_jaccard_model_err, cell2nuc_jaccard_model_err_cb] = pval_dist_all(index_pred_nuc, index_dists_nuc );

    nuc2cell_pvals_jaccard(inds) = nuc2cell_jaccard_pval;
    cell2nuc_pvals_jaccard(inds) = cell2nuc_jaccard_pval;
 
    
    nuc2cell_index{i}.y_pval = nuc2cell_jaccard_pval;
    nuc2cell_index{i}.y_pred_err = index_pred_cell;
    nuc2cell_index{i}.y_dists = index_dists_cell;

    nuc2cell_index{i}.y_norm_err = nuc2cell_jaccard_norm_err;
    nuc2cell_index{i}.y_model_pval = nuc2cell_jaccard_pval_model;
    nuc2cell_index{i}.y_model_err = nuc2cell_jaccard_model_err;
    nuc2cell_index{i}.y_model_err_cb = nuc2cell_jaccard_model_err_cb;


    cell2nuc_index{i}.y_pval = cell2nuc_jaccard_pval;    
    cell2nuc_index{i}.y_pred_err = index_pred_nuc;
    cell2nuc_index{i}.y_dists = index_dists_nuc;

    cell2nuc_index{i}.y_norm_err = cell2nuc_jaccard_norm_err;
    cell2nuc_index{i}.y_model_pval = cell2nuc_jaccard_pval_model;
    cell2nuc_index{i}.y_model_err = cell2nuc_jaccard_model_err;
    cell2nuc_index{i}.y_model_err_cb = cell2nuc_jaccard_model_err_cb;

end

save_pred_dat2(cell2nuc_index, nuc2cell_index, cell2nuc_pvals_jaccard, nuc2cell_pvals_jaccard, object_info.condition_names, object_info, [saveparent filesep 'pval-jaccard'], 'jaccard');



synth_img_parent_dir = [saveparent filesep 'synth_imgs'];

if ~exist(synth_img_parent_dir, 'dir')
    mkdir(synth_img_parent_dir)
end

keepinds = 1:length(object_info.obj_condition);
for j = 1:length(ucond)
    synth_img_dir = [synth_img_parent_dir filesep 'cond' num2str(j)];
    if ~exist(synth_img_dir, 'dir')
        mkdir(synth_img_dir)
    end
    
    compound_ind = find((object_info.obj_condition == ucond(j)) & seg_inds');
    
    pmat = [nuc2cell_dist_all];
    pmat = pmat(compound_ind,:);
    dists = pdist2(0, pmat);
    [~,min_inds] = min(dists,[],2);
    min_inds = find(dists(min_inds) == dists);
    shape_inds = compound_ind(min_inds);
    pred_img_saver(cellmodel, nucmodel, nuc2cell_framefiles_pred{j}, cell2nuc_framefiles_pred{j}, keepinds(shape_inds), min_inds, 'synth_dist_best_cell', synth_img_dir )
    
    pmat = [nuc2cell_dist_all];
    pmat = pmat(compound_ind,:);
    dists = pdist2(max(pmat), pmat);
    [~,min_inds] = min(dists,[],2);
    min_inds = find(dists(min_inds) == dists);
    shape_inds = compound_ind(min_inds);
    pred_img_saver(cellmodel, nucmodel, nuc2cell_framefiles_pred{j}, cell2nuc_framefiles_pred{j}, keepinds(shape_inds), min_inds, 'synth_dist_worst_cell', synth_img_dir )

    pmat = [cell2nuc_dist_all];
    pmat = pmat(compound_ind,:);
    dists = pdist2(0, pmat);
    [~,min_inds] = min(dists,[],2);
    min_inds = find(dists(min_inds) == dists);
    shape_inds = compound_ind(min_inds);
    pred_img_saver(cellmodel, nucmodel, nuc2cell_framefiles_pred{j}, cell2nuc_framefiles_pred{j}, keepinds(shape_inds), min_inds, 'synth_dist_best_nuc', synth_img_dir )

    pmat = [cell2nuc_dist_all];
    pmat = pmat(compound_ind,:);
    dists = pdist2(max(pmat), pmat);
    [~,min_inds] = min(dists,[],2);
    min_inds = find(dists(min_inds) == dists);
    shape_inds = compound_ind(min_inds);
    pred_img_saver(cellmodel, nucmodel, nuc2cell_framefiles_pred{j}, cell2nuc_framefiles_pred{j}, keepinds(shape_inds), min_inds, 'synth_dist_worst_nuc', synth_img_dir )

    
    pmat = [cell2nuc_dist_all, nuc2cell_dist_all];
    pmat = pmat(compound_ind,:);
    % pmat(pmat>0.05) = 1;
    dists = pdist2([0,0; 0,1;1,0;1,1], pmat);
    [~,min_inds] = min(dists,[],2);
    shape_inds = compound_ind(min_inds);

    pred_img_saver(cellmodel, nucmodel, nuc2cell_framefiles_pred{j}, cell2nuc_framefiles_pred{j}, keepinds(shape_inds), min_inds, 'synth_dist', synth_img_dir )
    
   
    pmat = [cell2nuc_dist_all, nuc2cell_dist_all, cell2nuc_pvals_jaccard, nuc2cell_pvals_jaccard];
    pmat = pmat(compound_ind,:);
    % pmat(pmat>0.05) = 1;
    dists = pdist2([0,0,0,0; 0,1,0,1; 1,0,1,0; 1,1,1,1], pmat);
    [~,min_inds] = min(dists,[],2);
    shape_inds = compound_ind(min_inds);

    pred_img_saver(cellmodel, nucmodel, nuc2cell_framefiles_pred{j}, cell2nuc_framefiles_pred{j}, keepinds(shape_inds), min_inds, 'synth_dist-jaccard', synth_img_dir )
   
  
    pmat = [cell2nuc_pvals_jaccard, nuc2cell_pvals_jaccard];
    pmat = pmat(compound_ind,:);
    % pmat(pmat>0.05) = 1;
    dists = pdist2([0,0; 0,1;1,0;1,1], pmat);
    [~,min_inds] = min(dists,[],2);
    shape_inds = compound_ind(min_inds);

    pred_img_saver(cellmodel, nucmodel, nuc2cell_framefiles_pred{j}, cell2nuc_framefiles_pred{j}, keepinds(shape_inds), min_inds, 'synth_jaccard', synth_img_dir )
    
end



% 
% pred_img_dir = [saveparent filesep 'synth_imgs2'];
% 
% if ~exist(pred_img_dir, 'dir')
%     mkdir(pred_img_dir)
% end
% 
% condition_inds = find(object_info.obj_condition == find(ismember(object_info.condition_names, 'CD164')));
% dists = pdist2([0,0; 0,1;1,0;1,1], [cell2nuc_pvals(condition_inds), nuc2cell_pvals(condition_inds)]);
% [~,min_inds] = min(dists,[],2);
% cell_inds = condition_inds(min_inds);
% 
% cell_pvals = nuc2cell_pvals(min_inds);
% nuc_pvals = cell2nuc_pvals(min_inds);
% 
% if ~param.build
%     for i = 1:length(cell_inds)
%         
%         synthfile = [ pred_img_dir filesep 'shape' num2str(i) '.mat'];
%         synthfile_temp = [synthfile '.tmp'];
% 
%         if ~exist(synthfile, 'file') && ~exist(synthfile_temp, 'file')
%             system(['touch ' synthfile_temp]);
%             synthparam.position = cell_pred_pos(cell_inds(i),:); 
%             synthparam.tempdir = [pred_img_dir filesep 'temp' filesep 'cell' num2str(i)];
%             if ~exist(synthparam.tempdir, 'dir')
%                 mkdir(synthparam.tempdir)
%             end
%             cd(synthparam.tempdir)
% 
%             cell_pred = model2img({cellmodel.model}, synthparam);
% 
%             synthparam.position = nuc_pred_pos(cell_inds(i),:);
%             synthparam.tempdir = [pred_img_dir filesep 'temp' filesep 'nuc' num2str(i)];
%             if ~exist(synthparam.tempdir, 'dir')
%                 mkdir(synthparam.tempdir)
%             end
%             cd(synthparam.tempdir)
% 
% 
%             nuc_pred = model2img({nucmodel.model}, synthparam);
% 
%             save(synthfile, 'cell_pred', 'nuc_pred')
% 
%             cell_true = cellmodel.model.cellShapeModel.imfunc(cell_inds(i));
%             cell_pred = cell_pred{1};
% 
%             cell_img = [cell_true.*255, cell_pred(:,:,2)];
%             imwrite(cell_img, [pred_img_dir filesep 'cell' num2str(i) '.png'], 'png')
% 
% 
%             nuc_true = nucmodel.model.cellShapeModel.imfunc(cell_inds(i));
%             nuc_pred = nuc_pred{1};
%             nuc_img = [nuc_true.*255, nuc_pred(:,:,2)];
%             
%             imwrite(nuc_img, [pred_img_dir filesep 'nuc' num2str(i) '.png'], 'png')
% 
%             system(['rm ' synthfile_temp])
%         end
%     end
% end
% 
% % 
% pred_img_dir = [saveparent filesep 'synth_imgs'];

% if ~param.build
%     for i = 1:length(ucond)
%         cellnums = find((object_info.obj_condition == ucond(i)) & seg_inds');
%         for j = 1:length(cellnums)
% 
%             synthfile = [ pred_img_dir filesep 'shape' num2str(cellnums(j)) '.mat'];
% 
%             synthfile_temp = [synthfile '.tmp'];
%             if ~exist(synthfile, 'file') && ~exist(synthfile_temp, 'file')
%                 system(['touch ' synthfile_temp]);
% 
%                 disp(['pred:' num2str(cellnums(j)) filesep num2str(length(object_info.obj_condition))])
% 
% 
%                 synthparam.position = nuc2cell{i}.y_pred(j,:); 
%                 synthparam.tempdir = [pred_img_dir filesep 'temp' filesep 'cell' num2str(cellnums(j))];
%                 if ~exist(synthparam.tempdir, 'dir')
%                     mkdir(synthparam.tempdir)
%                 end
%                 cd(synthparam.tempdir)
% 
%                 cell_pred = model2img({cellmodel.model}, synthparam);
% 
%                 synthparam.position = cell2nuc{i}.y_pred(j,:);
%                 synthparam.tempdir = [pred_img_dir filesep 'temp' filesep 'nuc' num2str(cellnums(j))];
%                 if ~exist(synthparam.tempdir, 'dir')
%                     mkdir(synthparam.tempdir)
%                 end
%                 cd(synthparam.tempdir)
%                 
% 
%                 nuc_pred = model2img({nucmodel.model}, synthparam);
% 
%                 save(synthfile, 'cell_pred', 'nuc_pred')
% 
%                 system(['rm ' synthfile_temp])
%             end
%         end
%     end
% end
% 
% for i = 1:length(ucond)
%     disp(['Printing images for '  object_info.condition_names{i}])
% 
%     savedir = [pred_img_dir filesep object_info.condition_names{i}];
%     if ~exist(savedir, 'dir')
%         mkdir(savedir)
%     end
% 
%     sigdir = [savedir filesep 'sig'];
%     notsigdir = [savedir filesep 'notsig'];
% 
% 
%     inds = find((object_info.obj_condition == ucond(i)) & seg_inds');
% 
%     
%     c1 = 1;
%     c2 = 1;
%     n1 = 1;
%     n2 = 1;
%     
%     cell_sig_imgs = {};
%     cell_notsig_imgs = {};
%     
%     nuc_sig_imgs = {};
%     nuc_notsig_imgs = {};
%     
%     for j = 1:length(inds)
%         synthfile = [ pred_img_dir filesep 'shape' num2str(inds(j)) '.mat'];
%         if exist(synthfile, 'file')
%             load(synthfile)
%             if ~isempty(cell_pred)
% 
%                 cell_true = cellmodel.model.cellShapeModel.imfunc(inds(j));
%                 cell_pred = cell_pred{1};
% 
%                 cell_img = [cell_true.*255, cell_pred(:,:,2)];
% 
%                 if nuc2cell_pvals(inds(j)) < 0.05
%                     suffix = '_sig';
%                     cell_sig_imgs{c1} = cell_img';
%                     c1 = c1+1;
%                 else
%                     suffix = '_not_sig';
%                     cell_notsig_imgs{c2} = cell_img';
%                     c2 = c2+1;
%                 end
% 
%                 
%                 imwrite(cell_img, [savedir filesep 'cell' num2str(inds(j)) suffix '.png'], 'png');
%                 
%                 
%                 imwrite(~cell_img,[savedir filesep 'cell' num2str(inds(j)) suffix 'white.png'], 'png')
%                 
%             end
% 
%             if ~isempty(nuc_pred)
% 
%                 nuc_true = nucmodel.model.cellShapeModel.imfunc(inds(j));
%                 nuc_pred = nuc_pred{1};
% 
%                 nuc_img = [nuc_true.*255, nuc_pred(:,:,2)];
% 
%                 if cell2nuc_pvals(inds(j)) < 0.05
%                     suffix = '_sig';
%                     nuc_sig_imgs{n1} = nuc_img';
%                     n1 = n1+1;
%                 else
%                     suffix = '_not_sig';
%                     nuc_notsig_imgs{n2} = nuc_img';
%                     n2 = n2+1;
%                 end
% 
%                 imwrite(nuc_img, [savedir filesep 'nuc' num2str(inds(j)) suffix '.png'], 'png');
%                 
%                 
%                 imwrite(~nuc_img,[savedir filesep 'nuc' num2str(inds(j)) suffix 'white.png'], 'png')
%             end 
%         end    
%     end
    
%     if ~isempty(cell_sig_imgs)
%         imwrite([cell_sig_imgs{:}], [savedir filesep 'all_cell_sig.png'], 'png')
%     end
%     if ~isempty(cell_notsig_imgs)
%         imwrite([cell_notsig_imgs{:}], [savedir filesep 'all_cell_notsig.png'], 'png')
%     end
%     
%     if ~isempty(nuc_sig_imgs)
%         imwrite([nuc_sig_imgs{:}], [savedir filesep 'all_cnuc_sig.png'], 'png')
%     end
%     
%     if ~isempty(nuc_notsig_imgs)
%         imwrite([nuc_notsig_imgs{:}], [savedir filesep 'all_nuc_notsig.png'], 'png')
%     end
%     
% end




% % 
% % for i = 1:length(ucond)
% %     mse_nuc(i) = mean(pred_err_nuc{i});
% %     mse_cell(i) = mean(pred_err_cell{i});
% % 
% %     p_nuc_err{i} = mean(repmat(pred_err_nuc{i}, 1, niter) >= [pred_err_nuc_perm{i}{:}],2);
% %     p_nuc_mse(i) = mean(repmat(mse_nuc(i), 1, niter) >= mean([pred_err_nuc_perm{i}{:}],1),2);
% %     p_cell_err{i} = mean(repmat(pred_err_cell{i}, 1, niter) >= [pred_err_cell_perm{i}{:}],2);
% %     p_cell_mse(i) = mean(repmat(mse_cell(i), 1, niter) >= mean([pred_err_cell_perm{i}{:}],1),2);
% % end
% % 
% % t = table(object_info.condition_names', mse_nuc', p_nuc_mse', mse_cell', p_cell_mse');
% % t.Properties.VariableNames = {'Condition', 'NuclerShapeMSE', 'NuclerShapeMSEp', 'CellShapeMSE', 'CellShapeMSEp'};
% % 
% % writetable(t, [saveparent filesep 'table.csv'])
% % 
% % [mse_nuc, mse_cell]
% % 
% % figure('color', 'w'), 
% % subplot(1,2, 1)
% % bar(mse_nuc')
% % xlabel('condition')
% % ylabel('mean squared prediction error')
% % 
% % subplot(1,2, 2), bar(mse_cell')
% % xlabel('condition')
% % 
% % 
% % param.subsize = 6000;
% % param.synthorder = randperm(size(p_nuc_all,1));
% % 
% % cell_shape_figure = [saveparent filesep 'shapeSpace_cell.tif'];
% % if ~exist(cell_shape_figure, 'file')
% %     load('cellModel_cell.mat')
% %     figure, 
% %     showShapeSpaceFigure(model, object_info.obj_condition, param);
% %     saveas(gcf, cell_shape_figure, 'tif')
% % end
% %     
% % param.subsize = 1000;
% % nuc_shape_figure = [saveparent filesep 'shapeSpace_nuc.tif'];
% % if ~exist(nuc_shape_figure, 'file')
% %     load('cellModel_nuc.mat')
% %     figure, 
% %     showShapeSpaceFigure(model, object_info.obj_condition, param);
% %     saveas(gcf, nuc_shape_figure, 'tif')
% % end

