function [ seg_dna_final, seg_cell_final ] = segCells_alon_v3(im_dapi)
%cell segmentation code specificly written to segment the images in the
%dataset from https://www.broadinstitute.org/bbbc/BBBC021/
%
% Gregory R. Johnson, 5/15/14

%SEGCELLS Summary of this function goes here
%   Detailed explanation goes here
% if ~exist('im_dapi', 'var')
%     im_dapi = imread('/Users/greg/Documents/Murphylab/projects/cellOrganizer/BBBC021/images/Week1_22123/Week1_150607_B03_s1_w114129FBE-AB04-4F79-B8AF-CFA7D5524D53.tif');
%     im_actin = imread('/Users/greg/Documents/Murphylab/projects/cellOrganizer/BBBC021/images/Week1_22123/Week1_150607_B03_s1_w41094DA98-6064-4EA2-B4A7-02A38208692F.tif');
%     im_tub = imread('/Users/greg/Documents/Murphylab/projects/cellOrganizer/BBBC021/images/Week1_22123/Week1_150607_B03_s1_w114129FBE-AB04-4F79-B8AF-CFA7D5524D53.tif');
% end

im_dapi = double(im_dapi);

im2 = imfilter(im_dapi, fspecial('gaussian', 10,1));

im2 = mat2gray(im2);

background = imopen(im2, strel('disk', 50));

im3 = imadjust(im2 - background);
im4 = im2bw(im3, graythresh(im3));

seg_dna = imfill(im4, 'holes');

seg_dna = (watershed(-imfilter(bwdist(~seg_dna), fspecial('gaussian', 30,3)))>0) .* (seg_dna);

seg_cell_fg = (im3.*32) > (ml_imgcommonpixel(round(im3.*32))+2);



% im_strel = imerode(seg_dna, strel('disk', 4));
% uregions = unique(im_strel(:));
% uregions(uregions == 0) = [];
% 
% seg_dna = bwlabeln(ismember(seg_dna, uregions));


seeds = seg_dna > 0;
seeds(:,1) = 1;
seeds(:,end) = 1;
seeds(1,:) = 1;
seeds(end,:) = 1;

%find the watershed cell boundaries from the segmented dna
regions = watershed(imimposemin(-im2, seeds));

seg_cell = (regions>0).*seg_cell_fg;

seg_cell = (watershed(-seg_cell.*imfilter(double(bwdist(~seg_cell)), fspecial('gaussian', 10,5)))>0).*seg_cell;

seg_dna = seg_dna.*seg_cell;

seg_cell = bwlabeln(seg_cell);

%get the number of cell regions
u_regions = unique(seg_cell(:));
u_regions(u_regions == 0) = [];

%get an image-boundary image
im_bounds = ones(size(seg_cell));
im_bounds(3:end-2,3:end-2) = 0;

seg_cell_final = zeros(size(seg_cell));
seg_dna_pre_final = zeros(size(seg_cell));

%for every cell region
for i = 1:length(u_regions)
    %get the largest object cell region. This is the segmented cell
    c = ml_findmainobj(seg_cell == u_regions(i));
        
    %only add the image to the final segmentation if
    if sum(c(:) .* im_bounds(:)) == 0
        cellimg = logical(imfill(c, 'holes'));
        
        if sum((seg_dna(:)>0) .*(cellimg(:)>0)) > pi*5^2 
            seg_cell_final(cellimg) = u_regions(i);
            seg_dna_pre_final(logical(cellimg .* seg_dna)) = u_regions(i);
        else
            seg_cell_final(cellimg) = -1;
            seg_dna_pre_final(logical(cellimg .* seg_dna)) = -1;
        end
    end
end


seg_dna_final = seg_dna_pre_final;
% %grj, added this extra segmentation step to get better nuclei. 1/18/15
% 
% u_regions = unique(seg_cell_final(:));
% %remove background and bad nuclei
% u_regions(ismember(u_regions, [-1, 0])) = [];
% 
% 
% seg_dna_final = zeros(size(seg_cell));
% 
% im_dapi_blur = imfilter(im_dapi, fspecial('gaussian', 30, 4));
% 
% for i = 1:length(u_regions)
%     cell_region = (seg_dna_pre_final == u_regions(i)).*im_dapi_blur;
%     
%     dna_region = (cell_region > ml_rcthreshold(cell_region));
%     dna_region = imfill(dna_region, 'holes');
%     seg_dna_final(dna_region) = u_regions(i);
%     
% end



end

