function [ seg_dna_final, seg_cell_final ] = segCells_alon_v3(im_dapi)
%cell segmentation code specificly written to segment the images in the
%dataset from https://www.broadinstitute.org/bbbc/BBBC021/
%
% Gregory R. Johnson, 5/15/14

%SEGCELLS Summary of this function goes here
%   Detailed explanation goes here
% if ~exist('im_dapi', 'var')
%     im_dapi = imread('/Users/greg/Documents/Murphylab/projects/cellOrganizer/BBBC021/images/Week1_22123/Week1_150607_B03_s1_w114129FBE-AB04-4F79-B8AF-CFA7D5524D53.tif');
%     im_actin = imread('/Users/greg/Documents/Murphylab/projects/cellOrganizer/BBBC021/images/Week1_22123/Week1_150607_B03_s1_w41094DA98-6064-4EA2-B4A7-02A38208692F.tif');
%     im_tub = imread('/Users/greg/Documents/Murphylab/projects/cellOrganizer/BBBC021/images/Week1_22123/Week1_150607_B03_s1_w114129FBE-AB04-4F79-B8AF-CFA7D5524D53.tif');
% end

im_dapi = double(im_dapi);

im2 = imfilter(im_dapi, fspecial('gaussian', 10,1));

im2 = mat2gray(im2);

background = imopen(im2, strel('disk', 50));

im3 = imadjust(im2 - background);
im4 = im2bw(im3, graythresh(im3));

seg_dna = imfill(im4, 'holes');

seg_dna = (watershed(-imfilter(bwdist(~seg_dna), fspecial('gaussian', 30,3)))>0) .* (seg_dna);

seg_cell_fg = (im3.*32) > (ml_imgcommonpixel(round(im3.*32))+2);



% im_strel = imerode(seg_dna, strel('disk', 4));
% uregions = unique(im_strel(:));
% uregions(uregions == 0) = [];
% 
% seg_dna = bwlabeln(ismember(seg_dna, uregions));


% regions = bwlabeln(seg_dna);
% uregions = unique(regions(:));
% uregions(uregions == 0) = [];

dna_regions = bwlabeln(seg_dna);

seg_cell = seg_cell_fg;

cell_regions = bwlabeln(seg_cell);
uregions = unique(cell_regions(:));
uregions(uregions == 0) = [];

%get an image-boundary image
im_bounds = ones(size(seg_cell));
im_bounds(3:end-2,3:end-2) = 0;

seg_cell_final = zeros(size(seg_cell));
seg_dna_pre_final = zeros(size(seg_cell));

%for every cell region
for i = 1:length(uregions)
    %count the number of dna objects
    dna_inds = unique(dna_regions(cell_regions == uregions(i)));
    dna_inds(dna_inds == 0) = [];
    
    c = cell_regions == uregions(i);
        
    %only add the image to the final segmentation if
    if (length(dna_inds) == 1) && (sum(c(:) .* im_bounds(:)) == 0)
        cellimg = logical(imfill(c, 'holes'));
        
        if sum((seg_dna(:)>0) .*(cellimg(:)>0)) > pi*5^2 
            seg_cell_final(cellimg) = uregions(i);
            seg_dna_pre_final(logical(cellimg .* seg_dna)) = uregions(i);
%         else
%             seg_cell_final(cellimg) = -1;
%             seg_dna_pre_final(logical(cellimg .* seg_dna)) = -1;
        end
    end
end


seg_dna_final = seg_dna_pre_final;
% %grj, added this extra segmentation step to get better nuclei. 1/18/15
% 
% u_regions = unique(seg_cell_final(:));
% %remove background and bad nuclei
% u_regions(ismember(u_regions, [-1, 0])) = [];
% 
% 
% seg_dna_final = zeros(size(seg_cell));
% 
% im_dapi_blur = imfilter(im_dapi, fspecial('gaussian', 30, 4));
% 
% for i = 1:length(u_regions)
%     cell_region = (seg_dna_pre_final == u_regions(i)).*im_dapi_blur;
%     
%     dna_region = (cell_region > ml_rcthreshold(cell_region));
%     dna_region = imfill(dna_region, 'holes');
%     seg_dna_final(dna_region) = u_regions(i);
%     
% end



end

