clearvars

parentdir = '/Users/greg/Documents/Murphylab/projects/cellOrganizer/BBBC021/';

imgdir = [parentdir filesep 'images'];

dataparent = 'https://www.broadinstitute.org/bbbc/BBBC021/';
dataprefix = 'BBBC021_v1_images_';

datafile = 'BBBC021_v1_image.csv';
moafile = 'BBBC021_v1_moa.csv';

datafile_target = [parentdir filesep 'images' filesep datafile];
datafile_source = [dataparent datafile];

if ~exist(datafile_target, 'file')
    urlwrite(datafile_source, datafile_target);
end

moafile_target = [parentdir filesep 'images' filesep moafile];
moafile_source = [dataparent moafile];
if ~exist(moafile_target, 'file')
    urlwrite(moafile_source, moafile_target)
end

imgdata = readtable(datafile_target);
moadata = readtable(moafile_target);

%Pick one of each MOA
[method_of_action, ind, ic] = unique(moadata.moa);

%just pick the compound with the highest concentration
for i = 1:length(method_of_action)
    moainds = find(ic == i);
    [~, maxind] = max(moadata.concentration(moainds));
    
    compounds{i} = moadata.compound{moainds(maxind)};
    concentrations{i} = moadata.concentration(moainds(maxind));
    
end

impath_dapi = imgdata.Image_FileName_DAPI;
impath_actin = imgdata.Image_FileName_Actin;
impath_tub = imgdata.Image_FileName_Tubulin;

filesuffix = imgdata.Image_Metadata_Plate_DAPI;

imgs_dapi = {};
imgs_actin = {};
imgs_tub = {};
imgs_suffix = {};
replicates = [];

for i = 1:length(compounds)
    inds_compound = strcmpi(imgdata.Image_Metadata_Compound, compounds{i});
    inds_concentration = imgdata.Image_Metadata_Concentration == concentrations{i};
    
    inds_comb = find(inds_compound & inds_concentration);
    
    %get the first image from each example
    [ureps, uind] = unique(imgdata.Replicate(inds_comb));
    
    imgs_dapi = [imgs_dapi; impath_dapi(inds_comb(uind))];
    imgs_actin = [imgs_actin; impath_actin(inds_comb(uind))];
    imgs_tub = [imgs_tub; impath_tub(inds_comb(uind))];
    
    imgs_suffix = [imgs_suffix; filesuffix(inds_comb(uind))];    
    replicates = [replicates; ureps];
end

usuffix = unique(imgs_suffix);

for i = 1:length(usuffix)
    targetfile = [imgdir filesep dataprefix usuffix{i} '.zip'];
    sourcefile = [dataparent filesep dataprefix usuffix{i} '.zip'];
    
    unzipdir = [imgdir filesep];
    
    
    
    if ~exist([unzipdir filesep usuffix{i}], 'dir')
        mkdir([unzipdir filesep usuffix{i}]);
        
        if ~exist(targetfile, 'file')
            disp(['Downloading ' sourcefile]);
            urlwrite(sourcefile, targetfile);
        end
        
        imginds = strcmpi(imgs_suffix, usuffix{i});
        keepimgs = [imgs_dapi(imginds), imgs_actin(imginds), imgs_tub(imginds)];

        impaths = cellfun(@(x) [usuffix{i} filesep x ' '], keepimgs, 'UniformOutput', false);
        system(['unzip -u ' targetfile  ' ' [impaths{:}] ' -d ' parentdir filesep 'images' filesep ]);

        delete(targetfile)
    end
    
  
end

clearvars -except imgs_dapi imgs_actin imgs_tub imgs_suffix compounds concentrations method_of_action replicates parentdir imgdir
