function script_classify_results(parentdir)

%      bbbc021setup_sub

    load([parentdir filesep 'cellModel_sub.mat']);

    figdir = [parentdir filesep 'figures'];
    if ~exist(figdir, 'dir')
        mkdir(figdir)
    end
    
    resdir = [parentdir filesep 'classify_results'];
    if ~exist(resdir, 'dir')
        mkdir(resdir)
    end
    
 
    
    script_get_object_feats
    script_get_haralick_feats

    load([parentdir filesep 'model_settings_sub.mat']);
    load([parentdir filesep 'images' filesep 'dat.mat']);
    
    
    conditions = imgs_compound(imnum);
    [~,~,labels] = unique(conditions);
    expnum = replicates(imnum);

    [~,~,cvinds] = unique([labels expnum], 'rows');

    uinds = unique(cvinds);

    keepinds = model.cellShapeModel.keepinds & ~cellfun(@isempty, dnafeats) & ~cellfun(@isempty, cellfeats) & ~cellfun(@isempty, hfeats);

    embed_dat_file = [resdir filesep 'embed_dat.mat'];
    
    if ~exist(embed_dat_file, 'file')
        method = struct('method',  'regress',   'weight_factor', 0, 'desired_shape_space_dimensionality', 7);
        [pos_partial, ~, ~] = embed_partial_distance_matrix(model.cellShapeModel.distances_incomplete, method);
        pos_partial = pos_partial(keepinds,:);
        
        


        method = struct('method',  'regress',   'weight_factor', 0, 'desired_shape_space_dimensionality', inf);
        [pos_full, ~, ~] = embed_partial_distance_matrix(model.cellShapeModel.distances_incomplete, method);
        pos_full = pos_full(keepinds,:);

        save(embed_dat_file, 'pos_partial', 'pos_full')
    else
        load(embed_dat_file)
    end
    
    method = struct('method',  'regress',   'weight_factor', 0, 'desired_shape_space_dimensionality', 30);
%         [pos_partial, ~, ~] = embed_partial_distance_matrix(model.cellShapeModel.distances_incomplete, method);
%         pos_partial = pos_partial(keepinds,:);
    
    pos_full = pos_full(:,1:end-1);
    
    pos_partial = zscore(pos_partial);
    pos_full = zscore(pos_full);
    
    hfeats = zscore(vertcat(hfeats{keepinds}));
    objfeats = zscore([vertcat(cellfeats{keepinds}), vertcat(dnafeats{keepinds})]);
    
    
    a = model.cellShapeModel.distances_incomplete(keepinds,keepinds);
    lddmmfeats_dists = a(:,~any(isnan(a),1));
    lddmmfeats_dists_complete = squareform(pdist(pos_partial));
    
    
    labels = labels(keepinds);
    cvinds = cvinds(keepinds);
    
    feats = {pos_partial, pos_full, lddmmfeats_dists, lddmmfeats_dists_complete, ...
        objfeats, hfeats, ...
        [hfeats, pos_partial], ...
        [hfeats, pos_full], ...
        [hfeats, lddmmfeats_dists], ...
        [hfeats, lddmmfeats_dists_complete], ...
        [hfeats, objfeats]};
    
    featnames = {'lddmm-embed', 'lddmm-embed full', 'lddmm-dists', 'lddmm-dists complete', ...
        'Objfeats', 'Haralick', ...
        'Haralick-lddmm-embed', ...
        'Haralick-lddmm-embed full', ...
        'haralick-lddmm-dists', ...
        'haralick-lddmm-dists complete', ...
        'Haralick-Obj'};
    
    classifiers = {@knn, @lda}; %@rf;
    
    classifiernames = {'knn','lda'};
    
    confmat = cell(length(feats), length(classifiers));
    

    for j = 1:length(classifiers)
        for i = 1:length(feats)

            method_name{i,j} = [classifiernames{j} ' ' featnames{i}];

            savefile = [resdir filesep method_name{i,j} '.mat'];

            if ~exist(savefile, 'file')

                confmat = crossVal(feats{i}, labels, cvinds, classifiers{j});
                percell_acc = get_average_per_cell_accuracy(confmat);
                perimage_acc = get_average_per_image_accuracy(confmat);

                save(savefile, 'confmat', 'percell_acc', 'perimage_acc')
            else
                load(savefile)
            end

            confmat_all{i,j} = confmat;
            percell_acc_all{i,j} = percell_acc;
            perimage_acc_all{i,j} = perimage_acc;

        end
    end

    confmat = confmat_all;
    percell_acc = percell_acc_all;
    perimage_acc = perimage_acc_all;
        
    
    figure('color', 'w')
    bar([percell_acc{:}; perimage_acc{:}])
        
    set(gca, 'XTickLabel', {'percell', 'perimage'})
    ylabel('Average class accuracy')
    
    a = xlim;
    
    hold on
    plot(a, [1/length(unique(labels)), 1/length(unique(labels))], 'k--')
    
    legend(gca, {method_name{:}, 'Random'}, 'Location', 'NorthWest')
    
    title('Hold-Out-Replicate Average Class Accuracy')
    
    saveas(gcf, [shapespacedir filesep 'figures' filesep 'fig4_b.png'], 'png')
    
end

function acc = get_average_per_image_accuracy(confmat)
    [~,maxinds] = max(confmat,[],2);
    maxinds = maxinds.*any(confmat,2);
    per_image_confmat = zeros(size(confmat,1), size(confmat,1));
    for i = 1:size(maxinds,3)
        class = find(maxinds(:,:,i));
        per_image_confmat(class, maxinds(class,:,i)) = per_image_confmat(class, maxinds(class,:,i))+1;
    end
    
    acc = mean(diag(sum(per_image_confmat,3))./ sum(sum(per_image_confmat,3),2));
end

function acc = get_average_per_cell_accuracy(confmat)
    acc = mean(diag(sum(confmat,3))./ sum(sum(confmat,3),2));
end