% Generates Table 1 and Figure 5a
% from G. R. Johnson, T. E. Buck,D. P. Sullivan, G. K. Rohde, and 
% R. F. Murphy (2015) Joint modeling of cell and nuclear shape variation. 
% Molecular Biology of the Cell,
% http://murphylab.web.cmu.edu/software/2015_MBoC_Cell_And_Nuclear_Shape/index.html
%
% Author: Gregory R. Johnson (gj@cs.cmu.edu)
% Created: July 7, 2015
% Nov 2, 2015 - added header block - gj@cs.cmu.edu


system('hostname');

bbbc021setup_sub

saveparent = [resultsdir filesep mfilename];

embedding_cell_model = [saveparent filesep 'cellModel_cell.mat'];
embedding_nuc_model = [saveparent filesep 'cellModel_nuc.mat'];
distances_cell_file = [saveparent filesep 'positions_cell.mat'];
distances_nuc_file = [saveparent filesep 'positions_nuc.mat'];

if ~exist('param', 'var')
    param = [];
end

param = ml_initparam(param, struct('build', false));


embedding_options.embedding.method = 'regress landmark';
embedding_options.niter = 30;

embedding_dir = [saveparent filesep 'embeddings'];

if ~exist(distances_cell_file, 'file')
    load([parentdir filesep 'cellModel_cell.mat'])
    
    embedding_model_cell = compute_best_embedding(model.cellShapeModel.distances_incomplete, [embedding_dir filesep 'cell'], embedding_options);
    
    positions_cell = embedding_model_cell.positions;
    model.cellShapeModel = ml_initparam(embedding_model_cell, model.cellShapeModel);
    model.nuclearShapeModel = model.cellShapeModel;
   
    
    if ~all(isnan(positions_cell(:)))
        save(embedding_cell_model, 'model')
         save(distances_cell_file, 'positions_cell')
    end
%     positions_cell = model.cellShapeModel.positions;
%     clear model
%     
%     save(distances_cell_file, 'positions_cell')
else
    load(distances_cell_file)
end

if ~exist(distances_nuc_file, 'file')
    load([parentdir filesep 'cellModel_nuc.mat'])

    embedding_model_nuc = compute_best_embedding(model.cellShapeModel.distances_incomplete, [embedding_dir filesep 'nuc'], embedding_options);
    
    positions_nuc = embedding_model_nuc.positions;
    model.cellShapeModel = ml_initparam(embedding_model_nuc, model.cellShapeModel);
    model.nuclearShapeModel = model.cellShapeModel;
    
    if ~all(isnan(positions_nuc(:)))
        save(embedding_nuc_model, 'model')
        save(distances_nuc_file, 'positions_nuc')
    end
%     positions_nuc = model.cellShapeModel.positions;
%     clear model
%     
%     save(distances_nuc_file, 'positions_nuc')
else
    load(distances_nuc_file)
end

if isnan(positions_cell) | isnan(positions_nuc)
    return
end


niter = 250;

naninds = any(isnan(positions_cell),2) | any(isnan(positions_cell),2);

positions_cell = positions_cell(~naninds,:);
positions_nuc = positions_nuc(~naninds,:);
conditions = condition(~naninds);


cvinds = 1:size(positions_cell,1);

param.split_on_sig = false;

alldatfile = [saveparent filesep 'alldat.mat'];

rng('shuffle')
perm = randperm(length(compounds));
% perm = 1:length(compounds);

nuc2cell = cell(size(compounds));
nuc2cell_pvals = zeros(size(conditions));
cell2nuc = cell(size(compounds));
cell2nuc_pvals = zeros(size(conditions));
% cell2nuc_pvals = cell(length(compounds));

for i = 1:length(compounds)
    counts(i) = sum(ismember(conditions, compounds(i)));
end



if ~exist(alldatfile, 'file') && ~param.build
    for i = 1:length(perm) 
        disp([num2str(perm(i)) filesep num2str(length(perm))]);
        
        
        compound = compounds(perm(i));
        inds = ismember(conditions, compound);
        
        pos_cell = positions_cell(inds,:);
        pos_nuc = positions_nuc(inds,:);
        
        cvinds = 1:sum(inds);
        
        param.method = 'gauss';

        savedir = [saveparent filesep param.method filesep 'd' num2str(perm(i))];
        param.name = 'nuc2cell';
        savedir_iter = [savedir filesep param.name];
        param.normfunc = @(x) x;
        nuc2cell{perm(i)} = kernel_pred_iter(pos_nuc, pos_cell, cvinds, niter, savedir_iter, param);

        nuc2cell_pvals(inds) = nuc2cell{perm(i)}.y_pval;

        param.name = 'cell2nuc';
        savedir_iter = [savedir filesep param.name];
        param.normfunc = @(x) x;
        cell2nuc{perm(i)} = kernel_pred_iter(pos_cell, pos_nuc, cvinds, niter, savedir_iter, param);

        cell2nuc_pvals(inds) = cell2nuc{perm(i)}.y_pval;
        
    end
    save(alldatfile, 'nuc2cell', 'cell2nuc', 'nuc2cell_pvals', 'cell2nuc_pvals')
else
    load(alldatfile)
end

condition = cellfun(@(x,y) [x ' - ' y], compounds, method_of_action', 'UniformOutput', false);
% 
pred_dat_pval = save_pred_dat(cell2nuc, nuc2cell, cell2nuc_pvals, nuc2cell_pvals, condition, niter, [], saveparent);
% 
% 
% %%%% synthesize each predicted shape
% 

nuc2cell_dist_all = nan(size(positions_cell,1), 1);
cell2nuc_dist_all = nan(size(positions_cell,1), 1);

cell2nuc_dist = cell2nuc;
nuc2cell_dist = nuc2cell;

for i = 1:length(compounds)
    inds = ismember(conditions, compounds(i));

    [cell2nuc_dist_pval, cell2nuc_pred_norm_err] = pred_pval(positions_nuc(inds,:), cell2nuc{i}.y_pred);
    [nuc2cell_dist_pval, nuc2cell_pred_norm_err] = pred_pval(positions_cell(inds,:), nuc2cell{i}.y_pred);
    
    [cell2nuc_dist_pval_model, cell2nuc_model_err, cell2nuc_model_err_cb] = pred_pval_all(positions_nuc(inds,:), cell2nuc{i}.y_pred);
    [nuc2cell_dist_pval_model, nuc2cell_model_err, nuc2cell_model_err_cb] = pred_pval_all(positions_cell(inds,:), nuc2cell{i}.y_pred);
 
    cell2nuc_dist_all(inds) = cell2nuc_pred_norm_err;
    nuc2cell_dist_all(inds) = nuc2cell_pred_norm_err;
    
    cell2nuc_dist{i}.y_pval = cell2nuc_dist_pval;
    cell2nuc_dist{i}.y_norm_err = cell2nuc_pred_norm_err;
    cell2nuc_dist{i}.y_model_pval = cell2nuc_dist_pval_model;
    cell2nuc_dist{i}.y_model_err = cell2nuc_model_err;
    cell2nuc_dist{i}.y_model_err_cb = cell2nuc_model_err_cb;

    nuc2cell_dist{i}.y_pval = nuc2cell_dist_pval;
    nuc2cell_dist{i}.y_norm_err = nuc2cell_pred_norm_err;
    nuc2cell_dist{i}.y_model_pval = nuc2cell_dist_pval_model;
    nuc2cell_dist{i}.y_model_err = nuc2cell_model_err;
    nuc2cell_dist{i}.y_model_err_cb = nuc2cell_model_err_cb;
    
end
pred_dat_pval2 = save_pred_dat2(cell2nuc_dist, nuc2cell_dist, cell2nuc_dist_all, nuc2cell_dist_all, condition, [], [saveparent filesep 'pval_dist'], 'pval_dist');




cellmodel = load(embedding_cell_model);
nucmodel = load(embedding_nuc_model);

nuc2cell_index = nuc2cell;
cell2nuc_index = cell2nuc;

nuc2cell_pvals_jaccard = nan(size(positions_cell,1), 1);
cell2nuc_pvals_jaccard = nan(size(positions_cell,1), 1);

keepinds = find(~naninds);

for i = 1:length(compounds)
    inds = find(ismember(conditions, compounds(i)));
    
    shape_inds = keepinds(inds);
        
    [nuc2cell_framefiles_pred{i}, ~ ] = shape_generator_pred_struct(cellmodel.model, nuc2cell{i}, [saveparent filesep 'synth_nuc2cell' filesep 'cond' num2str(i)], param);
    [cell2nuc_framefiles_pred{i}, ~ ] = shape_generator_pred_struct(nucmodel.model, cell2nuc{i}, [saveparent filesep 'synth_cell2nuc' filesep 'cond' num2str(i)], param);
    
    index_save_file = [saveparent filesep 'index_save_file' num2str(i) '.mat'];
    
    if ~exist(index_save_file, 'file') 
        [nuc2cell_index_dist, index_pred_cell, index_dists_cell] = index_sig2(cellmodel.model, shape_inds, nuc2cell_framefiles_pred{i}, @jaccard, [saveparent filesep 'index_save_cell' filesep 'cond' num2str(i)]);
        [cell2nuc_index_dist, index_pred_nuc, index_dists_nuc] = index_sig2(nucmodel.model, shape_inds, cell2nuc_framefiles_pred{i}, @jaccard, [saveparent filesep 'index_save_nuc' filesep 'cond' num2str(i)]);
        
        if ~any(isnan(index_dists_cell(:))) && ~any(isnan(index_dists_nuc(:)))
            save(index_save_file, 'nuc2cell_index_dist', 'index_pred_cell', 'index_dists_cell', ...
                                'cell2nuc_index_dist', 'index_pred_nuc', 'index_dists_nuc')
        else
            continue
        end
        
    else
        load(index_save_file)
    end
     
    index_dists_cell = 1 - index_dists_cell;
    index_pred_cell = 1 - index_pred_cell;

    index_dists_nuc = 1 - index_dists_nuc;
    index_pred_nuc = 1 - index_pred_nuc;

    [nuc2cell_jaccard_pval, nuc2cell_jaccard_norm_err] = pval_dist(index_pred_cell, index_dists_cell );
    [cell2nuc_jaccard_pval, cell2nuc_jaccard_norm_err] = pval_dist(index_pred_nuc, index_dists_nuc);

    [nuc2cell_jaccard_pval_model, nuc2cell_jaccard_model_err, nuc2cell_jaccard_model_err_cb] = pval_dist_all(index_pred_cell, index_dists_cell );
    [cell2nuc_jaccard_pval_model, cell2nuc_jaccard_model_err, cell2nuc_jaccard_model_err_cb] = pval_dist_all(index_pred_nuc, index_dists_nuc );

    nuc2cell_pvals_jaccard(inds) = nuc2cell_jaccard_pval;
    cell2nuc_pvals_jaccard(inds) = cell2nuc_jaccard_pval;
 
    
    nuc2cell_index{i}.y_pval = nuc2cell_jaccard_pval;
    nuc2cell_index{i}.y_pred_err = index_pred_cell;
    nuc2cell_index{i}.y_dists = index_dists_cell;

    nuc2cell_index{i}.y_norm_err = nuc2cell_jaccard_norm_err;
    nuc2cell_index{i}.y_model_pval = nuc2cell_jaccard_pval_model;
    nuc2cell_index{i}.y_model_err = nuc2cell_jaccard_model_err;
    nuc2cell_index{i}.y_model_err_cb = nuc2cell_jaccard_model_err_cb;


    cell2nuc_index{i}.y_pval = cell2nuc_jaccard_pval;    
    cell2nuc_index{i}.y_pred_err = index_pred_nuc;
    cell2nuc_index{i}.y_dists = index_dists_nuc;

    cell2nuc_index{i}.y_norm_err = cell2nuc_jaccard_norm_err;
    cell2nuc_index{i}.y_model_pval = cell2nuc_jaccard_pval_model;
    cell2nuc_index{i}.y_model_err = cell2nuc_jaccard_model_err;
    cell2nuc_index{i}.y_model_err_cb = cell2nuc_jaccard_model_err_cb;
    
    
end
pred_dat_jaccard = save_pred_dat2(cell2nuc_index, nuc2cell_index, cell2nuc_pvals_jaccard, nuc2cell_pvals_jaccard, condition, [], [saveparent filesep 'pval-jaccard'], 'jaccard');




nuc_pred_pos = nan(size(positions_cell));
cell_pred_pos = nan(size(positions_nuc));

for i = 1:length(compounds)
    compound = compounds(i);
    inds = ismember(conditions, compound);

    cell_pred_pos(inds,:) = nuc2cell_dist{i}.y_pred;
    nuc_pred_pos(inds,:) = cell2nuc_dist{i}.y_pred;
end


synth_img_dir = [saveparent filesep 'synth_imgs'];

if ~exist(synth_img_dir, 'dir')
    mkdir(synth_img_dir)
end

compound_ind = ismember(compounds, 'DMSO');
condition_inds = find(ismember(conditions, 'DMSO'));

pmat = [nuc2cell_dist_all];
pmat = pmat(condition_inds,:);
dists = pdist2(0, pmat);
[~,min_inds] = min(dists,[],2);
min_inds = find(dists(min_inds) == dists);
shape_inds = condition_inds(min_inds);
pred_img_saver(cellmodel, nucmodel, nuc2cell_framefiles_pred{compound_ind}, cell2nuc_framefiles_pred{compound_ind}, keepinds(shape_inds), min_inds, 'synth_dist_best_cell', synth_img_dir )

pmat = [nuc2cell_dist_all];
pmat = pmat(condition_inds,:);
dists = pdist2(max(pmat), pmat);
[~,min_inds] = min(dists,[],2);
min_inds = find(dists(min_inds) == dists);
shape_inds = condition_inds(min_inds);
pred_img_saver(cellmodel, nucmodel, nuc2cell_framefiles_pred{compound_ind}, cell2nuc_framefiles_pred{compound_ind}, keepinds(shape_inds), min_inds, 'synth_dist_worst_cell', synth_img_dir )


pmat = [cell2nuc_dist_all];
pmat = pmat(condition_inds,:);
dists = pdist2(0, pmat);
[~,min_inds] = min(dists,[],2);
min_inds = find(dists(min_inds) == dists);
shape_inds = condition_inds(min_inds);
pred_img_saver(cellmodel, nucmodel, nuc2cell_framefiles_pred{compound_ind}, cell2nuc_framefiles_pred{compound_ind}, keepinds(shape_inds), min_inds, 'synth_dist_best_nuc', synth_img_dir )

pmat = [cell2nuc_dist_all];
pmat = pmat(condition_inds,:);
dists = pdist2(max(pmat), pmat);
[~,min_inds] = min(dists,[],2);
min_inds = find(dists(min_inds) == dists);
shape_inds = condition_inds(min_inds);
pred_img_saver(cellmodel, nucmodel, nuc2cell_framefiles_pred{compound_ind}, cell2nuc_framefiles_pred{compound_ind}, keepinds(shape_inds), min_inds, 'synth_dist_worst_nuc', synth_img_dir )



pmat = [cell2nuc_dist_all, nuc2cell_dist_all];
pmat = pmat(condition_inds,:);
dists = pdist2([0,0; 0,1;1,0;1,1], pmat);
[~,min_inds] = min(dists,[],2);
shape_inds = condition_inds(min_inds);

pred_img_saver(cellmodel, nucmodel, nuc2cell_framefiles_pred{compound_ind}, cell2nuc_framefiles_pred{compound_ind}, keepinds(shape_inds), min_inds, 'synth_dist', synth_img_dir )


pmat = [cell2nuc_dist_all, nuc2cell_dist_all, cell2nuc_pvals_jaccard, nuc2cell_pvals_jaccard];
pmat = pmat(condition_inds,:);
dists = pdist2([0,0,0,0; 0,1,0,1; 1,0,1,0; 1,1,1,1], pmat);
[~,min_inds] = min(dists,[],2);
shape_inds = condition_inds(min_inds);

pred_img_saver(cellmodel, nucmodel, nuc2cell_framefiles_pred{compound_ind}, cell2nuc_framefiles_pred{compound_ind}, keepinds(shape_inds), min_inds, 'synth_dist-jaccard', synth_img_dir )

    
pmat = [cell2nuc_pvals_jaccard, nuc2cell_pvals_jaccard];
pmat = pmat(condition_inds,:);
dists = pdist2([0,0; 0,1;1,0;1,1], pmat);
[~,min_inds] = min(dists,[],2);
shape_inds = condition_inds(min_inds);

pred_img_saver(cellmodel, nucmodel, nuc2cell_framefiles_pred{compound_ind}, cell2nuc_framefiles_pred{compound_ind}, keepinds(shape_inds), min_inds, 'synth_jaccard', synth_img_dir )




% pred_img_dir = [saveparent filesep 'synth_imgs2'];
%
% if ~param.build
%     for i = 1:length(shape_inds)
%         
%         synthfile = [ pred_img_dir filesep 'shape' num2str(i) '.mat'];
%         synthfile_temp = [synthfile '.tmp'];
% 
%         if ~exist(synthfile, 'file') && ~exist(synthfile_temp, 'file')
%             system(['touch ' synthfile_temp]);
%             synthparam.position = cell_pred_pos(shape_inds(i),:); 
%             synthparam.tempdir = [pred_img_dir filesep 'temp' filesep 'cell' num2str(i)];
%             if ~exist(synthparam.tempdir, 'dir')
%                 mkdir(synthparam.tempdir)
%             end
%             cd(synthparam.tempdir)
% 
%             cell_pred = model2img({cellmodel.model}, synthparam);
% 
%             synthparam.position = nuc_pred_pos(shape_inds(i),:);
%             synthparam.tempdir = [pred_img_dir filesep 'temp' filesep 'nuc' num2str(i)];
%             if ~exist(synthparam.tempdir, 'dir')
%                 mkdir(synthparam.tempdir)
%             end
%             cd(synthparam.tempdir)
% 
% 
%             nuc_pred = model2img({nucmodel.model}, synthparam);
% 
%             save(synthfile, 'cell_pred', 'nuc_pred')
% 
%             true_ind = find(all(cellmodel.model.cellShapeModel.positions == repmat(positions_cell(shape_inds(i),:), [cellmodel.model.cellShapeModel.numimgs,1]),2));
%             cell_true = cellmodel.model.cellShapeModel.imfunc(true_ind);
%             cell_pred = cell_pred{1};
% 
%             cell_img = [cell_true.*255, cell_pred(:,:,2)];
%             imwrite(cell_img, [pred_img_dir filesep 'cell' num2str(i) '.png'], 'png')
% 
% 
%             nuc_true = nucmodel.model.cellShapeModel.imfunc(true_ind);
%             nuc_pred = nuc_pred{1};
%             nuc_img = [nuc_true.*255, nuc_pred(:,:,2)];
%             
%             imwrite(nuc_img, [pred_img_dir filesep 'nuc' num2str(i) '.png'], 'png')
% 
%             system(['rm ' synthfile_temp])
%         end
%     end
% end

% 
% 
% pred_img_dir = [saveparent filesep 'synth_imgs'];

% if ~exist(pred_img_dir, 'dir')
%     mkdir(pred_img_dir)
% end
% 
% if ~param.build
% for i = 1:length(compounds)
%     compound = compounds(perm(i));
%     cellnums = find(ismember(conditions, compound));
%         
%     disp([num2str(i) filesep num2str(length(compounds))]);
%     
%     for j = 1:length(cellnums)
%         
%         synthfile = [ pred_img_dir filesep 'shape' num2str(cellnums(j)) '.mat'];
%         
%         synthfile_temp = [synthfile '.tmp'];
%         if ~exist(synthfile, 'file') && ~exist(synthfile_temp, 'file')
%             system(['touch ' synthfile_temp]);
% 
%             disp(['pred:' num2str(j) filesep num2str(length(cellnums))])
%             
%             synthparam.position = nuc2cell{i}.y_pred(j,:); 
%             synthparam.tempdir = [pred_img_dir filesep 'temp' filesep 'cell' num2str(cellnums(j))];
%             
% 
%             cell_pred = model2img({cellmodel.model}, synthparam);
% 
%             synthparam.position = cell2nuc{i}.y_pred(j,:);
%             synthparam.tempdir = [pred_img_dir filesep 'temp' filesep 'nuc' num2str(cellnums(j))];
% 
%             nuc_pred = model2img({nucmodel.model}, synthparam);
%       
%             save(synthfile, 'cell_pred', 'nuc_pred')
%             
%             system(['rm ' synthfile_temp])
%         end
%     end
% end
% end

% 
%  for i = 1:length(compounds)
%      
%      compound = compounds(i);
%     
%      
%     disp(['Printing images for '  compound{1}])
% 
%     savedir = [pred_img_dir filesep compounds{i}];
%     if ~exist(savedir, 'dir')
%         mkdir(savedir)
%     end
% 
%     sigdir = [savedir filesep 'sig'];
%     notsigdir = [savedir filesep 'notsig'];
% 
% 
%     inds = find(ismember(conditions, compound));
% 
%     c1 = 1;
%     c2 = 1;
%     n1 = 1;
%     n2 = 1;
%     
%     cell_sig_imgs = {};
%     cell_notsig_imgs = {};
%     
%     nuc_sig_imgs = {};
%     nuc_notsig_imgs = {};
%     
%     
%     for j = 1:length(inds)
%         synthfile = [ pred_img_dir filesep 'shape' num2str(inds(j)) '.mat'];
%         if exist(synthfile, 'file')
%             load(synthfile)
%             
%             model_ind = find(all(cellmodel.model.cellShapeModel.positions == repmat(positions_cell(inds(j),:), [cellmodel.model.cellShapeModel.numimgs,1]),2));
%             
%             cell_true = cellmodel.model.cellShapeModel.imfunc(model_ind);
%             if ~isempty(cell_pred) & ~isempty(cell_true)
% 
%                 
%                 cell_pred = cell_pred{1};
% 
%                 cell_img = [cell_true.*255, cell_pred(:,:,2)];
% 
%                 if nuc2cell_pvals(inds(j)) < 0.05
%                     suffix = '_sig';
%                     
%                     cell_sig_imgs{c1} = cell_img';
%                     c1 = c1+1;
%                 else
%                     suffix = '_not_sig';
%                     cell_notsig_imgs{c2} = cell_img';
%                     c2 = c2+1;
%                 end
% 
%                 save_cell_img = [savedir filesep 'cell' num2str(inds(j)) suffix '.png'];
%                 imwrite(cell_img, save_cell_img, 'png');
%             end
% 
%             model_ind = find(all(nucmodel.model.cellShapeModel.positions == repmat(positions_nuc(inds(j),:), [nucmodel.model.cellShapeModel.numimgs,1]),2));
%             
%             nuc_true = nucmodel.model.cellShapeModel.imfunc(model_ind);
%             
%             if ~isempty(nuc_pred) & ~isempty(nuc_true)
% 
%                 
%                 
%                 if isempty(nuc_true)
%                     1;
%                 end
%                 
%                 nuc_pred = nuc_pred{1};
% 
%                 nuc_img = [nuc_true.*255, nuc_pred(:,:,2)];
% 
%                 if cell2nuc_pvals(inds(j)) < 0.05
%                     suffix = '_sig';
%                     nuc_sig_imgs{n1} = nuc_img';
%                     n1 = n1+1;
%                 else
%                     suffix = '_not_sig';
%                     nuc_notsig_imgs{n2} = nuc_img';
%                     n2 = n2+1;
%                 end
% 
%                 save_nuc_img = [savedir filesep 'nuc' num2str(inds(j)) suffix '.png'];
%                 imwrite(nuc_img, save_nuc_img, 'png');
%             end 
%         end    
%     end
%     
%     if ~isempty(cell_sig_imgs)
%         imwrite([cell_sig_imgs{:}], [savedir filesep 'all_cell_sig.png'], 'png')
%     end
%     if ~isempty(cell_notsig_imgs)
%         imwrite([cell_notsig_imgs{:}], [savedir filesep 'all_cell_notsig.png'], 'png')
%     end
%     
%     if ~isempty(nuc_sig_imgs)
%         imwrite([nuc_sig_imgs{:}], [savedir filesep 'all_cnuc_sig.png'], 'png')
%     end
%     
%     if ~isempty(nuc_notsig_imgs)
%         imwrite([nuc_notsig_imgs{:}], [savedir filesep 'all_nuc_notsig.png'], 'png')
%     end
%  end
% 
% 
% 
