function [ seg_dna_final, seg_cell_final ] = segCells(im_dapi, im_actin, im_tub)
%cell segmentation code specificly written to segment the images in the
%dataset from https://www.broadinstitute.org/bbbc/BBBC021/
%
% Gregory R. Johnson, 5/15/14

%SEGCELLS Summary of this function goes here
%   Detailed explanation goes here
% if ~exist('im_dapi', 'var')
%     im_dapi = imread('/Users/greg/Documents/Murphylab/projects/cellOrganizer/BBBC021/images/Week1_22123/Week1_150607_B03_s1_w114129FBE-AB04-4F79-B8AF-CFA7D5524D53.tif');
%     im_actin = imread('/Users/greg/Documents/Murphylab/projects/cellOrganizer/BBBC021/images/Week1_22123/Week1_150607_B03_s1_w41094DA98-6064-4EA2-B4A7-02A38208692F.tif');
%     im_tub = imread('/Users/greg/Documents/Murphylab/projects/cellOrganizer/BBBC021/images/Week1_22123/Week1_150607_B03_s1_w114129FBE-AB04-4F79-B8AF-CFA7D5524D53.tif');
% end

im_dapi = mat2gray(double(im_dapi));
im_prot = mat2gray(mat2gray(double(im_dapi)) + mat2gray(double(im_actin)) + mat2gray(double(im_tub)));



% init_seg_regions = [40, 40];
% 
% i_sp = linspace(1, size(im_dapi,1), init_seg_regions(1));
% j_sp = linspace(1, size(im_dapi,2), init_seg_regions(2));
% 
% [x,y] = meshgrid(i_sp, j_sp);
% 
% x = round(x);
% y = round(y);
% 
% im_regions = zeros(size(im_dapi));
% im_regions(x(:), y(:)) = 1;
% 
% im_region_dist = bwdist(im_regions);
% 
% dist_cutoff = min(i_sp(2) - i_sp(1), i_sp(2) - i_sp(1)) / 3;

%use the vessleness filter to find edges between nuclei
[dxx, dxy, dyy] = Hessian2D(double(im_dapi),6);
[lambda1,lambda2,~,~]=eig2image(dxx,dxy,dyy);
eigim = max(cat(4, lambda1, lambda2),[],4);

%segment all dna regions using the active contour
% seg_dna = region_seg(im_dapi, im_region_dist < dist_cutoff, 3000, 0.05, 0,  1E-4);

% 
% %sometimes the active contour is reversed, so assume that the segmented
% %nuclei takes less area than the image background
% if mean(im_dapi(~seg_dna(:))) > mean(im_dapi(seg_dna(:)))
%     seg_dna = ~seg_dna;
% end
% 
% seg_dna = imfill(seg_dna, 'holes');

im2 = imfilter(im_dapi, fspecial('gaussian', 10,1));

% im2 = mat2gray(im2);

% background = imopen(im2, strel('disk', 50));
% 
% im3 = imadjust(im2 - background);

im4 = im2 > 0.1;
% im4 = im2bw(im3, graythresh(im3));
seg_dna = imfill(im4, 'holes');


%get dna watershed seeds by removing the boundaries between nuclei from the
%segmented dna image
seg_dna_seeds = double((eigim < ml_rcthreshold(eigim)).* seg_dna);

%find the dna boundaries
seg_dna = (double(watershed(imimposemin(-im_dapi, seg_dna_seeds))).*seg_dna) > 0;

%find the watershed cell boundaries from the segmented dna
regions = watershed(imimposemin(-im_prot, seg_dna));

%find the cell boundaries with active contour using an entropy filter
% seg_cell = region_seg(entropyfilt(im_actin + im_tub, ones(3,3)), im_region_dist < dist_cutoff, 3000, 0.05, 0, 1E-5);
% seg_cell = adapthisteq(imfilter(im_prot, fspecial('gaussian', 10, 1)))> 4000;

seg_cell = imfilter(im_prot, fspecial('gaussian', 10, 1)) > 0.10;

% %sometimes the active contour is reversed, so assume that the segmented
% %nuclei intersecting with the segmented cells should be maximized
% if mean(im_prot(~seg_cell(:))) > mean(im_prot(seg_cell(:)))
%     seg_cell = ~seg_cell;
% end


% seg_cell = imfill(seg_cell, 'holes');


%label the cell boundaries
seg_cell = double(regions) .* double(seg_cell);

%get the number of cell regions
u_regions = unique(seg_cell(:));
u_regions(u_regions == 0) = [];

%get an image-boundary image
im_bounds = ones(size(seg_cell));
im_bounds(2:end-1,2:end-1) = 0;

seg_cell_final = zeros(size(seg_cell));
seg_dna_final = zeros(size(seg_cell));

%for every cell region
for i = 1:length(u_regions)
    %get the largest object cell region. This is the segmented cell
    c = ml_findmainobj(seg_cell == u_regions(i));
        
    %only add the image to the final segmentation if
    if sum(c(:) .* im_bounds(:)) == 0
        cellimg = logical(imfill(c, 'holes'));
        
        if sum(seg_dna(:).*cellimg(:)) > 0
            seg_cell_final(cellimg) = u_regions(i);
            seg_dna_final(logical(cellimg .* seg_dna)) = u_regions(i);
        end
    end
    
end

end

function im = im_adj(im)

    im2 = imfilter(im, fspecial('gaussian', 10,1));

    im2 = mat2gray(im2);

    background = imopen(im2, strel('disk', 50));

    im = imadjust(im2 - background);
end
    

