function SSPaper_figure2
% This function creates the second figure in Buck et al. 
% The output is two images 
% The first is a 2D projection of the shape space with ray-traced 3D cells
% and nuclei
% The second is a tetrahedron from the shape space interpolation process. 
% These images are shown and saved as .tiff images. 
%
%Created by: Devin Sullivan 5/15/14
%Updated by: D. Sullivan 8/12/14

%move to correct directory so relative file paths work
file = 'SSPaper_figure2.m';
currpath = which(file);
currfolder = currpath(1:end-length(file));
cd(currfolder)

%%%PART A%%%
%First just show the shape space where all the cells are shown as
%ray-traces
%load the diffeomorphic HeLa model
% modelpath = '../shapeSpaces/HeLa300.mat';
modelpath = './cellModel.mat';
load(modelpath);
labels = [];%1:n; - this is default in showShapeSpace
skipmissing = 1;% this is default in showShapeSpace
proj_orig = [];
cm = @hsv;
traces = [];
param.mesh = 1;
param.ndims = 2;
param.grisize = 2000;

% [img, proj_orig, eig] = showShapeSpace(model, labels, skipmissing, proj_orig, cm, traces,param);
%%%END PART A%%%

%%%PART B%%%
%Pick a spot and synthesize it, then show it.
%This is how the point was selected using the cellorganizer demo
%triplot(model.cellShapeModel.tessellation, model.cellShapeModel.positions(:,1), model.cellShapeModel.positions(:,2))
%scatter(positions(:,1),positions(:,2),40,'.b');
%spot = demo3DDiffeoPick(1)
%
%For consistancy we've fixed this point
%spot = [   0.2460   -0.3385];
%ndims = length(spot);
ndims = 2;
%   spot = [ -0.9556   -0.2566];

% %Next we need to begin interpolation 
% param.position = spot;

%set remaining parameters 
param.targetDirectory = mfilename;
param.showInterp = 1;
% model positions into 2D 
model.cuclearShapeModel.positions = model.cellShapeModel.positions(:,1:ndims);
model.nuclearShapeModel.positions = model.nuclearShapeModel.positions(:,1:ndims);

cellinds = find(all(~isnan(model.cellShapeModel.positions),2));
positions = model.cellShapeModel.positions(:,1:ndims);
tri = delaunay(positions(cellinds,:));
tri = cellinds(tri);

model.cellShapeModel.tessellation = tri; 
model.nuclearShapeModel.tessellation = tri; 
%Pick a spot and synthesize it, then show it.
%This is how the point was selected using the cellorganizer demo
%  triplot(model.cellShapeModel.tessellation, model.cellShapeModel.positions(:,1), model.cellShapeModel.positions(:,2))
%  scatter(positions(:,1),positions(:,2),40,'.b');
%  spot = demo3DDiffeoPick(1)
%spot = [1.0983,0.7133];%old SS
%  spot = [1.6033    0.6859];%old SS
% spot = [1.8594,1.1118];
spot =  [-0.0346, 0.2566];%10/8/14
%%Next we need to begin interpolation 
param.position = spot;

%For consistancy we've fixed this point
%spot = [   0.2460   -0.3385];
ndims = length(spot);
%   spot = [ -0.9556   -0.2566];
options = model.cellShapeModel.shape_space_options;
%options.convergence_absolute_error = 1;
%Should probably compress these. It's confusing to have so many. maybe set
%with param.debug? 
options.save_stages = 1;
options.save_intermediates = 1;
options.save_intermediate_images = 1;
% options.convergence_absolute_error_difference = 1;
model.cellShapeModel.shape_space_options = options;

tstart = tic
imgs = model2img({model}, param);

telapsed = toc(tstart)
%%%END PART B%%%


%%%SUP FIG 2%%%
%generate random walk movie within shape space and show it on the shape
%space
%Here we show the shape space as 2D summations in grey and generated images
%in colors with warmner colors corresponding to later times

%%%END SUP FIG 2%%%