function demo3D25()
% demo3D25
%
% Trains a generative model

% Created: Ivan E. Cao-Berg
%
% Copyright (C) 2014 Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu
    
curr_path = which('demo3D25.m');
curr_path = curr_path(1:end-10);
cd(curr_path);

%icaoberg 27/7/2012
if nargin == 0 | nargin == 1
 if strcmpi( lower(frameworkflag), 'yes' )
    frameworkflag = true;
 else
    frameworkflag = false;
 end
end

if ~exist('param', 'var')
    param = [];
end

if ~isa( frameworkflag, 'logical' )
   warning( 'Input variable framework flag must be boolean. Exiting demo.' );
   return
end

pattern = 'Tub';
directory = [ pwd filesep Tub ];
param.protein.class = 'endosome';
param.protein.name = 'transferrin';
param.nucleus.name = 'transferrin';
param.cell.model = 'transferrin';

dimensionality = '3D';
answer = false;

dna = [directory filesep 'cell*ch0*.tif'] ;
cell = [directory filesep 'cell*ch1*.tif'];
protein = [directory filesep 'cell*ch2*.tif'];
param.masks = [directory filesep 'cell*mask*.tif'];

% param.masks = [directory filesep 'cell*mask*.tif'];

% generic model options
% ---------------------
% model.name                (optional) Holds the name of the model. Default is empty.
param.model.name = 'all';

% model.id                  (optional) Holds the id of the model. Default is empty.
try
   [status,result] = system( 'uuidgen' ); 
   param.model.id = result(1:end-1);
catch
   param.model.id = num2str(now);
end

% model.filename            Holds the output filename.
param.model.filename = [ lower(pattern) '.xml' ];

% nuclear shape model options
% ---------------------------
% nucleus.type              Holds the nuclear model type. Default is "major axis".
param.nucleus.type = 'major axis';

% nucleus.id                (optional) Holds the id of the nuclear model. Default is empty.
try
   [status,result] = system( 'uuidgen' ); 
   param.nucleus.id = result(1:end-1);
catch
   param.nucleus.id = num2str(now);
end

% cell shape model options
% ------------------------
% cell.type                 Holds the cell model type. Default is "ratio".
param.cell.type = 'ratio';

% cell.id                   (optional) Holds the id the cell model. Default is empty.
try
   [status,result] = system( 'uuidgen' ); 
   param.cell.id = result(1:end-1);
catch
   param.cell.id = num2str(now);
end

% protein shape model options
% ---------------------------
% protein.type              (optional) Holds the protein model type. The default is "vesicle".
param.protein.type = 'vesicle';

% protein.id                (optional) Holds the id of the protein model. The default is empty.
try
   [status,result] = system( 'uuidgen' ); 
   param.protein.id = result(1:end-1);
catch
   param.protein.id = num2str(now);
end

% protein.cytonuclearflag   (optional) Determines whether the protein pattern will be generated in
%                           the cytosolic space ('cyto'), nuclear space ('nuc') or everywhere ('all').
%                           Default is cyto.
if strcmpi( param.protein.class, 'nuc' )
   param.protein.cytonuclearflag = 'nuc';
else
   param.protein.cytonuclearflag = 'cyto';
end

% other options
% -------------
% verbose                   (optional) Displays messages to screen. The default is true.
param.verbose = true;

% debug                     (optional) Reports errors and warnings. Default is false.
param.debug = true;

% train.flag                (optional) Selects what model is going to be trained ('nuclear',
%                           'framework', or 'all'). Default is 'all'
param.train.flag = 'all';

%documentation
param.documentation.description = 'This model has been trained using demo3D12 from CellOrganizer';

%model resolution
%this is the resolution of the datasets
param.model.resolution = [0.049, 0.049, 0.2000];

answer = img2slml( dimensionality, dna, cell, protein, param );

if frameworkflag
   disp('Replacing current framework with a framework trained on the four protein patterns in the HeLa dataset');
   temp = load( frameworklocation );
   load([lower(pattern) '.mat']);
   model.nuclearShapeModel = temp.model.nuclearShapeModel;
   model.cellShapeModel = temp.model.cellShapeModel;
   save([lower(pattern) '.mat'], 'model' ); 
end
