function [ output_args ] = demo3DDiffeoLearn_rand_columns()
%This demo illustrates how to change how the distance matrix is completed
%
%this particular example fills columns randomly

%Cd to the directory of this function
scriptname = mfilename;
scriptpath = which(scriptname);
tokens = regexp(scriptpath, filesep);
cd(scriptpath(1:tokens(end)));

param.targetDirectory = scriptname;

pattern = 'LAM';
directory = [ pwd filesep 'LAM' ];
param.protein.class = 'lysosome';
param.protein.name = 'lamp2';
param.nucleus.name = 'lamp2';
param.cell.model = 'lamp2';

tarball = [ '3DHela_' pattern '.tgz' ];
url = 'http://murphylab.web.cmu.edu/data/Hela/3D/multitiff';
   
if ~exist( [pwd filesep tarball], 'file')
    disp('Downloading dataset');
    urlwrite([url filesep tarball], tarball);
    disp('Extracting files');
    untar( tarball );
else
    disp('Pattern found on disk. Extracting files from existing tarball');
    if ~exist( directory, 'dir')
        disp('Extracting files');
        system(['tar -xvf ' tarball] );
    else
        disp('Directory found. Skipping decompression');
    end
end

%reduce the dataset to 15 images
for i = 16:50
    delete([directory filesep 'cell' num2str(i) '_*']);
end


dimensionality = '3D';


dna = [directory filesep 'cell*ch0*.tif'] ;
cell = [directory filesep 'cell*ch1*.tif'];
protein = [];

param.masks = [directory filesep 'cell*mask*.tif'];

% param.masks = [directory filesep 'cell*mask*.tif'];

% generic model options
% ---------------------
% model.name                (optional) Holds the name of the model. Default is empty.
param.model.name = 'all';

% model.id                  (optional) Holds the id of the model. Default is empty.
try
   [status,result] = system( 'uuidgen' ); 
   param.model.id = result(1:end-1);
catch
   param.model.id = num2str(now);
end

% model.filename            Holds the output filename.
param.model.filename = [ lower(pattern) '.xml' ];

% nucleus.id                (optional) Holds the id of the nuclear model. Default is empty.
param.nucleus.type = 'diffeomorphic';
try
   [status,result] = system( 'uuidgen' ); 
   param.nucleus.id = result(1:end-1);
catch
   param.nucleus.id = num2str(now);
end

% cell shape model options
% ------------------------
% cell.type                 Holds the cell model type. Default is "ratio".
param.cell.type = 'diffeomorphic';

% cell.id                   (optional) Holds the id the cell model. Default is empty.
try
   [status,result] = system( 'uuidgen' ); 
   param.cell.id = result(1:end-1);
catch
   param.cell.id = num2str(now);
end

param.verbose = true;
param.debug = true;
param.train.flag = 'framework';

%documentation
param.documentation.description = ['This model has been trained using ' scriptname ' from CellOrganizer'];

param.model.diffeomorphic.embedding_type = 'nystrom-euclidean';
param.model.diffeomorphic.max_columns = 4;

%model resolution
param.model.downsampling = [4.0816*2,4.0816*2,2];

%this is the resolution of the datasets
param.model.resolution = [0.049, 0.049, 0.2000];

model = img2slml( dimensionality, dna, cell, protein, param );

img = model2img({model});


end

