function [docNode,GeowrapperNode,geometryDefNode] = addCSGObjects(CSGdata,docNode,geometryDefNode,GeowrapperNode)

s = 'spatial:';
% listedCompartments = 0;

    %%%Set up the ListOfDomainTypes node
    ListOfDomainTypesNode = docNode.createElement([s,'listOfDomainTypes']);
    %get list of domain types from CSG data list
    domainlist = unique(extractfield(CSGdata.list,'name'));
    for j = 1:length(domainlist)
        DomainTypeNode = docNode.createElement([s,'domainType']);
        DomainTypeNode.setAttribute([s,'id'],domainlist{j});
        DomainTypeNode.setAttribute([s,'spatialDimensions'],'3');
        ListOfDomainTypesNode.appendChild(DomainTypeNode);
    end
    GeowrapperNode.appendChild(ListOfDomainTypesNode);
    %%%Set up ListOfDomains
    ListOfDomains = docNode.createElement([s,'listOfDomains']);
    for j=1:length(CSGdata.list)
        %     if (j~=67&&j~=length(CSGdata.list))
        %         continue
        %     end
        object = CSGdata.list(j);
        name = object.name;
        type = object.type;
        
        DomainNode = docNode.createElement([s,'domain']);
        DomainNode.setAttribute([s,'id'],[name,num2str(j-1)]);%[DomainID,num2str(j-1)])%'0']);
        DomainNode.setAttribute([s,'domainType'],name);
        ListOfInteriorPoints = docNode.createElement([s,'listOfInteriorPoints']);
        InteriorPoint = docNode.createElement([s,'interiorPoint']);
        if isfield(object,'position')
            position = object.position;
        else
            position = [0,0,0];
        end
        InteriorPoint.setAttribute([s,'coord1'],num2str(position(1)));%'0');
        InteriorPoint.setAttribute([s,'coord2'],num2str(position(2)));%'0');
        InteriorPoint.setAttribute([s,'coord3'],num2str(position(3)));%'0');
        ListOfInteriorPoints.appendChild(InteriorPoint);
        DomainNode.appendChild(ListOfInteriorPoints);
        ListOfDomains.appendChild(DomainNode);
    end
    GeowrapperNode.appendChild(ListOfDomains);
    %%%
    
    %%%List of adjacent domains
    ordinals = extractfield(CSGdata.list,'ordinal');
    [ordinal_list,order] = sort(unique(ordinals));
    ListOfAdjacentDomains = docNode.createElement([s,'listOfAdjacentDomains']);
    for j=1:length(ordinal_list)-1
        startobjs = find(ordinals==ordinal_list(j));
        linkobjs = find(ordinals==ordinal_list(j+1));
        %ugh, this is going to be really slow, but I can't think of a
        %better way of coding it right now.
        for k = 1:length(startobjs)
            start_object = CSGdata.list(startobjs(k));
            start_name = start_object.name;
            start_type = start_object.type;
            
            for i = 1:length(linkobjs)
                link_object = CSGdata.list(linkobjs(i));
                link_name = link_object.name;
                link_type = link_object.type;
                AdjacentDomainNode = docNode.createElement([s,'adjacentDomains']);
                AdjacentDomainNode.setAttribute([s,'id'],[start_name,'_',link_name]);%[DomainID,num2str(j-1)])%'0']);
                AdjacentDomainNode.setAttribute([s,'domain1'],start_name);
                AdjacentDomainNode.setAttribute([s,'domain2'],link_name);
                ListOfAdjacentDomains.appendChild(AdjacentDomainNode);
            end
        end
        
    end
    GeowrapperNode.appendChild(ListOfAdjacentDomains);
    %%% 
    
    
    %%%Define all the CSG (Primitive) type geometries
    CSGeometryNode = docNode.createElement([s,'csGeometry']);
    CSGeometryNode.setAttribute([s,'id'], 'CSG_Geometry1');
    % geometryDefNode.appendChild(CSGeometryNode);
    
    ListOfCSGObjectsNode = docNode.createElement([s,'listOfCSGObjects']);
    % ListOfCSGObjectsNode.setAttribute('nameId', CSGdata.name);
    %     ind = length(CSGdata.list)+1;
    %     CSGdata.list(ind).name = ['EC'];
    %     %calculate size required
    %     imgsize =size(Meshdata.list(1).img);
    %     imgsize = [imgsize(2),imgsize(1),imgsize(3)];
    %     ECsize = imgsize.*resolution+[1,1,1];
    %     %calculate position
    %     CSGdata.list(ind).position = [0,0,0]+imgsize./2.*resolution;
    %     CSGdata.list(ind).rotation = [0,0,0];
    %     CSGdata.list(ind).scale = ECsize;
    %     CSGdata.list(ind).ordinal = 0;
    %     CSGdata.list(ind).type = 'cube';
    
    %add each CSGObject
    for j=1:length(CSGdata.list)
        %     if (j~=67&&j~=length(CSGdata.list))
        %         continue
        %     end
        object = CSGdata.list(j);
        name = object.name;
        type = object.type;
        
        
        CSGObjectNode = docNode.createElement([s,'csgObject']);
        %     CSGObjectNode.setAttribute([s,'spatialID'],['Sp_', name]);
        CSGObjectNode.setAttribute([s,'id'],name);
        CSGObjectNode.setAttribute([s,'domainType'],name);
        
        if isfield(object, 'ordinal')
            ordinal = num2str(object.ordinal);
            CSGObjectNode.setAttribute([s,'ordinal'], ordinal);
            % CSGObjectNode.setAttribute([s,'ordinal'], '0');
        end
        
        CSGTransformationNode = docNode.createElement([s,'csgTransformation']);
        
        if isfield(object, 'position')
            position = object.position;
            CSGTranslationNode = docNode.createElement([s,'csgTranslation']);
            CSGTranslationNode.setAttribute([s,'id'],'translation');
            CSGTranslationNode.setAttribute([s,'translateX'], num2str(position(1)));
            CSGTranslationNode.setAttribute([s,'translateY'], num2str(position(2)));
            CSGTranslationNode.setAttribute([s,'translateZ'], num2str(position(3)));
        else
            error('No position specified. Unable to create SBML Spatial file.');
        end
        
        if isfield(object, 'rotation')
            rotation = object.rotation;
            CSGRotationNode = docNode.createElement([s,'csgRotation']);
            CSGRotationNode.setAttribute([s,'id'], 'rotation');
            %         CSGRotationNode.setAttribute([s,'rotateAxisX'], num2str(rotation(1)));
            %         CSGRotationNode.setAttribute([s,'rotateAxisY'], num2str(rotation(2)));
            %         CSGRotationNode.setAttribute([s,'rotateAxisZ'], num2str(rotation(3)));
            CSGRotationNode.setAttribute([s,'rotateAxisX'], num2str(deg2rad(rotation(1))));
            CSGRotationNode.setAttribute([s,'rotateAxisY'], num2str(deg2rad(rotation(2))));
            CSGRotationNode.setAttribute([s,'rotateAxisZ'], num2str(deg2rad(rotation(3))));
        else
            error('No rotation specified. Unable to create SBML Spatial file.');
        end
        
        if isfield(object, 'scale')
            objsize = object.scale;%.*resolution;
            CSGScaleNode = docNode.createElement([s,'csgScale']);
            CSGScaleNode.setAttribute([s,'id'], 'scale');
            CSGScaleNode.setAttribute([s,'scaleX'], num2str(objsize(1)));
            CSGScaleNode.setAttribute([s,'scaleY'], num2str(objsize(2)));
            CSGScaleNode.setAttribute([s,'scaleZ'], num2str(objsize(3)));
        else
            error('No scale specified. Unable to create SBML Spatial file.');
        end
        
        CSGPrimitiveNode = docNode.createElement([s,'csgPrimitive']);
        CSGPrimitiveNode.setAttribute([s,'id'],type);
        % CSGPrimitiveNode.setAttribute([s,'spatialId'],'cube');
        if strcmpi('sphere',type)
            CSGPrimitiveNode.setAttribute([s,'primitiveType'],'SOLID_SPHERE');
        elseif strcmpi('cube',type)
            CSGPrimitiveNode.setAttribute([s,'primitiveType'],'SOLID_CUBE');
        else
            warning('unrecognized primitive type specified, defaulting to SOLID_CUBE');
            CSGPrimitiveNode.setAttribute([s,'primitiveType'],'SOLID_CUBE');
        end
        
        %     CSGScaleNode.appendChild(CSGPrimitiveNode);
        CSGTransformationNode.appendChild(CSGRotationNode);
        CSGTransformationNode.appendChild(CSGTranslationNode);
        CSGTransformationNode.appendChild(CSGScaleNode);
        %         CSGRotationNode.appendChild(CSGPrimitiveNode);
        %         CSGScaleNode.appendChild(CSGRotationNode);
        %         CSGTranslationNode.appendChild(CSGScaleNode);
        %     CSGNodeNode.appendChild(CSGTransformationNode);
        %     CSGNodeNode.appendChild(CSGPrimitiveNode);
        %         CSGObjectNode.appendChild(CSGTranslationNode);
        CSGObjectNode.appendChild(CSGTransformationNode);
        CSGObjectNode.appendChild(CSGPrimitiveNode);
        %     CSGObjectNode.appendChild(CSGNodeNode);
        ListOfCSGObjectsNode.appendChild(CSGObjectNode);
        
    end
    
    CSGeometryNode.appendChild(ListOfCSGObjectsNode);
    geometryDefNode.appendChild(CSGeometryNode);
    %%%