function [docNode,GeowrapperNode,geometryDefNode] = addMeshObjects(meshData,docNode,geometryDefNode,GeowrapperNode)

    %%Define all the Mesh objects
    % Parametric Geometry
    ParaGeometryNode = docNode.createElement('spatial:ParametricGeometry');
    % geometryDefNode.appendChild(ParaGeometryNode);
    
    %List of Parametric Objects
    ListOfParaObjectsNode = docNode.createElement('spatial:ListOfParametricObjects');
    ListOfParaObjectsNode.setAttribute('id', meshData.name);%(name e.g. 'cell'/'nuc')
    ListOfParaObjectsNode.setAttribute('polygonType', 'triangle');
    % ListOfParaObjectsNode.setAttribute('domain', DomainID);
    
    zslices = zeros(size(meshData.list(1).img,3),1);
    for i = 1:length(meshData.list)
        object = meshData.list(i);
        name = object.name;
        type = object.type;
        
        %assume the cell comes first
        zslicesold = zslices;
        zslices = sum(sum(object.img(1),1),2);
        
        ParaObjectNode = docNode.createElement('spatial:ParaObject');
        ParaObjectNode.setAttribute('id',[name]);%['Sp_', name]);
        %for now we will assume we are only dealing with triangulated meshes
        ParaObjectNode.setAttribute('polygonType','triangle');
        %D.Sullivan 4/15/14 - fixed naming bug.
        %ParaObjectNode.setAttribute('domain',DomainID);
        ParaObjectNode.setAttribute('domain',name);
        
        %This is actually not in the specification for parametric objects,
        %but I think it is important to match with CSGObjects. D. Sullivan
        if isfield(object,'ordinal')
            ParaObjectNode.setAttribute('ordinal', num2str(object.ordinal));
        end
        
        %Get mesh points
        %remove the top and bottom slice of the nucleus fluorescence to ensure
        %that the nucleus lies within the cell.
        if strcmpi(object.name,'NU')
            slices = find(max(max(object.img))>0);
            botslice = min(slices);
            topslice = max(slices);
            object.img(:,:,1) = object.img(:,:,botslice).*0;
            object.img(:,:,end) = object.img(:,:,topslice).*0;
            
        end
        FV = getMeshPoints(object.img);
        FV.vertices = FV.vertices.*repmat(meshData.resolution,size(FV.vertices,1),1);
        if strcmpi(object.name,'CP')
            save('./FVCP_tut.mat','FV');
            %        load('./FVCP1.mat','FV');
            %        FV.vertices = FV.vertices-repmat([3,3,3],size(FV.vertices,1),1);
            %         FV.vertices = FV.vertices.*repmat([1,1,2],size(FV.vertices,1),1);%.*1.1-repmat([0.1,0.1,0.1],size(FV.vertices,1),1);
            %         FV.faces = FV.faces*1.1;
        else
            %         FV = getMeshPoints(object.img);
            save('./FVNU_tut.mat','FV')
            %         load('./FVNU1.mat','FV');
            %         FV.vertices = FV.vertices-repmat([3,3,3],size(FV.vertices,1),1);
            %         FV.vertices = FV.vertices;%*0.9+repmat([0.1,0.1,0.1],size(FV.vertices,1),1);
            %         FV.faces = FV.faces*0.9;
        end
        PolyObjectNode = docNode.createElement('spatial:PolygonObject');
        PolyObjectNode.setAttribute('pointIndex', mat2str(FV.vertices) );
        % Rohan Arepally (added faces as an attribute of PolyObjectNode)
        PolyObjectNode.setAttribute('faces', mat2str(FV.faces) );
        ParaObjectNode.appendChild(PolyObjectNode);
        ListOfParaObjectsNode.appendChild(ParaObjectNode);
        
        %         compartment = docNode.createElement('compartment');
        %         dicomWrapper = num2str(dicomuid());
        %         dicomWrapper = strrep(dicomWrapper, '.', '');
        % %         compartment.setAttribute('metaid',dicomWrapper);
        %         compartment.setAttribute('id',meshData.list(i).name);
        %         compartment.setAttribute('name',meshData.list(i).name);
        %         compartment.setAttribute('spatialDimensions','3');
        %         compartment.setAttribute('size','50000');
        %         compartment.setAttribute('units','m');
        %         compartment.setAttribute('constant','true');
        %         %compartment mapping
        %         mapping = docNode.createElement([s,'compartment']);
        %         DomainID = meshData.list(i).name;%'subdomain0';
        %         mapping.setAttribute([s,'spatialId'],[DomainID,meshData.list(i).name]);
        %         mapping.setAttribute([s,'compartment'],[meshData.list(i).name]);
        %         mapping.setAttribute([s,'domainType'],DomainID);
        %         mapping.setAttribute([s,'unitSize'],'1');
        %         %add children node
        %         compartment.appendChild(mapping);
        %         ListOfCompartments.appendChild(compartment);
    end
%     wrapperNode.appendChild(ListOfCompartments);
    ParaGeometryNode.appendChild(ListOfParaObjectsNode);
    geometryDefNode.appendChild(ParaGeometryNode);
