function [ embedding_model ] = compute_best_embedding( distance_matrix, savedir, options )
%COMPUTE_BEST_EMBEDDING Summary of this function goes here
%   Detailed explanation goes here

if ~exist('options', 'var')
    options = [];
end

options = ml_initparam(options, struct('niter', 1));


if ~exist(savedir, 'dir')
    mkdir(savedir)
end

embedding_infos = cell(1, options.niter);
savefiles = cell(1, options.niter);

rng_state = rng;

for i = 1:options.niter
    savefiles{i} = [savedir filesep 'iter' num2str(i) '.mat'];
    tmpfile = [savefiles{i} '.tmp'];
    
    if ~exist(savefiles{i}, 'file') && ~exist(tmpfile, 'file')
        system (['touch ' tmpfile])
        rng(i)
        
        [embedding_model] = embed_distance_matrix(distance_matrix, options);
        save(savefiles{i}, 'embedding_model')
        
        system(['rm ' tmpfile])
    elseif exist(savefiles{i}, 'file')
        load(savefiles{i})
        
    else
        continue;
    end
    
    embedding_infos{i} = embedding_model.embedding_info;
end

rng(rng_state);

keepinds = find(~cellfun(@isempty, embedding_infos));

if isempty(keepinds)
    embedding_model = [];
    return;
end
    
stresses = cellfun(@(x) x.stress, embedding_infos(keepinds));

[~, ind] = min(stresses);

load(savefiles{keepinds(ind)});


end

