function [ cell_feats, nuc_feats, bothfeat, featnames] = cell2objfeat( img )
%takes a trinary image


%%%%% Cell Feats
imcell = img>0;
imdna = img>1;

objs = ml_3dfindobj(imcell);
[~, ind] = max(cellfun(@(x) size(x.voxels,2), objs));
cell_sof_feat = ml_objfeat3d_noDNA(objs(ind), [], size(imcell));

cell_zproj_props = regionprops(sum(imcell,3)>0, 'All');

cell_align = imrotate(imcell, -cell_zproj_props.Orientation);

cell_maj_props = regionprops(squeeze(sum(cell_align,1)>0), 'All');
cell_min_props = regionprops(squeeze(sum(cell_align,2)>0), 'All');

cell_zproj_feats = [cell_zproj_props.Area, cell_zproj_props.MajorAxisLength, cell_zproj_props.MinorAxisLength, cell_zproj_props.Eccentricity, cell_zproj_props.ConvexArea, cell_zproj_props.FilledArea, cell_zproj_props.EquivDiameter, cell_zproj_props.Solidity, cell_zproj_props.Extent, cell_zproj_props.Perimeter];
cell_majproj_feats = [cell_maj_props.Area, cell_maj_props.MajorAxisLength, cell_maj_props.MinorAxisLength, cell_maj_props.Eccentricity, cell_maj_props.ConvexArea, cell_maj_props.FilledArea, cell_maj_props.EquivDiameter, cell_maj_props.Solidity, cell_maj_props.Extent, cell_maj_props.Perimeter];
cell_minproj_feats = [cell_min_props.Area, cell_min_props.MajorAxisLength, cell_min_props.MinorAxisLength, cell_min_props.Eccentricity, cell_min_props.ConvexArea, cell_min_props.FilledArea, cell_min_props.EquivDiameter, cell_min_props.Solidity, cell_min_props.Extent, cell_min_props.Perimeter];

cell_surfacearea = bwperim(imcell, 18);
cell_surfacearea = sum(cell_surfacearea(:));

%%%%% DNA Feats
objs = ml_3dfindobj(imdna);
[~, ind] = max(cellfun(@(x) size(x.voxels,2), objs));
nuc_sof_feat = ml_objfeat3d_noDNA(objs(ind), [], size(imdna));

nuc_zproj_props = regionprops(sum(imdna,3)>0, 'All');

nuc_align = imrotate(imdna, -nuc_zproj_props.Orientation);

nuc_maj_props = regionprops(squeeze(sum(nuc_align,1)>0), 'All');
nuc_min_props = regionprops(squeeze(sum(nuc_align,2)>0), 'All');

nuc_zproj_feats = [nuc_zproj_props.Area, nuc_zproj_props.MajorAxisLength, nuc_zproj_props.MinorAxisLength, nuc_zproj_props.Eccentricity, nuc_zproj_props.ConvexArea, nuc_zproj_props.FilledArea, nuc_zproj_props.EquivDiameter, nuc_zproj_props.Solidity, nuc_zproj_props.Extent, nuc_zproj_props.Perimeter];
nuc_majproj_feats = [nuc_maj_props.Area, nuc_maj_props.MajorAxisLength, nuc_maj_props.MinorAxisLength, nuc_maj_props.Eccentricity, nuc_maj_props.ConvexArea, nuc_maj_props.FilledArea, nuc_maj_props.EquivDiameter, nuc_maj_props.Solidity, nuc_maj_props.Extent, nuc_maj_props.Perimeter];
nuc_minproj_feats = [nuc_min_props.Area, nuc_min_props.MajorAxisLength, nuc_min_props.MinorAxisLength, nuc_min_props.Eccentricity, nuc_min_props.ConvexArea, nuc_min_props.FilledArea, nuc_min_props.EquivDiameter, nuc_min_props.Solidity, nuc_min_props.Extent, nuc_min_props.Perimeter];

nuc_surfacearea = bwperim(imdna, 18);
nuc_surfacearea = sum(nuc_surfacearea(:));

%%% Combine Feats
cell_centroid = regionprops(imcell, 'Centroid');
dna_centroid = regionprops(imdna, 'Centroid');

cell_nuc_dist = pdist([cell_centroid.Centroid;dna_centroid.Centroid]);
cell_nuc_zdist = dna_centroid.Centroid(3) - cell_centroid.Centroid(3);
cell_nuc_xydist = pdist([cell_centroid.Centroid(1:2);dna_centroid.Centroid(1:2)]);

%%% Concatonate all the feats
cell_feats  = [cell_sof_feat(:,[1 4 6 7 8 9 11]) pc12_cylindricity(imcell), cell_surfacearea, cell_surfacearea/cell_sof_feat(:,1), cell_zproj_feats, cell_majproj_feats, cell_minproj_feats];
nuc_feats = [nuc_sof_feat(:,[1 4 6 7 8 9 11]) pc12_cylindricity(imdna), nuc_surfacearea, nuc_surfacearea/nuc_sof_feat(:,1), nuc_zproj_feats, nuc_majproj_feats, nuc_minproj_feats];
bothfeat = [cell_nuc_dist, cell_nuc_zdist, cell_nuc_xydist, cell_zproj_props.Orientation - nuc_zproj_props.Orientation];

%%% Get all the feature names
proj_feat_names = {'Area', 'MajorAxisLength', 'MinorAxisLength', 'Eccentricity', 'ConvexArea', 'FilledArea', 'EquivDiameter', 'Solidity', 'Extent', 'Perimeter'};

cell_sof_name = {'Cell - Number of voxels in object', ...
    'Cell - A measure of eccentricity of the object', ...
    'Cell - A measure of roundness of the object', ...
    'Cell - The length of the object''s skeleton by homotopic thinning', ...
    'Cell - The ratio of skeleton length to the area of the convex hull of the skeleton', ...
    'Cell - The fraction of object pixel contained within the skeleton', ...
    'Cell - The ratio of the number of branch points in skeleton to length of skeleton', ...
    'Cell - Cylindricity', ...
    'Cell - Surface area', ...
    'Cell - Surface area to volume ratio'};

cell_zproj_names = cellfun(@(x) ['Cell - zproj - ' x], proj_feat_names, 'UniformOutput', false);
cell_majproj_names = cellfun(@(x) ['Cell - majproj - ' x], proj_feat_names, 'UniformOutput', false);
cell_minproj_names = cellfun(@(x) ['Cell - minproj - ' x], proj_feat_names, 'UniformOutput', false);

cell_feat_names = [cell_sof_name, cell_zproj_names, cell_majproj_names, cell_minproj_names];

nuc_sof_name = {'Nuc - Number of voxels in object', ...
    'Nuc - A measure of eccentricity of the object', ...
    'Nuc - A measure of roundness of the object', ...
    'Nuc - The length of the object''s skeleton by homotopic thinning', ...
    'Nuc - The ratio of skeleton length to the area of the convex hull of the skeleton', ...
    'Nuc - The fraction of object pixel contained within the skeleton', ...
    'Nuc - The ratio of the number of branch points in skeleton to length of skeleton', ...
    'Nuc - Cylindricity', ...
    'Nuc - Surface area', ...
    'Nuc - Surface area to volume ratio'};

nuc_zproj_names = cellfun(@(x) ['Nuc - zproj -' x], proj_feat_names, 'UniformOutput', false);
nuc_majproj_names = cellfun(@(x) ['Nuc - majproj -' x], proj_feat_names, 'UniformOutput', false);
nuc_minproj_names = cellfun(@(x) ['Nuc - minproj -' x], proj_feat_names, 'UniformOutput', false);

nuc_feat_names = [nuc_sof_name, nuc_zproj_names, nuc_majproj_names, nuc_minproj_names];

bothfeat_name = {'Euclidean distance between cell and dna shape center of mass', ...
    'Vertical offset between dna and cell shape center of mass', ...
    'Horizontal offset between dna and cell shape center of mass', ...
    'Angle between z-projected major axis' };

featnames = {cell_feat_names, nuc_feat_names, bothfeat_name};
    
end

