function SSPaper_figure5(model, dnaintens, figuredir)
%This function creates the fifth figure in Buck et al. 
% The output is two images showing projections of the shape space along its
% 1st and 2nd principal components and 3rd and 2nd components respectively.
% These images are shown and saved as .tiff images. 
%
%Created by: Devin Sullivan 5/13/14

%move to correct directory so relative file paths work
file = 'SSPaper_figure5.m';
currpath = which(file);
currfolder = currpath(1:end-length(file));
cd(currfolder)
%load shape space - actually only 231 shapes
% load('../shapeSpaces/HeLa300.mat');
load('../shapeSpaces/cellModel.mat');
%load dna intensities - referenced in the same order
load('./dnaintens.mat');

%%%SHOW SHAPE SPACE (PART A&B)%%%

%remove the distances that are nans 
%We do this outside of showShapeSpace here b/c we want to be able to use
%the eigen decomposition easily
%remove rows fully populated with nans
d = model.cellShapeModel.distances_incomplete;
d_nan = d;
d_nan(logical(eye(size(d)))) = nan;

rminds = all(isnan(d_nan),1);
d(rminds,:) = [];
d(:,rminds) = [];

keepinds = find(~rminds);

% labels = labels(keepinds);
dnaintens = dnaintens(keepinds);
nimgs = model.cellShapeModel.numimgs;
cellnums = 1:nimgs;
cellnums = cellnums(keepinds);


%remove any other entries with nans
rminds = any(isnan(triu(d)),1);
d(rminds,:) = [];
d(:,rminds) = [];

keepinds = find(~rminds);

dnaintens = dnaintens(keepinds);
cellnums = cellnums(keepinds);

% nimgs = size(d,1);
model_complete = model;
model_complete.nuclearShapeModel.distances = d;
model_complete.cellShapeModel.distances = d;
model_complete.cellShapeModel.numimgs = size(d,1);
% nimgs = size(d,1);

%decompose the distance matrix 
[proj_orig, eig] = cmdscale(d);
% [A,B,r,U2,V2,stats] = canoncorr(d, dnaintens');
neigs = size(proj_orig,2);
R = zeros(neigs,1);
for i = 1:neigs
    Rtot = corrcoef([proj_orig(:,i),dnaintens']);
    R(i) = abs(Rtot(2));
end

%%%COMPUTE PART A%%%
%show the shape space for the 1vs2 components colored by DNA
[img12,proj_12,eig] = showShapeSpace(model_complete,dnaintens,0, proj_orig(:,1:2), @jet);

%%%COMPUTE PART B%%%
%show the second and 3rd 
%show the shape space for components 3vs2 components colored by DNA
[img32,proj_32,eig] = showShapeSpace(model_complete,dnaintens,0, proj_orig(:,3:-1:2), @jet);

%show the images
figure,imshow(img12,[])
titlestr = 'Figure 5 a';
xlabelstr = ['PC1 R = ' num2str(R(1))];
ylabelstr = ['PC2 R = ' num2str(R(2))];
title(titlestr,'FontSize',20,'Fontname','Ariel')
xlabel(xlabelstr,'FontSize',20,'Fontname','Ariel');
ylabel(ylabelstr,'FontSize',20,'Fontname','Ariel');

%Save the image for 1vs2 (without labels)
imwrite(img12,'Fig5a','tiff');

figure,imshow(img32,[])
titlestr = 'Figure 5 b';
xlabelstr = ['PC3 R = ' num2str(R(3))];
ylabelstr = ['PC2 R = ' num2str(R(2))];
title(titlestr,'FontSize',20,'Fontname','Ariel')
xlabel(xlabelstr,'FontSize',20,'Fontname','Ariel');
ylabel(ylabelstr,'FontSize',20,'Fontname','Ariel');

%Save the image for 3vs2 (without labels)
imwrite(img32,'Fig5b','tiff');

%%%END SHOW SHAPE SPACE (PART A&B)%%%

%%%GENERATE DNA WEIGHTED SHAPE SPACE WALK (SUP FIG 5)%%%


%%%END GENERATE DNA WEIGHTED SHAPE SPACE WALK (SUP FIG 5)%%%