% Author: Gregory R. Johnson (gj@cs.cmu.edu)
% Created: Oct 30, 2015
% masterscript.m reproduces the figures and tables from 
% G. R. Johnson, T. E. Buck, D. P. Sullivan, G. K. Rohde, and R. F. Murphy 
% (2015) Joint modeling of cell and nuclear shape variation. Molecular Biology of the Cell,
% http://murphylab.web.cmu.edu/software/2015_MBoC_Cell_And_Nuclear_Shape/index.html

% Copyright (C) 2015 Murphy Lab
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

filepath = which(mfilename);
fileseps = regexp(filepath, filesep);
parentdir = filepath(1:fileseps(end-1));


%HeLa block
try
    build = true;
    addpath([parentdir filesep 'diffeo_tub' filesep 'code'])
    
    try
        img2slml_newseg_com_nuc(parentdir, build)
        img2slml_newseg_cell(parentdir, build)
        img2slml_newseg_nuc(parentdir, build)
    catch err
        warning('Intermediate results for HeLa cells not found. Attemping to build from scratch. This will take a LONG time.')
        getReport(err);
        
        build = false;
        img2slml_newseg_com_nuc(parentdir, build)
        img2slml_newseg_cell(parentdir, build)
        img2slml_newseg_nuc(parentdir,  build)
    end
    
    %build hela figures
    figuredir = [parentdir filesep 'figures' filesep];
    addpath(genpath(figuredir));

    dir_hela = [parentdir filesep 'diffeo_tub' filesep 'diffeomorphic_newesg' filesep];
    load([dir_hela filesep 'cellModel.mat']);
    
    script_figure1_pipeline_overview(model, figuredir)
    script_figure2_Hela3D; %Hela-3D hape space + simplex and interpolated shape
    script_figure3(model, figuredir); %Graph evaluating approximatons with Hela 3D

    clearvars -except parentdir
    script_test_cell_nuc_dep_hela_pred_all
    
    %do cell shape nuc shape prediction
catch err
    warning('HeLa images not found - figures and tables for HeLa cells will not be generated.')
    getReport(err);
end

rmpath(genpath([parentdir filesep 'diffeo_tub' filesep 'code']));

clearvars -except parentdir

%MCF7 block
try
    addpath([parentdir filesep 'BBBC021' filesep 'code'])
    build = true;
    try
        img2slml_bbbc021_cell(parentdir, build)
        img2slml_bbbc021_nuc(parentdir, build)
    catch err
        warning('Intermediate results for MCF7 cells not found. Attempting to build from scratch. This will take a LONG time.')
        getReport(err);
        
        build = false;
        img2slml_bbbc021_cell(parentdir, build)
        img2slml_bbbc021_nuc(parentdir, build)
    end
    
    %do cell shape nuc shape prediction
    script_test_cell_nuc_dep_pred_iter
    
catch err
    warning('MCF7 images not found - figures and tables for MCF7 cells will not be generated.')
    getReport(err);
end

rmpath(genpath([parentdir filesep 'BBBC021' filesep 'code']));
clearvars -except parentdir



%H1299 block
try
    addpath([parentdir filesep 'Alon' filesep 'code']);
    build = true;
    try
        img2slml_alon_nuc_align_v3(parentdir,build)
        img2slml_alon_cell_v3(parentdir,build)
        img2slml_alon_nuc_v3(parentdir,build)
    catch err
        warning('Intermediate results for H1299 cells not found. Attempting to build from scratch. This will take a LONG time.')
        getReport(err);
        
        build = false;
        
        img2slml_alon_nuc_align_v3(parentdir,build)
        img2slml_alon_cell_v3(parentdir,build)
        img2slml_alon_nuc_v3(parentdir,build)
    end
    
    script_kde_trans_model_alon([],[], [],parentdir,[])
    script_test_cell_nuc_pred
    
catch err
    warning('H1299 images not found - figures and tables for H1299 cells will not be generated.')
    getReport(err);
end
