function pred_table = save_pred_dat(cell2nuc, nuc2cell, cell2nuc_pvals, nuc2cell_pvals, ucond, niter, object_info, saveparent)

tot_num_cells =                 cellfun(@(x) length(x.y_pred_err),                  nuc2cell);

nuc2cell_avg_kernel_size =  cellfun(@(x) mean(x.k_x2y),                         nuc2cell);
nuc2cell_std_kernel_size =  cellfun(@(x) std(x.k_x2y),                          nuc2cell);
nuc2cell_mse =              cellfun(@(x) mean(x.y_pred_err),                    nuc2cell);
nuc2cell_mse_norm =         nuc2cell_mse ./ cellfun(@(x) mean(x.mean_rand_err), nuc2cell);
nuc2cell_tot_stat_sig =     cellfun(@(x) sum(x.y_pval < 0.05),                  nuc2cell);
nuc2cell_fract_stat_sig =   cellfun(@(x) mean(x.y_pval < 0.05),                 nuc2cell);
nuc2cell_num_perms =        cellfun(@(x) x.isdone*niter,                        nuc2cell);
nuc2cell_log10_pval =       cellfun(@(x) sum(log10(x.y_pval)),                  nuc2cell);

cell2nuc_avg_kernel_size =  cellfun(@(x) mean(x.k_x2y),                         cell2nuc);
cell2nuc_std_kernel_size =  cellfun(@(x) std(x.k_x2y),                          cell2nuc);
cell2nuc_mse =              cellfun(@(x) mean(x.y_pred_err),                    cell2nuc);
cell2nuc_mse_norm =         cell2nuc_mse ./ cellfun(@(x) mean(x.mean_rand_err), cell2nuc);
cell2nuc_tot_stat_sig =     cellfun(@(x) sum(x.y_pval < 0.05),                  cell2nuc);
cell2nuc_fract_stat_sig =   cellfun(@(x) mean(x.y_pval < 0.05),                 cell2nuc);
cell2nuc_num_perms =        cellfun(@(x) x.isdone*niter,                        cell2nuc);
cell2nuc_log10_pval =       cellfun(@(x) sum(log10(x.y_pval)),                  cell2nuc);

pval_ratio = nuc2cell_log10_pval-cell2nuc_log10_pval;

avg_pval_ratio = cellfun(@(x,y) mean(x.y_pval./y.y_pval),                       nuc2cell, cell2nuc);
std_err_pval_ratio = cellfun(@(x,y) std(x.y_pval./y.y_pval)/length(y.y_pval),       nuc2cell, cell2nuc);

nuc2cell_err_norm_over_cell2nuc_err_norm = nuc2cell_mse_norm./cell2nuc_mse_norm;


var_names = {'conditions', 'tot_num_cells', ...
             'nuc2cell_avg_kernel_size', 'nuc2cell_std_kernel_size', 'MSE_nuc2cell', 'MSE_nuc2cell_normalized', 'nuc2cell_tot_stat_sig', 'nuc2cell_fract_stat_sig', 'nuc2cell_log10_pval', 'nuc2cell_num_perms', ...
             'cell2nuc_avg_kernel_size', 'cell2nuc_std_kernel_size', 'MSE_cell2nuc', 'MSE_cell2nuc_normalized', 'cell2nuc_tot_stat_sig', 'cell2nuc_fract_stat_sig', 'cell2nuc_log10_pval', 'cell2nuc_num_perms', ...
             'pval_nuc2cell_over_pval_cell2nuc', 'avg_pval_ratio', 'std_err_pval_ratio', 'nuc2cell_err_norm_over_cell2nuc_err_norm'};
         
if ~exist('object_info', 'var') || isempty(object_info)
    row_names = cellstr(num2str((1:length(cell2nuc))'))';
else
    row_names = object_info.condition_names;
end

pred_table = table(ucond', tot_num_cells', ...
                   nuc2cell_avg_kernel_size', nuc2cell_std_kernel_size', nuc2cell_mse', nuc2cell_mse_norm', nuc2cell_tot_stat_sig', nuc2cell_fract_stat_sig', nuc2cell_log10_pval', nuc2cell_num_perms',  ...
                   cell2nuc_avg_kernel_size', cell2nuc_std_kernel_size', cell2nuc_mse', cell2nuc_mse_norm', cell2nuc_tot_stat_sig', cell2nuc_fract_stat_sig', cell2nuc_log10_pval', cell2nuc_num_perms', ... 
                   pval_ratio', ...
                   avg_pval_ratio', std_err_pval_ratio', nuc2cell_err_norm_over_cell2nuc_err_norm', ...
                   'VariableNames', var_names, 'RowNames', row_names');
               
pred_table_path = [saveparent filesep 'pred_table.csv'];
writetable(pred_table, pred_table_path, 'WriteRowNames', true);

var_names_short = {'conditions','MSE_nuc2cell_normalized', 'nuc2cell_fract_stat_sig', ...
                    'MSE_cell2nuc_normalized', 'cell2nuc_fract_stat_sig', ...
                    'pval_nuc2cell_over_pval_cell2nuc'};
                
pred_table_short = table(ucond', nuc2cell_mse_norm', nuc2cell_fract_stat_sig', ...
                        cell2nuc_mse_norm', cell2nuc_fract_stat_sig', ...
                        pval_ratio', ...
                        'VariableNames', var_names_short, 'RowNames', row_names');
pred_table_short_path = [saveparent filesep 'pred_table_short.csv'];
writetable(pred_table_short, pred_table_short_path, 'WriteRowNames', true)
                    


colors = jet(length(ucond));

pval_vs_mse_alldat_fig = [saveparent filesep 'pval_vs_mse_alldat_cell_and_nuc.tif'];

figure, 
for i = 1:length(cell2nuc)
    scatter(nuc2cell_mse_norm(i), nuc2cell_fract_stat_sig(i), 300, colors(i,:), '*')
    hold on
    scatter(cell2nuc_mse_norm(i), cell2nuc_fract_stat_sig(i), 300, colors(i,:), '.')
end
xlabel('Normalized MSE')
ylabel('% statistically significant predicitons')
saveas(gcf, pval_vs_mse_alldat_fig, 'tif')
close(gcf)



pval_vs_mse_alldat_fig = [saveparent filesep 'pval_vs_mse_alldat_cell_given_nuc.tif'];

figure, 
for i = 1:length(cell2nuc)
    scatter(nuc2cell_mse_norm(i), nuc2cell_fract_stat_sig(i), 300, colors(i,:), '.')
    hold on
end
xlabel('Normalized MSE')
ylabel('% statistically significant predicitons')
saveas(gcf, pval_vs_mse_alldat_fig, 'tif')
close(gcf)


pval_vs_mse_alldat_fig = [saveparent filesep 'pval_vs_mse_alldat_nuc_given_Cell.tif'];

figure, 
for i = 1:length(cell2nuc)
    scatter(cell2nuc_mse_norm(i), cell2nuc_fract_stat_sig(i), 300, colors(i,:), '.')
     hold on
end
xlabel('Normalized MSE')
ylabel('% statistically significant predicitons')
saveas(gcf, pval_vs_mse_alldat_fig, 'tif')
close(gcf)



pval_vs_mse_percell_fig = [saveparent filesep 'pval_vs_mse_percell.tif'];

figure,
for i = 1:length(nuc2cell)
    scatter(log10(cell2nuc{i}.y_pred_err), cell2nuc{i}.y_pval, 50, colors(i,:), 'x')
    hold on
    scatter(log10(nuc2cell{i}.y_pred_err), nuc2cell{i}.y_pval, 100, colors(i,:), '.')
end
xlabel('log10(Predicted position MSE)')
ylabel('P-value')
saveas(gcf, pval_vs_mse_percell_fig, 'tif')
close(gcf)



ucond = sort(ucond);

if ~exist('object_info', 'var') || isempty(object_info)
    return;
end

for i = 1:length(ucond)
    %find the objects that correspond to this condition
    ucondinds = object_info.obj_condition == ucond(i) ;
    objinds = find(ismember(object_info.transition_obj_inds(:,1), find(ucondinds)));

%     objinds = objinds(all(~ismember(object_info.transition_obj_inds(objinds,:), naninds),2));

    obj_inds0 = object_info.transition_obj_inds(objinds,1);
    obj_inds1 = object_info.transition_obj_inds(objinds,2);
    
    
    
    cell2nuc_pvals0 = cell2nuc_pvals(obj_inds0);
    cell2nuc_pvals1 = cell2nuc_pvals(obj_inds1);
    
    nuc2cell_pvals0 = nuc2cell_pvals(obj_inds0);
    nuc2cell_pvals1 = nuc2cell_pvals(obj_inds1);
    
    datsize = size(nuc2cell_pvals0);
    

    nuc2cell_transmat{i} = zeros(2,2);
    cell2nuc_transmat{i} = zeros(2,2);
    
    statsig = 0.05;
    for j = 1:length(nuc2cell_pvals0)
        ind0 = (nuc2cell_pvals0(j) >= statsig) + 1;
        ind1 = (nuc2cell_pvals1(j) >= statsig) + 1;
        nuc2cell_transmat{i}(ind0, ind1) = nuc2cell_transmat{i}(ind0, ind1) + 1;
        
        ind0 = (cell2nuc_pvals0(j) >= statsig) + 1;
        ind1 = (cell2nuc_pvals1(j) >= statsig) + 1;
        cell2nuc_transmat{i}(ind0, ind1) = cell2nuc_transmat{i}(ind0, ind1) + 1;
    end
        
    nuc2cell_transmat_norm{i} = nuc2cell_transmat{i}./repmat(sum(nuc2cell_transmat{i},2), [1,2]);
    cell2nuc_transmat_norm{i} = cell2nuc_transmat{i}./repmat(sum(cell2nuc_transmat{i},2), [1,2]);
    
    figure,
    
    jitter = 100;
    subplot(1,2,1)
    scatter(nuc2cell_pvals0+rand(datsize)/jitter, nuc2cell_pvals1+rand(datsize)/jitter, 50, colors(i,:))
    title('nuc2cell')
    xlabel('p-value t=0')
    ylabel('p-value t=1')
    axis([0,1,0,1])
    
    subplot(1,2,2)
    scatter(cell2nuc_pvals0+rand(datsize)/jitter, cell2nuc_pvals1+rand(datsize)/jitter, 50, colors(i,:))
    title('cell2nuc')
    xlabel('p-value t=0')
    ylabel('p-value t=1')
    axis([0,1,0,1])
    
    saveas(gcf, [saveparent filesep 'transitions_' object_info.condition_names{i} '.tif'], 'tif')
    close(gcf)
%     title(object_info.condition_names{i})
    
end

transmat_table_path = [saveparent filesep 'transitions_all.csv'];


nuc2cell_transmat_table = table(nuc2cell_transmat{:}, 'VariableNames', object_info.condition_names);
nuc2cell_transmat_table_norm = table(nuc2cell_transmat_norm{:}, 'VariableNames', object_info.condition_names);

nuc2cell_transmat_table_all = vertcat(nuc2cell_transmat_table, nuc2cell_transmat_table_norm);

cell2nuc_transmat_table = table(cell2nuc_transmat{:}, 'VariableNames', object_info.condition_names);
cell2nuc_transmat_table_norm = table(cell2nuc_transmat_norm{:}, 'VariableNames', object_info.condition_names);

cell2nuc_transmat_table_all = vertcat(cell2nuc_transmat_table, cell2nuc_transmat_table_norm);

transmat_table_all = vertcat(nuc2cell_transmat_table_all, cell2nuc_transmat_table_all);


writetable(transmat_table_all, transmat_table_path)




end

