function [ output_args ] = script_c2c12_ss_imgs(skip_embedding)
%SCRIPT_KDE_TRANS_MODEL Summary of this function goes here
%   Detailed explanation goes here

if ~exist('skip_embedding', 'var')
    skip_embedding = false;
end


addpath(genpath('/projects/cellorganizer/gj/shapeSpace/c2c12/code'))

script_img2diffeomorphic_newseg_setup


figuredir = [projectdir filesep 'figures'];
if ~exist(figuredir, 'dir')
    mkdir(figuredir)
end

tempdir = [figuredir filesep mfilename];
if ~exist(tempdir, 'dir')
    mkdir(tempdir)
end

load([savedir filesep 'cellModel.mat'])

methods = {struct('method',  'regress',   'weight_factor', 0, 'desired_shape_space_dimensionality', 7), ...
                struct('method',  'regress',   'weight_factor', 1, 'desired_shape_space_dimensionality', 7), ...
                struct('method',  'regress',   'weight_factor', 2, 'desired_shape_space_dimensionality', 7)};

niter_per_save = 200;
totiter = 10000;

numrounds = totiter / niter_per_save;

if ~skip_embedding
    for i = 1:length(methods)
        itercount = 0;
        embeddingfile = [tempdir filesep 'embedding_' num2str(i)];

        [can_start, final_name, final_exists, temp_name] = chunk_start( embeddingfile, '.mat');

        options = struct();

        if can_start 

            options.embedding = methods{i};
            options.embedding.statset.TolFun = 1E-7;
            options.embedding.statset.MaxIter = niter_per_save;
            options.compute_triangulation = false;

            if final_exists
                load([embeddingfile '.mat'])
                options.embedding.Start = embedding.positions;
            end

            while itercount < totiter

                %on the last one, compute the triangulation
                if itercount == numrounds - 1
                    options.compute_triangulation = true;
                end

                embedding = embed_distance_matrix(model.cellShapeModel.distances_incomplete, options);

                %update the start positions
                options.embedding.Start = embedding.positions;
                itercount = itercount + niter_per_save;

                save(embeddingfile, 'embedding', 'itercount')
                %copy the file as a backup
                copyfile([embeddingfile '.mat'], [embeddingfile '_' num2str(itercount) '.mat'])

            end

            chunk_finish(temp_name);
        end
    end
end

ucell = unique(cellid);

imsize = size(model.cellShapeModel.imfunc(1));

for j = 1:length(methods)

    methodsdir = [tempdir filesep 'method_' num2str(j)];
    
    if ~exist(methodsdir, 'dir')
        mkdir(methodsdir)
    end
    
    embeddingfile = [tempdir filesep 'embedding_' num2str(j) '.mat'];
    
    if exist(embeddingfile, 'file')
        load(embeddingfile)
        
        model.cellShapeModel = ml_initparam(embedding, model.cellShapeModel);
    
        for i = 1:length(ucell)

            imfile = [methodsdir filesep 'cell_' num2str(ucell(i)) '.tif'];

            if ~exist(imfile, 'file')
                colors = ones(length(cellid),3) * 0.3;

                cellinds = cellid==ucell(i);

                cellidcolors = grayrange2rgb(fractlife(cellinds));

                colors(cellinds,:) = cellidcolors;

                imfunc = @(x) repmat(model.cellShapeModel.imfunc(x), [1,1,3]) .* repmat(permute(colors(x,:), [3,1,2]), imsize);

                param.imfunc = imfunc;
                param.synthorder = [find(~cellinds) find(cellinds)];

                figure('color', 'w'),
                showShapeSpaceFigure(model, [], param);
                saveas(gcf, imfile, 'tif')
                close(gcf);
            end

        end
        
    end

end

end