function [model_out, x_pos, y_pos] = script_figure1(model, figuredir)
% script_figure1.m generates the pipeline overview (figure 1)
% from G. R. Johnson, T. E. Buck,D. P. Sullivan, G. K. Rohde, and 
% R. F. Murphy (2015) Joint modeling of cell and nuclear shape variation. 
% Molecular Biology of the Cell,
% http://murphylab.web.cmu.edu/software/2015_MBoC_Cell_And_Nuclear_Shape/index.html
%
% Author: Gregory R. Johnson (gj@cs.cmu.edu)
% Created: March 5, 2015
% Nov 2, 2015 - added header block - gj@cs.cmu.edu

% Copyright (C) 2015 Murphy Lab
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu


tempdir = [figuredir filesep mfilename filesep];
if ~exist(tempdir, 'dir')
    mkdir(tempdir)
end



contrast_stretch = 1.8;

embedfile = [tempdir filesep 'embedfile_2d.mat'];
if ~exist(embedfile, 'file')
    options.embedding.desired_shape_space_dimensionality = 2;
    embed_model_2d = embed_distance_matrix(model.cellShapeModel.distances_incomplete, options);
    save(embedfile, 'embed_model_2d')
else
    load(embedfile)
end
model.cellShapeModel = ml_initparam(embed_model_2d, model.cellShapeModel);
model.nuclearShapeModel = model.cellShapeModel;


cellnums = find(~any(isnan(model.cellShapeModel.positions),2));

numshapes_to_show = 5;
fig_param.imfunc = @(x) im2projection_RGB(model.cellShapeModel.imfunc(x), struct('justz', true))*contrast_stretch;
fig_param.subsize = 750;
% 
panel_w = 8.7;
panel_h = 8.7;


x_pos = [.45];
y_pos = [0.0];

shapespaceimg = [tempdir filesep 'shapespace_2d.tif'];

if ~exist(shapespaceimg, 'file')
     h(1) = figure('color', 'w', 'PaperUnits', 'inches', 'PaperPosition',  [0, 0, 1.5 , 1.5], 'PaperSize', [1.5, 1.5]);
%     set(h(1), 'PaperUnits', 'inches')
%     set(h(1), 'PaperSize', [1.5, 1.5])
%     set(h(1), 'FontSize', 8)
    [fig_param] = showShapeSpaceFigure(model,[], fig_param);
    
    hold on,
    h2 = scatter(x_pos, y_pos, 1, '.');
    
    axis equal
    axis tight
    
    set(gca, 'FontSize', 6)

%     axis([-0.8, 0.8, -0.6, 0.6])
    
%     xlabel('MDS 1', 'FontSize', 12)
%     ylabel('MDS 2', 'FontSize', 12)
    print(shapespaceimg, '-dtiffn', '-r1200')
end




%%%%% Panel 
npts = 1;


% for i = 1:npts
%     [x_pos(i), y_pos(i)] = ginput(1);
% end



%For consistancy we've fixed this point. It is model specific so it may
%need to be changed if a different/updated model is input

frames = cell(1,npts);
for i = 1:npts
    param.position = [ x_pos(i), y_pos(i), zeros(1, size(model.cellShapeModel.positions,2)-2)];

% %     options = model.cellShapeModel.shape_space_options;
%     options.convergence_absolute_error = 0.1;
%     options.convergence_absolute_error_difference = 0.5;
%     
    model.cellShapeModel.shape_space_options.save_stages = 1;
    model.cellShapeModel.shape_space_options.save_intermediates = 1;
    model.cellShapeModel.shape_space_options.save_intermediate_images = 1;

    param.framefolder = [figuredir filesep mfilename filesep 'frames_' num2str(i)];
    
    if ~exist(param.framefolder, 'dir'); 
        mkdir(param.framefolder); 
    end
    
    
    savefile = [param.framefolder filesep 'frame_info.mat'];
    if ~exist(savefile, 'file')
        [imgs{i}, param_out] = model2img({model}, param);
    end
    load([param.framefolder filesep 'frame_info.mat']);
    
    frames{i} = frame;
end



h(2) = figure('color', 'w', 'PaperPosition', [0, 0, panel_w , panel_h]);
hold on

for i = 1:length(frame.xidx)
    for j = 1:length(frame.xidx)
        if i ~= j 
            frame_ind_i = frame.simplex_vertex_ordering(i);
            frame_ind_j = frame.simplex_vertex_ordering(j);
            handle = plot([frame.xtri(frame_ind_i,1), frame.xtri(frame_ind_j,1)], [frame.xtri(frame_ind_i,2), frame.xtri(frame_ind_j,2)], '-');
            set(handle, 'Color', [0.75, 0.75, 0.75])
        end
    end
end

% for i = 1:npts
%     place_image(im2projection_RGB(round(frames{i}.interpolated_image.get_data()), struct('justz', true)), [x_pos(i), y_pos(i)], place_param)
% %     place_image(img2flat(round(frames{i}.interpolated_image.get_data()), repmat(.3, [1,3])), [x_pos(i), y_pos(i)], place_param)
%     text(x_pos(i), y_pos(i), num2str(i))
% end

axis equal

place_param.subsize = 2500;

for i = 1:npts
    x1 = model.cellShapeModel.positions(frames{i}.xidx(frames{i}.simplex_vertex_ordering(1)),:);
    for j = 2:length(frames{i}.simplex_vertex_ordering)
      
        x2 = model.cellShapeModel.positions(frames{i}.xidx(frames{i}.simplex_vertex_ordering(j)),:);
        

        d = sqrt(sum((x1-x2).^2));
            
        fractdist = frames{i}.lambda(j-1)./(frames{i}.lambda(j-1)+1);
        
        x1_new = (x2 - x1).*(fractdist) + x1;
         
        place_image(im2projection_RGB(round(frames{i}.ri{j-1}.interpolated_image.get_data()), struct('justz', true))*contrast_stretch, [x1_new], place_param)
    
        arrow_pos{i,j-1,1} = x1(1:2);
        arrow_pos{i,j-1,2} = x1_new(1:2);
%         scatter(pos(1), pos(2), 
        x1 = x1_new; 
    end
end

for i = 1:size(arrow_pos,1)
    for j = 1:size(arrow_pos,2)
        arrow(arrow_pos{i,j,1}, arrow_pos{i,j,2}, 'Length', 30, 'LineWidth', 1.5, 'linestyle', '-', 'EdgeColor', [0,0,0], 'FaceColor', [0,0,0])
    end
end

txt_offset = 0.015;
for i = 1:length(frame.xidx)
    frame_ind = frame.simplex_vertex_ordering(i);
%     scatter(frame.xtri(frame_ind,1), frame.xtri(frame_ind,2))
     place_image(fig_param.imfunc(frame.xidx(frame_ind)), frame.xtri(frame_ind,:), place_param)
    hold on
%     text(frame.xtri(frame_ind,1)-txt_offset, frame.xtri(frame_ind,2)+txt_offset, num2str(i))
end

axis equal
axis tight

synthimg = im2projection_RGB(round(frames{end}.ri{end}.interpolated_image.get_data()), struct('justz', true))*contrast_stretch;
synthimg(repmat(all(synthimg == 0,3), [1,1,3])) = 1;

synthsave = [tempdir filesep 'fig1_e_cell_only.tif'];
imwrite(synthimg, synthsave, 'tif')

%%%%% Panel B %%%%%
distances_of_interest = model.cellShapeModel.distances_incomplete(cellnums(1:numshapes_to_show), cellnums(1:numshapes_to_show));

for i = 1:numshapes_to_show
    shape_img{i} = fig_param.imfunc(cellnums(i));
    [~, region(i,:)] = pc12_cropImg(shape_img{i});
end

minx = min([region(:,1);region(:,3)]);
maxx = max([region(:,2);region(:,4)]);

for i = 1:length(shape_img)
    shape_img{i} = shape_img{i}(minx:maxx, minx:maxx,:);
end

imwidth = size(shape_img{1},1);

topshapes = [shape_img{:}];
topshapes(repmat(all(topshapes == 0,3), [1,1,3])) = 1;
sideshapes = vertcat(shape_img{:});
sideshapes(repmat(all(sideshapes == 0,3), [1,1,3])) = 1;

white_grid_square = repmat(ones(imwidth), [1,1,3]);
gray_grid_square = white_grid_square;
gray_grid_square(3:end-2, 3:end-2,:) = 0.85;

shape_grid = [[white_grid_square topshapes];[sideshapes, repmat(gray_grid_square, [numshapes_to_show, numshapes_to_show])]];

%add a white bottom row and a white right row;
shape_grid = [[shape_grid ; [white_grid_square repmat(gray_grid_square, [1,numshapes_to_show])]] [white_grid_square; repmat(gray_grid_square, [numshapes_to_show+1,1])]];

h(3) = figure('color', 'w', 'PaperPosition', [0, 0, panel_w , panel_h]);
imshow(shape_grid)
%populate the top row, the left column and diagonal with entries
for i = 1:numshapes_to_show
    for j = 1:numshapes_to_show
        xpos = imwidth + imwidth*(i-1) + imwidth/2;
        ypos = imwidth + imwidth*(j-1) + imwidth/2;
        text(xpos, ypos, sprintf('%1.2f',distances_of_interest(i,j)), 'HorizontalAlignment', 'center')
    end
end

ypos = imwidth + imwidth*(numshapes_to_show) + imwidth/2;
xpos = imwidth + imwidth*(numshapes_to_show) + imwidth/2;
text(xpos, ypos, '0', 'HorizontalAlignment', 'center')


for i = 1:numshapes_to_show
    xpos = imwidth + imwidth*(i-1) + imwidth/2;
    ypos = imwidth + imwidth*(numshapes_to_show) + imwidth/2;
    text(xpos, ypos, '...', 'HorizontalAlignment', 'center')
    
    xpos = imwidth + imwidth*(numshapes_to_show) + imwidth/2;
    ypos = imwidth + imwidth*(i-1) + imwidth/2;
    text(xpos, ypos, '...', 'HorizontalAlignment', 'center')
end


% saveas(h(1), [figuredir 'fig1_d.tif'], 'tif');
% close(h(1));
set(0, 'currentfigure', h(2))
 print([tempdir filesep 'fig1_e.tif'], '-dtiffn', '-r1200')
% saveas(h(2), [tempdir filesep 'fig1_e.tif'], 'tif');
close(h(2))
set(0, 'currentfigure', h(3))
print([tempdir filesep 'fig1_b.tif'], '-dtiffn', '-r1200')
% saveas(h(3), [tempdir filesep 'fig1_b.tif'], 'tif');
close(h(3))


model_out = model;

end