function [ output_args ] = script_figure2_Hela3D
% Generates Figure 3 
% from G. R. Johnson, T. E. Buck,D. P. Sullivan, G. K. Rohde, and 
% R. F. Murphy (2015) Joint modeling of cell and nuclear shape variation. 
% Molecular Biology of the Cell,
% http://murphylab.web.cmu.edu/software/2015_MBoC_Cell_And_Nuclear_Shape/index.html
%
% Author: Gregory R. Johnson (gj@cs.cmu.edu)
% Created: Nov 11, 2014
% Nov 2, 2015 - added header block - gj@cs.cmu.edu

% Copyright (C) 2015 Murphy Lab
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

%Partially adapated from SSPaper_figure2 by Devin Sullivan
%Gregory Johnson, 11/20/14


parentdirs = {'/Volumes/Data/Documents/Murphylab/projects/shapeSpace/', ...
               '/projects/cellorganizer/gj/shapeSpace/', ...
               '/Users/Greg/Documents/murphylab/projects/shapeSpace/', ...
               '/Users/admin/shapeSpace/'};
% parentdirs = {};
   
for i = 1:length(parentdirs)
    if exist(parentdirs{i}, 'dir')
        parentdir = parentdirs{i};
        break
    end
end

addpath(genpath([parentdir filesep 'code']));

expdir = [parentdir filesep 'diffeo_tub'];
addpath(genpath([expdir filesep 'code']));

diffeodir = [expdir filesep 'diffeomorphic_newesg'];

resdir = [diffeodir filesep 'results'];
if ~exist(resdir, 'dir')
    mkdir(resdir)
end

savedir = [resdir filesep mfilename];
if ~exist(savedir, 'dir')
    mkdir(savedir)
end


load([diffeodir filesep 'cellModel.mat']);
embedfile = [savedir filesep 'embedfile.mat'];

if ~exist(embedfile, 'file')
    dists_incomplete = model.cellShapeModel.distances_incomplete;
    [keepinds] = get_complete_distance_matrix(dists_incomplete);
    
    param.replicates = 10;
    param.embedding_dims = 7;
    [dim_struct_euc, dim_struct_orig] = compute_embedding_dimensionality(dists_incomplete(keepinds, keepinds), param);
    
    pos = nan(size(dists_incomplete,1), size(dim_struct_orig.pos,2));
    pos(keepinds,:) = dim_struct_orig.pos;
    
    embedding_options.positions = pos;
    
    [ embed_model ] = embed_distance_matrix( dists_incomplete, embedding_options );
    
    save(embedfile, 'embed_model', 'dim_struct_euc', 'dim_struct_orig', 'keepinds')
else
    load(embedfile)
end
model.cellShapeModel = ml_initparam(embed_model, model.cellShapeModel);
model.nuclearShapeModel = model.cellShapeModel;

shapespacefile_white = [savedir filesep 'shapeSpace_white.tif'];
shapespacefile_black = [savedir filesep 'shapeSpace_black.tif'];

if ~exist(shapespacefile_white, 'file')
    try
        fig_param.imfunc = @(x) im2projection_RGB(model.cellShapeModel.imfunc(x), struct('justz', true))*1.5;
        fig_param.subsize = 900;
        figure, showShapeSpaceFigure(model, [],fig_param)
        xlabel('MDS 1')
        ylabel('MDS 2')
        saveas(gcf, shapespacefile_white, 'tif')
        
        %ghetto image recoloring
        img = ml_readimage(shapespacefile_white);
        whtpix = all(img == 255, 3);
        blkpix = all(img == 0,3);
        
        img(repmat(whtpix,[1,1,3])) = 0;
        img(repmat(blkpix,[1,1,3])) = 255;
        
        imwrite(img, shapespacefile_black, 'tif')
       
        close(gcf);
        
        
    catch
    end
end

pos = model.cellShapeModel.positions;
tes = model.cellShapeModel.tessellation;


minpos_orig = min(model.cellShapeModel.positions);
maxpos_orig = max(model.cellShapeModel.positions);

nframes = 5;
ndims = size(model.cellShapeModel.positions,2);

posfile = [savedir filesep 'posfile.mat'];

if ~exist(posfile, 'file')

    for i = 1:ndims
        position = zeros(1, ndims);
        position(i) = maxpos_orig(i);

        position_highbound = position;
        position_lowbound = zeros(1, ndims);

        while true

            if isnan(tsearchn(pos,tes,position))
                position_highbound = position;
                position = position - (position-position_lowbound)./2;
            else
                if norm(position_highbound - position_lowbound) < 0.01;
                    maxpos(i) = position(i);
                    break
                end

                position_lowbound = position;
                position = position + (position_highbound - position)./2;  
            end
        end

        position = zeros(1,ndims);
        position(i) = minpos_orig(i);

        position_lowbound = position;
        position_highbound = zeros(1,ndims);

        while true
            if isnan(tsearchn(pos,tes,position))
                position_lowbound = position;
                position = position - (position-position_highbound)./2;
            else
                if norm(position_highbound - position_lowbound) < 0.01;
                    minpos(i) = position(i);
                    break
                end

                position_highbound = position;
                position = position + (position_lowbound - position)./2;
            end
        end
    end

    maxpos = min(abs([maxpos;minpos]),[],1);
    minpos = -maxpos;

    save(posfile, 'minpos', 'maxpos')
else
    load(posfile)
end
% 
% imfunc = @(x) padarray(model.cellShapeModel.imfunc(x), [0,0,3], 'both');
% model.cellShapeModel.imfunc = imfunc;
% model.nuclearShapeModel.imfunc = imfunc;

% model.nuclearShapeModel.convolution_method = 'convnfft';
% model.cellShapeModel.convolution_method = 'convnfft';
% param.model.diffeomorphic.convolution_method = 'convnfft';


framedir = [savedir filesep 'frames'];
if ~exist(framedir, 'dir')
    mkdir(framedir)
end

for i = 1:ndims       
    pts = linspace(minpos(i), maxpos(i), nframes);
    for j = 1:nframes
        
        savefile = [framedir filesep 'frame' num2str(i) '_' num2str(j)];
        
        [can_start, final_name, final_exists, temp_name] = chunk_start( savefile);
        
        if can_start && ~final_exists
            disp(['Building ' savefile])
            system(['rm ' savefile filesep 'temp' filesep 'genframe*.tmp']);
            param.position = zeros(1, ndims);
            param.position(i) = pts(j);
            
            param.framefolder = savefile;
            img = model2img({model}, param);
            save(savefile, 'img', 'param')
        end
    end
end


imgs = cell(ndims, nframes);
for i = 1:ndims
    for j = 1:nframes
        savefile = [framedir filesep 'frame' num2str(i) '_' num2str(j) '.mat'];
        
        if exist(savefile, 'file')
            load(savefile)
            imgs{i,j} = img;
        end
        
    end
end

imgsizes = cellfun(@(x) size(x{1}), imgs(~cellfun(@isempty, imgs(:))), 'UniformOutput', false);
maxsize = max(vertcat(imgsizes{:}),[],1);

imflat = cell(ndims, nframes);
imflat_wht = cell(ndims, nframes);
% imrows{i} = [imflat{i,:}];


for i = 1:numel(imgs)
    if isempty(imgs{i})
        imgs{i} = {zeros(maxsize), zeros(maxsize)};
    else
        imgs{i}{1} = padarray(imgs{i}{1}, maxsize - size(imgs{i}{1}), 'post');
        imgs{i}{2} = padarray(imgs{i}{2}, maxsize - size(imgs{i}{2}), 'post');
    end
    imflat{i} = im2projection_RGB(ml_findmainobj(imgs{i}{1}*2 + imgs{i}{2}), struct('justz', true));
    imflat_wht{i} = im2projection_RGB(ml_findmainobj(imgs{i}{1}*2 + imgs{i}{2}), struct('bg', [1,1,1], 'justz', true));
end

for i = 1:numel(imflat)
    [~, croprange(i,:)] = cropImg(imflat{i}, 2);
end
croprange = [min(croprange(:,1)), max(croprange(:,2)), min(croprange(:,3)), max(croprange(:,4))];

for i = 1:numel(imflat)
    imflat{i} = imflat{i}(croprange(1):croprange(2), croprange(3):croprange(4),:);
    imflat_wht{i} = imflat_wht{i}(croprange(1):croprange(2), croprange(3):croprange(4),:);
end


for i = 1:size(imflat,1)
    immerge{i} = [imflat{i,:}];
    immerge_white{i} = [imflat_wht{i,:}];
end

imout = vertcat(immerge{:});
imout_wht = vertcat(immerge_white{:});

imwrite(imout, [savedir filesep 'imgs.tif'], 'tif')
imwrite(imout_wht, [savedir filesep 'imgs_white.tif'], 'tif')

save([savedir filesep 'imgs.mat'])


%%%SUP FIG 2%%%
%generate random walk movie within shape space and show it on the shape
%space
%Here we show the shape space as 2D summations in grey and generated images
%in colors with warmner colors corresponding to later times

%%%END SUP FIG 2%%%

end

