function script_figureS1(parentdir)
    
    cd(parentdir);
    
    figdir = [parentdir filesep 'figures'];
    
    if ~exist(figdir, 'dir')
        mkdir(figdir)
    end

    generate_options = struct;
    % generate_options.number_images = 10;
    % generate_options.number_images = 16;
    generate_options.number_images = 15;
    
    generate_options.minimum_relative_semidiameter = 1 / 4;
    generate_options.maximum_relative_semidiameter = 2 / 3;
    generate_options.generate_cycle = true;
    generate_options.image_width = 64;
    [superellipse_shapes, superellipse_shape_parameters] = generate_superellipse_shape_set(generate_options);

    superellipse_shape_parameters = zscore(superellipse_shape_parameters);
    
    dimensionality = '3D';
    param.model.name = '';
    param.model.id = 'num2str(now)';
    param.model.resolution = [1, 1, 1];
   
    param.nucleus.type = 'diffeomorphic';
    param.cell.type = 'diffeomorphic';
    param.train.flag = 'framework';
    param.verbose = true;
    param.debug = true;
    param.model.downsampling = [1,1,1];
    param.preprocess = true;

    param.model.diffeomorphic.embedding_type = 'most_complete';

    param.model.diffeomorphic.convergence_absolute_error_difference = 1;
    param.model.diffeomorphic.convergence_absolute_error = 1;

    param.model.diffeomorphic.number_windows = 1;
    param.model.diffeomorphic.maximum_deformation_per_step = ones(1,3) * 0.25;
    
    param.saveIntermediateResults = false;
    
    param.preprocess = false;
    
    for i = 1:length(superellipse_shapes)
        im_dna{i} = @()superellipse_shapes{i} > 0;
        param.model.regfunc = @(x,y) deal(feval(im_dna{x}), []);
    end
    
    param.model.diffeomorphic.embedding_type = 'most_complete';
    tempparent = [parentdir filesep 'temp_param_check'];
    param.tempparent = tempparent;
     
    filtersizes = [0.1, 0.25, 1];
    def_per_step = [0.05, 0.1, 0.25, 0.5, 1];
    
    [filtersizes, def_per_step] = meshgrid(filtersizes, def_per_step);
    
    models = cell(size(filtersizes));
    
    for i = 1:length(filtersizes(:))
        
        modelsuffix = [num2str(filtersizes(i)) '_' num2str(def_per_step(i))];
        tempdir = [param.tempparent filesep 'diffeomorphic' modelsuffix];
        param.model.diffeomorphic.tempdir = tempdir;
        param.model.filename = [tempdir filesep 'superellipseModel.mat'];

        param.model.diffeomorphic.window_radius = generate_options.image_width;
        param.model.diffeomorphic.filter_radius = round(generate_options.image_width * filtersizes(i));
        param.model.diffeomorphic.maximum_deformation_per_step = ones(1,3) * def_per_step(i);
        
        if ~exist(param.model.filename, 'file')
            img2slml(dimensionality, im_dna, [], [], param);
        end
    end
    
    for i = 1:length(filtersizes(:))
         modelsuffix = [num2str(filtersizes(i)) '_' num2str(def_per_step(i))];
        tempdir = [param.tempparent filesep 'diffeomorphic' modelsuffix];
        param.model.diffeomorphic.tempdir = tempdir;
        param.model.filename = [tempdir filesep 'superellipseModel.mat'];
        
        load(param.model.filename);
        models{i} = model;
    end
    
    model_temp = models{1};
    param.subsize = 20;
    model_temp.cellShapeModel.positions = superellipse_shape_parameters;
    figure, showShapeSpaceFigure(model_temp, [], param)
    saveas(gcf, [figdir filesep 'ssf_superellipse_params.png'] , 'png');
    close(gcf);
    
    mses = zeros(size(models));
    
    param.rebuild_pos = true;
    
    for i = 1:length(models(:));
        param.rebuild_pos = true;
        param.subsize = 400;
        
        figure,
        embed_final = showShapeSpaceFigure(models{i},[], param);
        saveas(gcf, [figdir filesep 'ssf_' num2str(filtersizes(i)) '_' num2str(def_per_step(i)) '.png'] , 'png');
        close(gcf);
        
        [mses(i), ~, pos_align] = procrustes_mse(superellipse_shape_parameters, embed_final);
        
        
        param.rebuild_pos = false;
        param.subsize = 400;
        model = models{i};
        model.cellShapeModel.positions = pos_align;
        
        figure,
        embed_final = showShapeSpaceFigure(model,[], param);
        saveas(gcf, [figdir filesep 'ssf_' num2str(filtersizes(i)) '_' num2str(def_per_step(i)) '_align.png'] , 'png');
        close(gcf);
 
    end
    
    [~, ind] = min(mses(:));
    disp(['filter size: ' num2str(filtersizes(ind))]);
    disp(['def per step: ' num2str(def_per_step(ind))]);
    
    
    
end

