function [ output_args ] = script_kde_trans_model(skipsynth)
%SCRIPT_KDE_TRANS_MODEL Summary of this function goes here
%   Detailed explanation goes here

if ~exist('skipsynth', 'var')
    skipsynth = false;
end

addpath(genpath('/projects/cellorganizer/gj/shapeSpace/c2c12/code'))

script_img2diffeomorphic_newseg_setup


figuredir = [projectdir filesep 'figures'];
if ~exist(figuredir, 'dir')
    mkdir(figuredir)
end

tempdir = [figuredir filesep mfilename];
if ~exist(tempdir, 'dir')
    mkdir(tempdir)
end

framedir = [tempdir filesep 'frames'];
if ~exist(framedir, 'dir')
    mkdir(framedir)
end

kerneldir = [tempdir filesep 'kernel'];
if ~exist(kerneldir, 'dir')
    mkdir(kerneldir)
end


load([savedir filesep 'cellModel.mat'])


load([figuredir '/script_c2c12_ss_imgs/embedding_1.mat'])


% 
% model.cellShapeModel = ml_initparam(embedding, model.cellShapeModel);
% % model.cellShapeModel.positions = embedding.positions;


rng(0);

embeddingfile = [tempdir filesep 'embedding.mat'];

if ~exist(embeddingfile, 'file')
    [~,~,stats] = manova1(embedding.positions, fractlife);

    pos = stats.canon;
    % pos = model.cellShapeModel.positions;

    keepinds = ~any(isnan(pos),2);

    embedding_model.positions               = pos;

    embedding_model.convex_hull             = convhulln( pos(keepinds,:)); %result.convex_hull;
    embedding_model.tessellation            = delaunayn( pos(keepinds,:)); %result.tes;

    keepinds_temp = find(keepinds);

    embedding_model.convex_hull             = keepinds_temp(embedding_model.convex_hull);
    embedding_model.tessellation            = keepinds_temp(embedding_model.tessellation);

    save(embeddingfile, 'embedding_model')
else
    load(embeddingfile)
end

model.cellShapeModel = ml_initparam(embedding_model, model.cellShapeModel);

pos = model.cellShapeModel.positions;

pos0 = nan(size(pos));
pos1 = nan(size(pos));
tp = nan(size(pos,1),1);

for i = 1:(size(pos,1)-1)
    if cellid(i) == cellid(i+1) && ~any(isnan(pos(i+1,:))) && ~any(isnan(pos(i,:)))
        pos0(i,:) = pos(i,:);
        pos1(i,:) = pos(i+1,:);
        tp(i) = fractlife(i);
        cellids(i) = cellid(i);
    end
end

keepinds = ~any(isnan(pos0),2);

pos0 = pos0(keepinds,:);
pos1 = pos1(keepinds,:);

cellids = cellids(keepinds);

dmat_embed = squareform(pdist(pos0));

kernelfile = [tempdir filesep 'kernel.mat'];
if ~exist(kernelfile, 'file')    

    [dpos_grid] = 10.^(-4:0.05:2);


    fvals = nan(1, length(dpos_grid));

    for i = 1:length(dpos_grid(:))

        kernelfile_search = [kerneldir filesep 'k_' num2str(i)];

        if ~exist([kernelfile_search, '.mat'], 'file')
            [can_start, ~, final_exists, temp_name] = chunk_start( kernelfile_search, '.mat');
            if can_start & ~final_exists
                fval = opt_func(pos0, pos1, dmat_embed, [dpos_grid(i)], cellids);
                fvals(i) = fval;
                save(kernelfile_search, 'fval')
            end

        else
            load(kernelfile_search)
            fvals(i) = fval;
        end
    end

    if any(isnan(fvals))
        return;
    end

    figure, 
    plot(log10(dpos_grid), fvals);
    
    saveas(gcf, [tempdir filesep 'kernel_error.tif'], 'tif')
    close(gcf);
    
    [~, ind] = min(fvals);
    
    
    kernelfile 

    disp('Computing kernel');
    [ksize, fval, exitflag] = fminsearch(@(x) opt_func(pos0, pos1, dmat_embed, x, cellids), dpos_grid(ind), optimset('Display','iter', 'MaxIter', 2000));
    save(kernelfile, 'ksize', 'fval', 'exitflag')
else
    load(kernelfile)
end 

% ksize = [ksize(1), 0;0, ksize(2)];



vecfieldimg = [tempdir filesep 'vecfield.tif'];
if ~exist(vecfieldimg, 'file')
    [xpos,ypos] = meshgrid([-2:0.1:2], [-2:0.1:2]);


    coords = [xpos(:), ypos(:), zeros(length(xpos(:)), size(pos,2)-2)];
    coords1 = zeros(size(coords));



    for i = 1:length(coords)
        w = mvnpdf([pdist2(coords(i,:), pos0)'], 0, ksize);
        w = w./sum(w(:));
        coords1(i,:) = pos1'*w;
    end

    delta = coords1 - coords;

    d = zeros(size(xpos));
    d(:) = delta(:,3);

    figure('color', 'w')
    [~,h] = contourf(xpos, ypos, d, 1000);
    caxis([-1.25 1.25]) %this is determined after plotting all the figures and looking at the max and min
    set(h,'LineStyle','none');
    hold on
    quiver(coords(:,1), coords(:,2), delta(:,1), delta(:,2), 2, 'k')
    colorbar
    xlabel('MDS 1')
    ylabel('MDS 2')

    axis equal
    axis tight

    saveas(gcf, vecfieldimg, 'tif')
end


[ucellid, first_occur] = unique(cellid);

nframes = histc(cellid, ucellid);

order = randperm(length(nframes));

numsyntracks = 20;


startpos = pos(first_occur,:);

mpos = mean(startpos);
cpos = cov(startpos);

model.nuclearShapeModel = model.cellShapeModel

% hold on, scatter(coords(delta(:,3) < 0,1), coords(delta(:,3) < 0,2), abs(delta(delta(:,3) < 0,3))*1000, '.b')
% hold on, scatter(coords(delta(:,3) > 0,1), coords(delta(:,3) > 0,2), abs(delta(delta(:,3) > 0,3))*1000, '.r')
% axis equal
% axis tight
if ~skipsynth

    for i = 1:numsyntracks

        rng(i);
        
        trackfile = [tempdir filesep 'syntracks' num2str(i)];


        if ~exist([trackfile '.mat'], 'file')
            [can_start, ~, final_exists, temp_name] = chunk_start( trackfile, '.mat');

            if can_start & ~final_exists

                disp(['Generating tracks: ' trackfile]);

                totframes = nframes(order(i));

                fpos = nan(totframes,size(pos0,2));
                fpos(1,:) = mvnrnd(mpos, cpos, 1);
                a = nan;
                while isnan(a)
                    fpos(1,:) = mvnrnd(mpos, cpos, 1);
                    a = tsearchn(model.cellShapeModel.positions,model.cellShapeModel.tessellation,fpos(1,:));
                end

                for j = 2:totframes
                    disp([num2str(j) filesep num2str(totframes)]);
                    %compute the weights
                    d_dist = pdist2(fpos(j-1,:), pos0)';
                    

        %             t_dist(t_dist > 0.5) = t_dist(t_dist > 0.5)-1;
        %             t_dist(t_dist < -0.5) = t_dist(t_dist < -0.5)+1;
        %     

                    w = mvnpdf(d_dist,0, ksize);
                    w = w./sum(w);

                    %average displacement
                    m_disp = (pos1'*w)';

                    residuals = pos1 - repmat(m_disp, [size(pos1,1),1]);
                    % Weighted Covariance Matrix
                    cov_disp = residuals' * (residuals .* repmat(w, 1, size(residuals,2))); 
                    cov_disp = 0.5 * (cov_disp + cov_disp');     

                    fpos(j,:) = m_disp;

                     a = nan;  
                    while isnan(a)
                        fpos(j,:) = mvnrnd(m_disp, cov_disp);
                        a = tsearchn(model.cellShapeModel.positions,model.cellShapeModel.tessellation,fpos(j,:));
                    end

        %             h = plot_gaussian_ellipsoid(m_disp(1:2), cov_disp(1:2, 1:2)/10)
        %             set(h, 'color', a(j,:))
                end
                save(trackfile, 'fpos')
                chunk_finish(temp_name)
            end
        end

    end



    for i = 1:numsyntracks
        
        trackfile = [tempdir filesep 'syntracks' num2str(i) '.mat'];
        
        if exist(trackfile, 'file')
            load(trackfile)
            
            for j = 1:size(fpos,1)
                framefile = [framedir filesep 'f' num2str(i) '_' num2str(j)];

                if ~exist([framefile '.mat'], 'file')
                    [can_start, ~, final_exists] = chunk_start( framefile);

                    if can_start & ~final_exists
                        disp(framefile)
                        
                        param.position = fpos(j,:);
                        param.framefolder = [framedir filesep 'frame' num2str(i) '_' num2str(j)];

                        system(['rm ' param.framefolder filesep 'temp' filesep '*.tmp'])

                        img = model2img({model}, param);

                        save(framefile, 'img', 'param')
                    end
                end
            end
        end
    end
end

imsize = size(model.cellShapeModel.imfunc(1));
missingimgs = cell(numsyntracks,1);


moviedir = [tempdir filesep 'movies' filesep];

if ~exist(moviedir, 'dir')
    mkdir(moviedir)
end


for i = 1:numsyntracks
    trackfile = [tempdir filesep 'syntracks' num2str(i) '.mat'];
    if exist(trackfile, 'file')
        load(trackfile)
        missingimgs{i} = false(1, size(fpos,1));

        imgs = cell(1, size(fpos,1));
        for j = 1:size(fpos,1)
            framefile = [framedir filesep 'f' num2str(i) '_' num2str(j) '.mat'];

            if exist(framefile, 'file')
                load(framefile)
                if ~isempty(img)
                    imgs{j} = ml_findmainobj(double((img{1}+img{2})>0));
                else
                    imgs{j} = zeros(imsize);
                    missingimgs{i}(j) = true;
                end       
            else
                imgs{j} = zeros(imsize);
                missingimgs{i}(j) = true;
            end
        end

        try
           imgs2movie(imgs, [moviedir filesep 'movie_' num2str(i) '.avi'], 10)
        catch
        end
    end
end

for i = 1:numsyntracks
   
end


end

function [diff, pred_pos1] = opt_func(pos0,pos1,dmat,h, cv)


if ~exist('dmat', 'var') | isempty(dmat)
    dmat = squareform(pdist(pos0));
end

if ~exist('cv', 'var') | isempty(cv)
    cv = true;
end
    

%if not positive definite, the kernel is no good
if h <= 0
    diff = inf;
    pred_pos1 = inf;
    return
end

pred_pos1 = inf(size(pos0));

for i = 1:size(pos0,1)

    
%     t_dist(t_dist > 0.5) = t_dist(t_dist > 0.5)-1;
%     t_dist(t_dist < -0.5) = t_dist(t_dist < -0.5)+1;
%     
    ddist = dmat(i,:)';

    w = mvnpdf(ddist, 0, h);

    if isnumeric(cv)
        w(cv == cv(i)) = 0;
    elseif cv
        w(i) = 0;
    end
    
    if all(w == 0)
        w(:) = 1;
    end
    
    w = w./sum(w(:));


    pred_pos1(i,:) = pos1'*w;

    
end

diff = sum(sum((pos1-pred_pos1).^2,2),1);

end