function [k_cell, pred_err_nuc, k_nuc, pred_err_cell, k_cell_perm, pred_err_nuc_perm, k_nuc_perm, pred_err_cell_perm,cellinds] = test_cell_nuc_dep(p_cell, p_nuc, cvinds, niter, savedir)

if ~exist('cvinds', 'var') | isempty(cvinds)
    cvinds = 1:size(p_cell,1);
end

if ~exist(savedir, 'dir')
    mkdir(savedir)
end

if ~exist('niter', 'var') | isempty(niter)
    niter = 500;
end

cellinds = find((~any(isnan(p_cell),2))&(~any(isnan(p_nuc),2)));

ndat = length(cellinds);

p_cell = p_cell(cellinds,:);
p_nuc = p_nuc(cellinds,:);

k_c2n_save = [savedir filesep 'k_c2n.mat'];
k_n2c_save = [savedir filesep 'k_n2c.mat'];

[k_cell, pred_err_nuc] = find_kernel(p_cell, p_nuc, cvinds, k_c2n_save, [savedir filesep 'kcell2nuc.png']);
[k_nuc, pred_err_cell] = find_kernel(p_nuc, p_cell, cvinds, k_n2c_save, [savedir filesep 'knuc2cell.png']);

rng(0)


iterdir = [savedir filesep 'iterdir'];
if ~exist(iterdir, 'dir')
    mkdir(iterdir)
end

k_cell_perm = nan(1, niter);
k_nuc_perm = nan(1,niter);

pred_err_nuc_perm = cell(1, niter);
pred_err_cell_perm = cell(1,niter);

for i = 1:niter
%     disp([num2str(i) filesep num2str(niter)])
    k_c2n_save = [iterdir filesep 'k_c2n' num2str(i) '.mat'];
    k_n2c_save = [iterdir filesep 'k_n2c' num2str(i) '.mat'];

    rng(i)
    perm = randperm(ndat);

    [k_cell_perm(i), pred_err_nuc_perm{i}] = find_kernel(p_cell, p_nuc(perm,:), cvinds, k_c2n_save);
    [k_nuc_perm(i), pred_err_cell_perm{i}] = find_kernel(p_nuc(perm,:), p_cell, cvinds(perm), k_n2c_save);

end

if isnan(any(k_nuc_perm(:)))
    return
end

pval_cell_all = mean(k_cell >= k_cell_perm);
pval_nuc_all = mean(k_nuc >= k_nuc_perm);

disp(['p cell: ' num2str(pval_cell_all)]);
disp(['p nuc: ' num2str(pval_nuc_all)]);

pval_nucs = mean(repmat(pred_err_nuc, [1, niter]) >= [pred_err_nuc_perm{:}],2);
pval_cells = mean(repmat(pred_err_cell, [1, niter]) >= [pred_err_cell_perm{:}],2);


% [~, p_nuc_align] = procrustes(p_cell(:,1:2), p_nuc(:,1:2));

% [~,~,stats_n] = manova1(p_nuc, pval_nucs);
% [~,~,stats_c] = manova1(p_cell, pval_cells);
% 
% positions = zeros(size(model.cellShapeModel.positions,1),2);
% p_nuc_tmp = positions;
% p_nuc_tmp(cellinds,:) = p_nuc_align;
% 
% p_cell_tmp = positions;
% p_cell_tmp(cellinds,:) = p_cell(:,1:2);
% 
% nucmodel.cellShapeModel.positions = p_nuc_tmp;
% cellmodel.cellShapeModel.positions = p_cell_tmp;
% 
% param.synthorder = cellinds;
% param.subsize = 800;
% 
% figure('color', 'white'),
% hold on
% 
% showShapeSpaceFigure(cellmodel,[], param)
% showShapeSpaceFigure(nucmodel,[], param);
% 
% 
% for i = 1:size(p_nuc_align,1)
%     plot([p_nuc_align(i,1), p_cell(i,1)], ...
%             [p_nuc_align(i,2), p_cell(i,2)])
% end

imgfile = [savedir filesep 'cell_ksize.tif'];
if ~exist(imgfile, 'file')
    figure('color', 'w')
    plot(sort(log10(k_cell_perm)))
    hold on
    plot([0,500], log10([k_cell, k_cell]), 'r--')
    xlabel('permutations')
    ylabel('log_1_0(h)')
    title('h-size E(nuc|cell)')
    saveas(gcf, imgfile, 'tif')
    close(gcf)
end

imgfile = [savedir filesep 'nuc_ksize.tif'];
if ~exist(imgfile, 'file')
    figure('color', 'w')
    plot(sort(log10(k_nuc_perm)))
    hold on
    plot([0,500], log10([k_nuc, k_nuc]), 'r--')
    xlabel('permutations')
    ylabel('log_1_0(h)')
    title('h-size E(cell|nuc)')
    saveas(gcf, imgfile, 'tif')
    close(gcf)
end

nuc_err_pred = mean([pred_err_nuc_perm{:}],1);
nuc_err = mean(pred_err_nuc);

imgfile = [savedir filesep 'nuc_err_mse.tif'];
if ~exist(imgfile, 'file')
    figure('color', 'w')
    plot(sort((nuc_err_pred)))
    hold on
    plot([0,500], ([nuc_err, nuc_err]), 'r--')
    xlabel('permutations')
    ylabel('MSE')
    title('MSE E(nuc|cell)')
    saveas(gcf, imgfile, 'tif')
    close(gcf)
end

cell_err_pred = mean([pred_err_cell_perm{:}],1);
cell_err = mean(pred_err_cell);

imgfile = [savedir filesep 'cell_err_mse.tif'];
if ~exist(imgfile, 'file')
    figure('color', 'w')
    plot(sort((cell_err_pred)))
    hold on
    plot([0,500], ([cell_err, cell_err]), 'r--')
    xlabel('permutations')
    ylabel('MSE')
    title('MSE E(cell|nuc)')
    saveas(gcf, imgfile, 'tif')
    close(gcf)
end


end

function [k_best, pred_err_sq] = find_kernel(p1,p2, cvinds, kernelfile, savefigurepath)
    if ~exist('savefigurepath', 'var') | isempty(savefigurepath)
        plot_err = false;
    else
        plot_err = true;
    end


    [kernel_grid] = 10.^(-5:0.05:2);
    dmat_embed = squareform(pdist(p1));
    
    if ~exist(kernelfile, 'file')    
        
        fvals = nan(1, length(kernel_grid));
        
        [can_start, ~, final_exists, temp_name] = chunk_start( kernelfile(1:end-4), '.mat');
        if can_start && ~final_exists

            for i = 1:length(kernel_grid(:))
                
                fval = opt_func(p1, p2, dmat_embed, [kernel_grid(i)], cvinds);
                fvals(i) = fval;
                
            end

            if any(isnan(fvals))
                return;
            end

            [~, ind] = min(fvals);

            disp('Computing kernel');
            [k_best, fval, exitflag] = fminsearch(@(x) opt_func(p1, p2, dmat_embed, x, cvinds), kernel_grid(ind), optimset('Display','iter', 'MaxIter', 2000));

            [~, pred_pos] = opt_func(p1, p2, dmat_embed, k_best);

            pred_err_sq = sum((p2 - pred_pos).^2,2);

            save(kernelfile, 'k_best', 'fval', 'exitflag', 'fvals', 'kernel_grid', 'pred_pos', 'pred_err_sq')
            
            chunk_finish(temp_name)
        else
            k_best = nan;
            pred_err_sq = nan;
        end
    else
        load(kernelfile)
    end 

    if plot_err
        plot_kernel_error(savefigurepath, fvals, kernel_grid)
    end
    
    
    
end

function plot_kernel_error(savename, fvals, kernel_grid)
    
    if ~exist(savename, 'file')
        
        figure('color', 'w'), 
        hold on

        plot(log10(kernel_grid), fvals)

        xlabel('log10(kernel size)')
        ylabel('HOO MSE')

        saveas(gcf, savename, 'png')
        close(gcf)
    end
end

function [diff, pred_pos1] = opt_func(pos0,pos1,dmat,h, cv)


if ~exist('dmat', 'var') | isempty(dmat)
    dmat = squareform(pdist(pos0));
end

if ~exist('cv', 'var') | isempty(cv)
    cv = true;
end
    

%if not positive definite, the kernel is no good
if h <= 0
    diff = inf;
    pred_pos1 = inf;
    return
end

pred_pos1 = inf(size(pos1));

for i = 1:size(pos0,1)

    
%     t_dist(t_dist > 0.5) = t_dist(t_dist > 0.5)-1;
%     t_dist(t_dist < -0.5) = t_dist(t_dist < -0.5)+1;
%     
    ddist = dmat(i,:)';

    w = mvnpdf(ddist, 0, h);

    if isnumeric(cv)
        w(cv == cv(i)) = 0;
    elseif cv
        w(i) = 0;
    end
    
    if all(w == 0)
        w(:) = 1;
    end
    
    w = w./sum(w(:));


    pred_pos1(i,:) = pos1'*w;

    
end

diff = sum(sum((pos1-pred_pos1).^2,2),1);

end