function [pval_nuc, pval_nuc_all, pval_cell, pval_cell_all] = test_cell_nuc_pred(p_cell, p_nuc, cvinds, niter, savedir)
%does nested hold one out cross validation

if ~exist('cvinds', 'var')
    cvinds = 1:size(p_cell,1);
end

if ~exist(savedir, 'dir')
    mkdir(savedir)
end

if ~exist('niter', 'var') | isempty(niter)
    niter = 500;
end

preddir = [savedir filesep 'pred'];
if ~exist('preddir', 'dir')
    mkdir(preddir)
end

kernel_grid = 10.^(-5:0.05:2);

cellinds = find((~any(isnan(p_cell),2))&(~any(isnan(p_nuc),2)));

ndat = length(cellinds);

p_cell = p_cell(cellinds,:);
p_nuc = p_nuc(cellinds,:);

dmat_cell = squareform(pdist(p_cell));
dmat_nuc = squareform(pdist(p_nuc));

[ucvinds, ~, cvuid] = unique(cvinds);
cvinds_per_cv = cell(1, length(ucvinds));

for i = 1:length(ucvinds)
    cvinds_dmat = ucvinds(i) == cvinds;
    dmat_cell(cvinds_dmat, cvinds_dmat) = inf;
    dmat_nuc(cvinds_dmat, cvinds_dmat) = inf;
    cvinds_per_cv{i} = find(cvinds_dmat);
end

pred_nuc = nan(size(p_nuc));
pred_cell = nan(size(p_cell));

pred_save_file = [savedir filesep 'pred_all.mat'];

if ~exist(pred_save_file, 'file')

    for i = 1:ndat
        k_save = [preddir filesep 'k_' num2str(i) '.mat'];
        k_save_tmp = [k_save '.tmp'];
        if ~exist(k_save, 'file') && ~exist(k_save_tmp, 'file')
            system(['touch ' k_save_tmp]);

            disp(['pred:' num2str(i) filesep num2str(ndat)])

            traininds = 1:ndat;
            traininds(i) = [];

            k_cell = find_kernel(dmat_cell(traininds, traininds), p_nuc(traininds,:), kernel_grid);
            k_nuc = find_kernel(dmat_nuc(traininds, traininds), p_cell(traininds,:), kernel_grid);

            if ~isnan(k_cell) && ~isnan(k_nuc) 
                pred_nuc = kernel_pred(p_nuc,dmat_cell(i,:),k_cell);
                pred_cell = kernel_pred(p_cell, dmat_nuc(i,:),k_nuc);            
            end

            save(k_save, 'k_cell', 'k_nuc', 'pred_cell', 'pred_nuc')
            
            system(['rm ' k_save_tmp]);
        end
    end
    
    files = dir([preddir filesep 'k_*.mat']);
    
    any_errors = false;
    if length(files) == ndat
        for i = 1:ndat
            k_save = [preddir filesep 'k_' num2str(i) '.mat'];
            try
                load(k_save)
                k_cell_all(i) = k_cell;
                k_nuc_all(i) = k_nuc;
                pred_cell_all(i,:) = pred_cell;
                pred_nuc_all(i,:) = pred_nuc;
            catch
                any_errors = true;
                system(['rm ' k_save])
            end
        end
        
        if ~any_errors
            k_cell = k_cell_all;
            k_nuc = k_nuc_all;
            pred_cell = pred_cell_all;
            pred_nuc = pred_nuc_all;

            save(pred_save_file, 'pred_nuc', 'pred_cell', 'k_nuc', 'k_cell')
        end
    end


end

rng(0)

iterdir = [savedir filesep 'iterdir'];
if ~exist(iterdir, 'dir')
    mkdir(iterdir)
end

k_cell_perm = nan(1, niter);
k_nuc_perm = nan(1,niter);

pred_err_nuc_perm = cell(1, niter);
pred_err_cell_perm = cell(1,niter);

for j = 1:niter
    disp(['iter ' num2str(j) filesep num2str(niter)])
    
    permfile = [savedir filesep 'perm' num2str(j) '.mat'];
        
    permdir = [iterdir filesep 'dat' num2str(j)];
    if ~exist(permdir, 'dir')
        mkdir(permdir)
    end
    
    rng(j);
    
    traininds = 1:ndat;
    shuffle_inds = traininds(randperm(length(traininds)));
    
    if ~exist(permfile, 'file')
        

        for i = 1:ndat
            pred_nuc = nan(ndat, size(p_nuc,2));
            pred_cell = nan(ndat, size(p_cell,2));
        
            
            k_save = [permdir filesep 'k_' num2str(i) '_' num2str(j) '.mat'];
            k_save_tmp = [k_save '.tmp'];
            if ~exist(k_save, 'file') && ~exist(k_save_tmp, 'file')
                system(['touch ' k_save_tmp]);
                
                disp([num2str(i) filesep num2str(ndat)])

                traininds_tmp = traininds;
                traininds_tmp(i) = [];
                shuffle_inds_tmp = shuffle_inds;
                shuffle_inds_tmp(i) = [];

                k_cell_perm = find_kernel(dmat_cell(traininds_tmp, traininds_tmp), p_nuc(shuffle_inds_tmp,:), kernel_grid);
                k_nuc_perm = find_kernel(dmat_nuc(traininds_tmp, traininds_tmp), p_cell(shuffle_inds_tmp,:), kernel_grid);

                if ~isnan(k_cell_perm) && ~isnan(k_nuc_perm) 
                    pred_nuc = kernel_pred(p_nuc(shuffle_inds_tmp,:), dmat_cell(i,traininds_tmp),k_cell_perm);    
                    pred_cell = kernel_pred(p_cell(shuffle_inds_tmp,:), dmat_nuc(i,traininds_tmp),k_nuc_perm);
                end
                
                 save(k_save, 'k_cell_perm', 'k_nuc_perm', 'pred_cell', 'pred_nuc')
            
                 system(['rm ' k_save_tmp]);
            end
        end
        
        
        files = dir([permdir filesep 'k_*.mat']);
        
        any_errors = false;
        if length(files) == ndat
            disp(['Saving files for this iteration'])
            for i = 1:ndat
                try
                    k_save = [permdir filesep 'k_' num2str(i) '_' num2str(j) '.mat'];
                    load(k_save)
                    k_cell_all(i) = k_cell_perm;
                    k_nuc_all(i) = k_nuc_perm;
                    pred_cell_all(i,:) = pred_cell;
                    pred_nuc_all(i,:) = pred_nuc;
                catch
                    any_errors = true;
                    system(['rm ' k_save])
                end
            end

            if ~any_errors
                k_cell_perm = k_cell_all;
                k_nuc_perm = k_nuc_all;
                pred_cell = pred_cell_all;
                pred_nuc = pred_nuc_all;

                save(permfile, 'pred_nuc', 'pred_cell', 'k_nuc_perm', 'k_cell_perm')
                disp(['Files saved'])
            end
        end
        
    end
end


p_cell_m = mean(p_cell(:));
p_nuc_m = mean(p_nuc(:));

p_cell_std = std(p_cell(:)-p_cell_m(:));
p_nuc_std = std(p_nuc(:)-p_nuc_m(:));

p_cell = (p_cell - p_cell_m)./p_cell_std;
p_nuc = (p_nuc - p_nuc_m)./p_nuc_std;

    
load(pred_save_file)
pred_cell = (pred_cell - p_cell_m)./p_cell_std;
pred_nuc = (pred_nuc - p_nuc_m)./p_nuc_std;

pred_cell_err = sum((p_cell - pred_cell).^2,2);
pred_nuc_err = sum((p_nuc - pred_nuc).^2,2);
    

for i = 1:niter
    permfile = [savedir filesep 'perm' num2str(i) '.mat'];
    try
        load(permfile)

        %do this cause i screwed up saving hte positions, and only the 1:niter
        %coordinates are the correct ones
        pred_cell = pred_cell(1:ndat,:);
        pred_nuc = pred_nuc(1:ndat,:);

        pred_cell = (pred_cell - p_cell_m)./p_cell_std;
        pred_nuc = (pred_nuc - p_nuc_m)./p_nuc_std;

        perm_cell_err(:,i) = sum((p_cell - pred_cell).^2,2);
        perm_nuc_err(:,i) = sum((p_nuc - pred_nuc).^2,2);
    catch
        perm_cell_err(:,i) = nan;
        perm_nuc_err(:,i) = nan;
    end
end


keepinds = ~any(isnan(perm_cell_err) | isnan(perm_nuc_err),2);


pval_cell = nan(ndat, 1);
pval_nuc = nan(ndat,1);

pval_cell(keepinds,:) = mean(repmat(pred_cell_err(keepinds), [1,niter]) >= perm_cell_err(keepinds,:),2);
pval_nuc(keepinds,:) = mean(repmat(pred_nuc_err(keepinds), [1,niter]) >= perm_nuc_err(keepinds,:),2);

pval_cell_all = mean(mean(repmat(pred_cell_err(keepinds), [1,niter]),2) >= mean(perm_cell_err(keepinds,:),2));
pval_nuc_all =  mean(mean(repmat(pred_nuc_err(keepinds), [1,niter]),2) >= mean(perm_nuc_err(keepinds,:),2));



end

function [k_best, pred_err_sq] = find_kernel(dmat_embed,p2, kernel_grid)

    
    fvals = nan(1, length(kernel_grid));


    for i = 1:length(kernel_grid(:))

        fval = opt_func(p2, dmat_embed, kernel_grid(i));
        fvals(i) = fval;

    end

    [~, ind] = min(fvals);

    disp('Computing kernel');
    [k_best, fval, exitflag] = fminsearch(@(x) opt_func(p2, dmat_embed, x), kernel_grid(ind), optimset('Display','off', 'MaxIter', 200));

    [~, pred_pos] = opt_func(p2, dmat_embed, k_best);

    pred_err_sq = sum((p2 - pred_pos).^2,2);


    
end

function plot_kernel_error(savename, fvals, kernel_grid)
    
    if ~exist(savename, 'file')
        
        figure('color', 'w'), 
        hold on

        plot(log10(kernel_grid), fvals)

        xlabel('log10(kernel size)')
        ylabel('HOO MSE')

        saveas(gcf, savename, 'png')
        close(gcf)
    end
end

function [diff, pred_pos] = opt_func(pos,dists,h)

    %if not positive definite, the kernel is no good
    if h <= 0
        diff = inf;
        pred_pos = inf;
        return
    end

    pred_pos = kernel_pred(pos, dists, h);
    
    diff = sum(sum((pos-pred_pos).^2,2),1);

end

function pred_pos = kernel_pred(pos, dists,h)
    w = zeros(size(dists));
    w(:) = mvnpdf(dists(:), 0, h);
    w(all(w == 0, 2),:) = 1;
    w = w./repmat(sum(w,2), [1, size(w,2)]);
    
    pred_pos = sum(repmat(permute(pos, [3,1,2]), [size(dists,1),1,1]).*repmat(w, [1,1,size(pos,2)]),2);
    pred_pos = permute(pred_pos, [1,3,2]);
    
end

