function [pval_nuc, pval_nuc_all, pval_cell, pval_cell_all] = test_cell_nuc_pred_linreg(p_cell, p_nuc, cvinds, niter, savedir)
%does nested hold one out cross validation

if ~exist('cvinds', 'var')
    cvinds = 1:size(p_cell,1);
end

if ~exist(savedir, 'dir')
    mkdir(savedir)
end

if ~exist('niter', 'var') | isempty(niter)
    niter = 500;
end

preddir = [savedir filesep 'pred'];
if ~exist('preddir', 'dir')
    mkdir(preddir)
end

cellinds = find((~any(isnan(p_cell),2))&(~any(isnan(p_nuc),2)));

ndat = length(cellinds);

p_cell = p_cell(cellinds,:);

p_nuc = p_nuc(cellinds,:);


k_cell = cell(ndat, 1);
k_nuc = cell(ndat,1);
pred_nuc = nan(ndat, size(p_nuc, 2));
pred_cell = nan(ndat, size(p_cell, 2));

pred_save_file = [preddir filesep 'pred_all'];

if ~exist([pred_save_file '.mat'], 'file')
    [can_start, pred_save_file, final_exists, temp_name] = chunk_start( pred_save_file);

    if can_start && ~final_exists

        for i = 1:ndat
            disp(['pred:' num2str(i) filesep num2str(ndat)])

            traininds = 1:ndat;
            traininds(cvinds == cvinds(i)) = [];

            k_cell{i}= find_kernel(p_cell(traininds,:), p_nuc(traininds,:));
            k_nuc{i} = find_kernel(p_nuc(traininds,:), p_cell(traininds,:));

            pred_nuc(i,:) = kernel_pred(p_cell(i,:),k_cell{i});
            pred_cell(i,:) = kernel_pred(p_nuc(i,:), k_nuc{i});            

        end
        save(pred_save_file, 'pred_nuc', 'pred_cell', 'k_nuc', 'k_cell')
        
        chunk_finish(temp_name)
    end
end



rng(0)

iterdir = [savedir filesep 'iterdir'];
if ~exist(iterdir, 'dir')
    mkdir(iterdir)
end



for i = 1:niter
    
    disp(num2str(i))
    rng(i)
    
   

    perm_file = [iterdir filesep 'perm' num2str(i)];
    
    p_perm = randperm(ndat);
    
    p_nuc_perm = p_nuc(p_perm,:);
    p_cell_perm = p_cell(p_perm,:);

    if ~exist([perm_file '.mat'], 'file')
        [can_start, perm_file, final_exists, temp_name] = chunk_start( perm_file);

        if can_start && ~final_exists
            pred_nuc_perm = nan(ndat,size(p_nuc,2));
            pred_cell_perm = nan(ndat,size(p_cell,2));

            k_cell_perm = cell(1, niter);
            k_nuc_perm = cell(1,niter);


            
            for j = 1:ndat
                disp(['pred:' num2str(j) filesep num2str(ndat)])

                traininds = 1:ndat;
                traininds(cvinds == cvinds(j)) = [];

                k_cell_perm{j}= find_kernel(p_cell(traininds,:), p_nuc_perm(traininds,:));
                k_nuc_perm{j} = find_kernel(p_nuc(traininds,:), p_cell_perm(traininds,:));

                pred_nuc_perm(j,:) = kernel_pred(p_cell(j,:),k_cell_perm{j});
                pred_cell_perm(j,:) = kernel_pred(p_nuc(j,:), k_nuc_perm{j});            

            end
            save(perm_file, 'pred_nuc_perm', 'pred_cell_perm', 'k_nuc_perm', 'k_cell_perm')

            chunk_finish(temp_name)
        end
    end
    
    
end

load([pred_save_file '.mat'])

nuc_err = sum((p_nuc - pred_nuc).^2,2);
cell_err = sum((p_cell - pred_cell).^2,2);


for i = 1:niter
    load([iterdir filesep 'perm' num2str(i) '.mat']);
    nuc_err_perm = sum((p_nuc - pred_nuc_perm).^2,2);
    cell_err_perm = sum((p_cell - pred_cell_perm).^2,2);
    
    nuc_err_perm_all(i,:) = nuc_err_perm;
    cell_err_perm_all(i,:) = cell_err_perm;

end

pval_nuc = mean(repmat(nuc_err', [niter, 1]) >= nuc_err_perm_all,1);
pval_nuc_all = mean(sum(repmat(nuc_err', [niter, 1]),2) >= sum(nuc_err_perm_all,2));

pval_cell = mean(repmat(cell_err', [niter, 1]) >= cell_err_perm_all,1);
pval_cell_all = mean(sum(repmat(cell_err', [niter, 1]),2) >= sum(cell_err_perm_all,2));


end

function [beta, pred_err_sq] = find_kernel(p1,p2, savefigurepath)
    if ~exist('savefigurepath', 'var') || isempty(savefigurepath)
        plot_err = false;
    else
        plot_err = true;
    end

    [beta, sigma, resid] = mvregress([ones(size(p1,1), 1) p1], p2);
    
    pred_err_sq = sum((resid).^2,2);

    if plot_err
        plot_kernel_error(savefigurepath, fvals, kernel_grid)
    end
    
    
    
end

function plot_kernel_error(savename, fvals, kernel_grid)
    
    if ~exist(savename, 'file')
        
        figure('color', 'w'), 
        hold on

        plot(log10(kernel_grid), fvals)

        xlabel('log10(kernel size)')
        ylabel('HOO MSE')

        saveas(gcf, savename, 'png')
        close(gcf)
    end
end


function pred_pos = kernel_pred(pos, beta)
    pred_pos = [ones(size(pos,1),1) pos] * beta;
end


