function [p_nuc_pred, p_cell_pred, k_nuc, k_cell, save_all_file] = weighted_kernel_pred(p_cell, p_nuc, cvinds,  savedir, load_dat)
%does nested hold one out cross validation

if ~exist('cvinds', 'var')
    cvinds = 1:size(p_cell,1);
end

if ~exist(savedir, 'dir')
    mkdir(savedir)
end

if ~exist('load_dat', 'var')
    load_dat = true;
end

save_all_file = [savedir filesep 'all_dat.mat'];

if ~exist(save_all_file, 'file')

    kernel_grid = 10.^(-5:0.05:2);

    cellinds = find((~any(isnan(p_cell),2))&(~any(isnan(p_nuc),2)));

    ndat = length(cellinds);

    p_cell = p_cell(cellinds,:);
    p_nuc = p_nuc(cellinds,:);

    dmat_cell = squareform(pdist(p_cell));
    dmat_nuc = squareform(pdist(p_nuc));

    [ucvinds, ~, ~] = unique(cvinds);
    cvinds_per_cv = cell(1, length(ucvinds));

    for i = 1:length(ucvinds)
        cvinds_dmat = ucvinds(i) == cvinds;
        dmat_cell(cvinds_dmat, cvinds_dmat) = inf;
        dmat_nuc(cvinds_dmat, cvinds_dmat) = inf;
        cvinds_per_cv{i} = find(cvinds_dmat);
    end

    p_nuc_pred = nan(size(p_nuc));
    p_cell_pred = nan(size(p_cell));



    for i = 1:ndat
        k_save = [savedir filesep 'k_' num2str(i) '.mat'];
        k_save_tmp = [k_save '.tmp'];
        if ~exist(k_save, 'file') && ~exist(k_save_tmp, 'file')
            system(['touch ' k_save_tmp]);

            disp(['pred:' num2str(i) filesep num2str(ndat)])

            traininds = 1:ndat;
            traininds(i) = [];

            k_cell = find_kernel(dmat_cell(traininds, traininds), p_nuc(traininds,:), kernel_grid);
            k_nuc = find_kernel(dmat_nuc(traininds, traininds), p_cell(traininds,:), kernel_grid);

            if ~isnan(k_cell) && ~isnan(k_nuc) 
                p_nuc_pred = kernel_pred(p_nuc,dmat_cell(i,:),k_cell);
                p_cell_pred = kernel_pred(p_cell, dmat_nuc(i,:),k_nuc);            
            end

            save(k_save, 'k_cell', 'k_nuc', 'p_cell_pred', 'p_nuc_pred')
            
            system(['rm ' k_save_tmp]);
        end
    end
    
    files = dir([savedir filesep 'k_*.mat']);
    
    any_errors = false;
    if length(files) == ndat
        for i = 1:ndat
            k_save = [savedir filesep 'k_' num2str(i) '.mat'];
            try
                load(k_save)
                k_cell_all(i) = k_cell;
                k_nuc_all(i) = k_nuc;
                pred_cell_all(i,:) = p_cell_pred;
                pred_nuc_all(i,:) = p_nuc_pred;
            catch
                any_errors = true;
                system(['rm ' k_save])
            end
        end
        
        if ~any_errors
            k_cell = k_cell_all;
            k_nuc = k_nuc_all;
            p_cell_pred = pred_cell_all;
            p_nuc_pred = pred_nuc_all;

            save(save_all_file, 'p_nuc_pred', 'p_cell_pred', 'k_nuc', 'k_cell')
        end
    end
end

if load_dat && exist(save_all_file, 'file')
    load(save_all_file)
else
    p_nuc_pred = [];
    p_cell_pred = [];
    k_nuc = [];
    k_cell = [];
end

end

function [k_best, pred_err_sq] = find_kernel(dmat_embed,p2, kernel_grid)

    
    fvals = nan(1, length(kernel_grid));


    for i = 1:length(kernel_grid(:))

        fval = opt_func(p2, dmat_embed, kernel_grid(i));
        fvals(i) = fval;

    end

    [~, ind] = min(fvals);

    disp('Computing kernel');
    [k_best, fval, exitflag] = fminsearch(@(x) opt_func(p2, dmat_embed, x), kernel_grid(ind), optimset('Display','off', 'MaxIter', 200));

    [~, pred_pos] = opt_func(p2, dmat_embed, k_best);

    pred_err_sq = sum((p2 - pred_pos).^2,2);


    
end

function plot_kernel_error(savename, fvals, kernel_grid)
    
    if ~exist(savename, 'file')
        
        figure('color', 'w'), 
        hold on

        plot(log10(kernel_grid), fvals)

        xlabel('log10(kernel size)')
        ylabel('HOO MSE')

        saveas(gcf, savename, 'png')
        close(gcf)
    end
end

function [diff, pred_pos] = opt_func(pos,dists,h)

    %if not positive definite, the kernel is no good
    if h <= 0
        diff = inf;
        pred_pos = inf;
        return
    end

    pred_pos = kernel_pred(pos, dists, h);
    
    diff = sum(sum((pos-pred_pos).^2,2),1);

end

function pred_pos = kernel_pred(pos, dists,h)
    w = zeros(size(dists));
    w(:) = mvnpdf(dists(:), 0, h);
    w(all(w == 0, 2),:) = 1;
    w = w./repmat(sum(w,2), [1, size(w,2)]);
    
    pred_pos = sum(repmat(permute(pos, [3,1,2]), [size(dists,1),1,1]).*repmat(w, [1,1,size(pos,2)]),2);
    pred_pos = permute(pred_pos, [1,3,2]);
    
end

