function [pval_nuc, pval_nuc_all, pval_cell, pval_cell_all] = weighted_kernel_pred_iter(p_cell, p_nuc, cvinds, niter, saveparent)
%does nested hold one out cross validation

save_pred_dir = [saveparent filesep 'pred'];
save_iter_dir = [saveparent filesep 'iter'];

ndat = size(p_cell,1);

[~, ~, ~, ~, save_pred_file] = weighted_kernel_pred(p_cell, p_nuc, cvinds, save_pred_dir, false);

save_perm_file = cell(niter,1);

rng('shuffle')
iters = randperm(niter);

%pick a random iteration to work on
for i = 1:length(iters)
    save_perm_dir = [save_iter_dir filesep 'perm' num2str(iters(i))];
    
    %set the random to the iteration index
    rng(iters(i));
    %shuffle both the cell positions and nuc positions cause why not
    p_cell_perm = p_cell(randperm(ndat),:);
    p_nuc_perm = p_nuc(randperm(ndat), :);

    [~, ~, ~, ~, save_perm_file{iters(i)}] = weighted_kernel_pred(p_cell_perm, p_nuc_perm, cvinds, save_perm_dir, false);
end

p_cell_m = mean(p_cell(:));
p_nuc_m = mean(p_nuc(:));

p_cell_std = std(p_cell(:)-p_cell_m(:));
p_nuc_std = std(p_nuc(:)-p_nuc_m(:));

p_cell = (p_cell - p_cell_m)./p_cell_std;
p_nuc = (p_nuc - p_nuc_m)./p_nuc_std;

pred_dat = load(save_pred_file);
p_nuc_pred = pred_dat.p_nuc_pred;
p_cell_pred = pred_dat.p_cell_pred;

p_cell_pred = (p_cell_pred - p_cell_m)./p_cell_std;
p_nuc_pred = (p_nuc_pred - p_nuc_m)./p_nuc_std;

pred_cell_err = sum((p_cell - p_cell_pred).^2,2);
pred_nuc_err = sum((p_nuc - p_nuc_pred).^2,2);
    


for i = 1:niter
    try
        pred_dat = load(save_perm_file{i});
        p_nuc_iter = pred_dat.p_nuc_pred;
        p_cell_iter = pred_dat.p_cell_pred;
        
        p_cell_iter = (p_cell_iter - p_cell_m)./p_cell_std;
        p_nuc_iter = (p_nuc_iter - p_nuc_m)./p_nuc_std;

        perm_cell_err(:,i) = sum((p_cell - p_cell_iter).^2,2);
        perm_nuc_err(:,i) = sum((p_nuc - p_nuc_iter).^2,2);
    catch
        perm_cell_err(:,i) = nan;
        perm_nuc_err(:,i) = nan;
    end
end


keepinds = ~any(isnan(perm_cell_err) | isnan(perm_nuc_err),2);


pval_cell = nan(ndat, 1);
pval_nuc = nan(ndat,1);

pval_cell(keepinds,:) = mean(repmat(pred_cell_err(keepinds), [1,niter]) >= perm_cell_err(keepinds,:),2);
pval_nuc(keepinds,:) = mean(repmat(pred_nuc_err(keepinds), [1,niter]) >= perm_nuc_err(keepinds,:),2);

pval_cell_all = mean(mean(repmat(pred_cell_err(keepinds), [1,niter]),2) >= mean(perm_cell_err(keepinds,:),2));
pval_nuc_all =  mean(mean(repmat(pred_nuc_err(keepinds), [1,niter]),2) >= mean(perm_nuc_err(keepinds,:),2));

