
parentdirs = {'/Volumes/Data/Documents/Murphylab/projects/shapeSpace/', ...
                '/projects/cellorganizer/gj/shapeSpace/', ...
                '/Users/Greg/Documents/murphylab/projects/shapeSpace/'};
            
            
for i = 1:length(parentdirs)
    if exist(parentdirs{i}, 'dir')
        parentdir = parentdirs{i};
        break
    end
end

addpath(genpath([parentdir filesep 'code']));

expdir = [parentdir filesep 'diffeo_tub'];
addpath(genpath([expdir filesep 'code']));


diffeodir = [expdir filesep 'diffeomorphic_newesg'];
if ~exist(diffeodir, 'dir')
    mkdir(diffeodir)
end
disp(diffeodir);
cd(diffeodir)

load('cellModel.mat');

resultsdir = [diffeodir filesep 'results'];

savedir = [resultsdir filesep mfilename];
if ~exist(savedir, 'dir')
    mkdir(savedir)
end

tempdir = [savedir filesep 'temp'];
if ~exist(tempdir, 'dir')
    mkdir(tempdir)
end

allfeats_file = [diffeodir filesep 'objfeats.mat'];

if ~exist(allfeats_file, 'file')

    for i = 1:model.cellShapeModel.numimgs
        disp([num2str(i) filesep num2str(model.cellShapeModel.numimgs)])
        savefile = [tempdir filesep 'feats' num2str(i) '.mat'];

        if ~exist(savefile, 'file')
            img = model.cellShapeModel.imfunc(i);

            if ~isempty(img)
                [cellfeat, nucfeat, bothfeat, featnames] = cell2objfeat(img);
            else
                cellfeat = [];
                nucfeat = [];
                bothfeat = [];
            end
            save(savefile, 'cellfeat', 'nucfeat', 'bothfeat', 'featnames')
        else
            load(savefile)
        end

        allfeats{i} = [cellfeat, nucfeat, bothfeat]; 
    end
    
    save(allfeats_file, 'allfeats', 'featnames')
else
    load(allfeats_file)
end

load([resultsdir filesep 'script_figure2_Hela3D' filesep 'embedfile.mat'])
model.cellShapeModel = ml_initparam(embed_model, model.cellShapeModel);

fig_param.subsize = 800;
fig_param.imfunc = @(x) im2projection_RGB(model.cellShapeModel.imfunc(x), struct('justz', true))*1.5;

figure, 
fig_param = showShapeSpaceFigure(model, [], fig_param);

obj_feats = vertcat(allfeats{fig_param.synthorder});
% obj_feats = [obj_feats, obj_feats(:,1:length(featnames{1}))./obj_feats(:,(length(featnames{1})+1):((length(featnames{1})+length(featnames{2}))))];
obj_feats = zscore(obj_feats);
   
pos_feats = model.cellShapeModel.positions(fig_param.synthorder,:);

hold on,
for i = 1:size(obj_feats,2)
%     B = inv(pos_feats'*pos_feats)*pos_feats'*obj_feats(:,i);
    lm{i} = fitlm(zscore(pos_feats), obj_feats(:,i));
    
    coeff(i,:) = lm{i}.Coefficients.Estimate;
    
    %skip the first coefficient since it's the intercept and should be zero
    %because the data are centered.
    plot3([0,coeff(i,2)], [0, coeff(i,3)], [0, coeff(i,4)], 'k')
end
axis tight

for i = 1:size(pos_feats,2)   
    for j = 1:size(obj_feats,2)
        
        [c, p] = corrcoef(pos_feats(:,i), obj_feats(:,j));
        corr(i,j) = c(1,2);
        pval(i,j) = p(1,2);
    end
end

stat_sig = (pval.*numel(pval)) < 0.01;


display_feat_names = {};
for i = 1:length(featnames{1})
    display_feat_names{end+1} = ['cell SF' num2str(i)];
end
for i = 1:length(featnames{2})
    display_feat_names{end+1} = ['nuc SF' num2str(i)];
end

display_feat_names{end+1} = 'cell nuc COM dist';
display_feat_names{end+1} = 'cell nuc COM z-dist';
display_feat_names{end+1} = 'cell nuc COM xy-dist';
display_feat_names{end+1} = 'Maj ax angle diff';

% for i = 1:length(featnames{1})
%     display_feat_names{end+1} = ['( cell SF' num2str(i) ' ) ' filesep ' ( nuc SF' num2str(i) ' )'];
% end

for i = 1:size(pos_feats,2)
    pos_feats_name{i} = ['MDS ' num2str(i)];
    [~, ind] = sort(pval(i,:), 'Ascend');
end
    
displayfeats = any(stat_sig,1);

hm = HeatMap(flipdim(corr(:,displayfeats).*stat_sig(:,displayfeats),1));
hm.Colormap = flipdim(1-othercolor('PiYG11'),1);
% hm.ColumnLabels = display_feat_names(displayfeats);

h = figure('color', 'w');
a = plot(hm, h);
axes(a);
h = colorbar;

set(h, 'Ycolor', [1/255, 1/255, 1/255])
set(h, 'Xcolor', [1/255, 1/255, 1/255])
set(h, 'Box', 'off')

columnlabels.Labels = display_feat_names(displayfeats);
columnlabels.Colors = repmat({[1/255, 1/255, 1/255]}, 1,sum(displayfeats));

% set(hm, 'ColumnLabelsColor', columnlabels)
% daspect([1, 0.3, 1])

saveas(gcf, [savedir filesep 'feat_heat_map.png'], 'png')
img = ml_readimage([savedir filesep 'feat_heat_map.png']);

img_text = repmat(all(img==1,3),[1,1,3]);
img_bg = repmat(all(img==255,3),[1,1,3]);

img(img_text) = 255;
img(img_bg) = 0;

imwrite(img, [savedir filesep 'feat_heat_map_black.png'], 'png')

featnames_cat = [featnames{:}]';
featnames_cat = [featnames_cat ; repmat({''}, length(display_feat_names) - length(featnames_cat), 1)];

save_table = array2table([featnames_cat, display_feat_names'], 'VariableNames', {'feat_names', 'display_feat_names'});

ndims = size(pos_feats,2);
corr_strs = [repmat({'corr_mds_'}, [ndims,1]) cellstr(num2str((1:ndims)'))];
corr_strs = cellfun(@(x,y) [x,y], corr_strs(:,1), corr_strs(:,2), 'UniformOutput', false);

pval_strs = [repmat({'pval_mds_'}, [ndims,1]) cellstr(num2str((1:ndims)'))];
pval_strs = cellfun(@(x,y) [x,y], pval_strs(:,1), pval_strs(:,2), 'UniformOutput', false);

stat_sig_strs = [repmat({'stat_sig_'}, [ndims,1]) cellstr(num2str((1:ndims)'))];
stat_sig_strs = cellfun(@(x,y) [x,y], stat_sig_strs(:,1), stat_sig_strs(:,2), 'UniformOutput', false);

corr_table = array2table([corr', pval', stat_sig', displayfeats', ], 'VariableNames', [corr_strs', pval_strs', stat_sig_strs', 'display_pvals']);

save_table = [save_table, corr_table];

writetable(save_table, [savedir filesep 'corr_table.csv'])


% figure
% for i = 1:size(pos_feats,2)
%     subplot(1,5, i)
%     for j = 1:size(obj_feats,2);
%         hold on
%         plot([0, corr(i,j)], [0, abs(coeff(j, i+1))])
%     end
%     axis([0,1, 0, 1])
%     axis
% end


