function [ dna_regions, cell_regions, imdna_sub, cell_sub ] = imfunc_seg_hela( imdir, downsize )

imdna_sub = ml_downsize(double(imfunc_helamodel([imdir filesep 'dna'])), downsize);
imdna = imfilter(imdna_sub, fspecial('gaussian', 5));

cell_sub = ml_downsize(double(imfunc_helamodel([imdir filesep 'cell'])), downsize);
imcell = imfilter(cell_sub, fspecial('gaussian', 5));

imseg = ml_downsize(imfunc_helamodel([imdir filesep 'crop']), [downsize(1:2) 1], 'nearest');

numpx = squeeze(sum(sum(imseg,1),2));
[~, ind] = max(numpx);
cropregion = double(imseg(:,:,ind));

%rc threshold the dna image
segdna = imdna > ml_rcthreshold(imdna);

%most common pixel threshold the cell image
imcell = imfilter(imcell, fspecial('gaussian', 5));
[n, val] = hist(double(imcell(:)), 128);
[~, maxind ] = max(n);
segcell = imcell > val(maxind(1)+1);

%seeded watershed
a = regionprops(segdna, 'FilledArea', 'PixelIdxList' );

minsize = 100;

if ~isempty(a([a.FilledArea] < minsize))
    segdna([a([a.FilledArea] < minsize).PixelIdxList']) = 0;
end

ws = watershed(imimposemin(-double(imcell), segdna));

%remove any regions that dont have DNA associated with them
cell_regions = bwlabeln(double(ws).*double(segcell), 6);
cell_regions = ismember(cell_regions, unique(cell_regions(segdna)));

cell_regions = bwlabeln(cell_regions, 6);

dna_regions = cell_regions.*segdna;

boundary = ones(size(imcell));
boundary(3:end-2,3:end-2,:) = 0;

region_nums = unique(dna_regions(boundary>0));
dna_regions = ~ismember(dna_regions, region_nums);
cell_regions = ~ismember(cell_regions, region_nums);

for i = 1:size(dna_regions,3)
    dna_regions(:,:,i) = imfill(dna_regions(:,:,i), 'holes');
    cell_regions(:,:,i) = imfill(cell_regions(:,:,i), 'holes');
end

dna_regions = double(dna_regions) .* repmat(cropregion, [1,1,size(dna_regions,3)]);
cell_regions = double(cell_regions) .* repmat(cropregion, [1,1,size(dna_regions,3)]);

%final cell shape regularization
cell_regions = imfilter(cell_regions, fspecial3('gaussian', 3)) > 0.5;

%again remove all little bits
cell_regions = bwlabeln(cell_regions, 6);
dna_regions = cell_regions.*segdna;

region_nums = unique(cell_regions(dna_regions>0));
dna_regions = ismember(dna_regions, region_nums);
cell_regions = ismember(cell_regions, region_nums);

for i = 1:size(dna_regions,3)
    dna_regions(:,:,i) = imfill(dna_regions(:,:,i), 'holes');
    cell_regions(:,:,i) = imfill(cell_regions(:,:,i), 'holes');
end



end
