%note: check out weighted orthogonal procrustes
% http://www8.cs.umu.se/~viklands/WOPP/index.html

for i = 1:506
    filename = ['cellcodes_' num2str(i) '.mat'];
    if exist(filename, 'file')
        load(filename)
        rad_ratios{i} = rad_ratio(:);
        nucdists{i} = nucdist(:);
        nucelldists{i} = nucelldist(:);
        keepinds(i) = true;
    end
end
a = (([nucelldists{:}]./([nucdists{:}]+[nucelldists{:}]))');
[coeff, score, latent] = pca(a);


[~, nd] = pca([nucdists{:}]');
[~, cd] = pca(([nucelldists{:}]+[nucdists{:}])');


[~, cd2] = procrustes(nd, cd);


m1 = model;
m1.cellShapeModel.positions = nan(length(keepinds),size(nd,2));
m1.cellShapeModel.positions(keepinds,:) = nd;



m2 = model;
m2.cellShapeModel.positions = nan(length(keepinds),size(nd,2));
m2.cellShapeModel.positions(keepinds,:) = cd2;



npts = size(nd,1);
neighbors = 20;
niter = 250;

nd_distmap = squareform(pdist(nd));
[~, nd_inds] = sort(nd_distmap,2);
cd_distmap = squareform(pdist(cd));
[~, cd_inds] = sort(cd_distmap,2);

nd_procrustes = zeros(npts,1);
nd_procrustes_iter = zeros(npts, niter);

cd_procrustes = zeros(npts,1);
cd_procrustes_iter = zeros(npts, niter);


for i = 1:npts
    disp(num2str(i));
    
    nd_procrustes(i) = procrustes(nd(nd_inds(i, 1:neighbors),:), cd(nd_inds(i, 1:neighbors),:),'Scaling',false);
    
    for j = 1:niter
        nd_procrustes_iter(i,j) = procrustes(nd(nd_inds(i, 1:neighbors),:), cd(nd_inds(i, randperm(neighbors)),:),'Scaling',false);
        nd_procrustes_iter_all(i,j) = procrustes(nd(nd_inds(i, 1:neighbors),:), cd(nd_inds(i, randperm(npts, neighbors)),:),'Scaling',false);
    end
end

for i = 1:size(cd,1)
    disp(num2str(i));
    
    cd_procrustes(i) = procrustes(cd(cd_inds(i, 1:neighbors),:), nd(cd_inds(i, 1:neighbors),:),'Scaling',false);
    
    for j = 1:niter
        cd_procrustes_iter(i,j) = procrustes(cd(cd_inds(i, 1:neighbors),:), nd(cd_inds(i, randperm(neighbors)),:),'Scaling',false);
        cd_procrustes_iter_all(i,j) = procrustes(nd(nd_inds(i, 1:neighbors),:), cd(nd_inds(i, randperm(npts, neighbors)),:),'Scaling',false);
    end
end

bonfcorr = length(nd_p);

nd_p = (1 - mean(repmat(nd_procrustes, [1,niter]) < nd_procrustes_iter, 2))*bonfcorr;
cd_p = (1 - mean(repmat(cd_procrustes, [1,niter]) < cd_procrustes_iter, 2))*bonfcorr;



%color by errors
nd_colors = zeros(npts, 3);
nd_colors(keepinds,:) = grayrange2rgb(zscore(nd_procrustes));

cd_colors = zeros(npts, 3);
cd_colors(keepinds,:) = grayrange2rgb(zscore(cd_procrustes));


% p1.imfunc = @(x) im2projection_RGB(m1.cellShapeModel.imfunc(x)>=2, struct('justz', true, 'cm', @(y) nd_colors(x,:)));
p1.imfunc = @(x) img2flat(m1.cellShapeModel.imfunc(x)>=2, nd_colors(x,:));
p1.alpha = [0,0,0];

% p2.imfunc = @(x) im2projection_RGB(m2.cellShapeModel.imfunc(x)>=1, struct('justz', true, 'cm', @(y) cd_colors(x,:)));
p2.imfunc = @(x) img2flat(m2.cellShapeModel.imfunc(x)>=1, cd_colors(x,:));
p2.alpha = [0,0,0];

%color by p value

p1.subsize = 4;
p2.subsize = 4;

figure
title('error')
for i = 1:npts
    line([nd(i,1), cd2(i,1)], [nd(i,2), cd2(i,2)], 'color', 'k')
    hold on
end

showShapeSpaceFigure(m1, [], p1);
showShapeSpaceFigure(m2, [], p2);


%color by p value
nd_colors = zeros(npts, 3);
nd_colors(keepinds,:) = grayrange2rgb(nd_p, 0, 1, @cool);

cd_colors = zeros(npts, 3);
cd_colors(keepinds,:) = grayrange2rgb(cd_p, 0, 1, @cool);


% p1.imfunc = @(x) im2projection_RGB(m1.cellShapeModel.imfunc(x)>=2, struct('justz', true, 'cm', @(y) nd_colors(x,:)));
p1.imfunc = @(x) img2flat(m1.cellShapeModel.imfunc(x)>=2, nd_colors(x,:));
p1.alpha = [0,0,0];

% p2.imfunc = @(x) im2projection_RGB(m2.cellShapeModel.imfunc(x)>=1, struct('justz', true, 'cm', @(y) cd_colors(x,:)));
p2.imfunc = @(x) img2flat(m2.cellShapeModel.imfunc(x)>=1, cd_colors(x,:));
p2.alpha = [0,0,0];

figure
title('p-vals')
for i = 1:npts
    line([nd(i,1), cd2(i,1)], [nd(i,2), cd2(i,2)], 'color', 'k')
    hold on
end

showShapeSpaceFigure(m1, [], p1);
showShapeSpaceFigure(m2, [], p2);

figure('color', 'w'), 
scatter(zscore(nd_procrustes), zscore(cd_procrustes))


