
figuredir = '/Users/Greg/Documents/murphylab/projects/shapeSpace/figures';
tempdir = [figuredir filesep mfilename];

posfile = [tempdir filesep 'positions.mat'];

if ~exist(posfile, 'file')
    method = struct('method',  'regress',   'weight_factor', 0, 'desired_shape_space_dimensionality', inf);

    [pos, ~, ~] = embed_partial_distance_matrix(model.cellShapeModel.distances_incomplete, method);
    save(posfile, 'pos')
else
    load(posfile)
end

keepinds = ~any(isnan(pos),2);

pos = pos(keepinds,:);
dists = squareform(pdist(pos));


dnaintens = dnaintens(keepinds);
dnavol = dnavol(keepinds);

avginten = dnaintens./dnavol;


pos = [ones(size(pos,1),1) pos];

hoofile = [tempdir filesep 'lasso_hoo.mat'];
if ~exist(hoofile, 'file')
    for i = 1:size(pos,1)
        disp([num2str(i) filesep num2str(size(pos,1))])
        inds = true(size(pos,1),1);
        inds(i) = false;

        inten_train =  avginten(inds);
        pos_train = pos(inds,:);

        [B, fitinfo] = lasso(pos_train, inten_train, 'CV', 10);

        pos_lasso = pos_train(:,B(:, fitinfo.IndexMinMSE)~= 0);
        linmodel = fitlm(pos_lasso, inten_train);

        pred_inten(i) = [1, pos(i,B(:, fitinfo.IndexMinMSE)~= 0)]*linmodel.Coefficients.Estimate;
    end
    save(hoofile)
else
    load(hoofile)
end

figure('color', 'w'), 
linmodel.plot
saveas(gcf, [figuredir filesep 'linfit_hoo.tif'], 'tif')

figure('color', 'w')
scatter(avginten, pred_inten)
saveas(gcf, [figuredir filesep 'linfit_inten_vs_model_hoo_scatter.tif'], 'tif')


model.cellShapeModel.positions = nan(size(model.cellShapeModel.positions,1),2);
model.cellShapeModel.positions(keepinds,:) = [avginten', pred_inten'];

inten = nan(model.cellShapeModel.numimgs,1);
inten(keepinds) = avginten;

param.subsize = 20;

figure, showShapeSpaceFigure(model, inten, param)
xlabel('Average DNA pixel intensity')
ylabel('XB')

saveas(gcf, [figuredir filesep 'linfit_inten_vs_model_hoo.tif'], 'tif')

hoofile = [tempdir filesep 'lasso.mat'];

if ~exist(hoofile, 'file')
    [B, fitinfo] = lasso(pos, avginten, 'CV', 10);

    pos_lasso = pos(:,B(:, fitinfo.IndexMinMSE)~= 0);
    linmodel = fitlm(pos_lasso, avginten);
    save(hoofile, 'linmodel', 'B', 'fitinfo')
else
    load(hoofile)
end

model.cellShapeModel.positions = nan(size(model.cellShapeModel.positions,1),2);
model.cellShapeModel.positions(keepinds,:) = [avginten', linmodel.predict];

inten = nan(model.cellShapeModel.numimgs,1);
inten(keepinds) = avginten;

param.subsize = 20;

figure, showShapeSpaceFigure(model, inten, param)
xlabel('Average DNA pixel intensity')
ylabel('XB')

saveas(gcf, [figuredir filesep 'linfit_inten_vs_model.tif'], 'tif')



% [~, inds ] = sort(abs(coeffs), 'descend');
% model.cellShapeModel.positions = nan(size(model.cellShapeModel.positions,1),2);
% model.cellShapeModel.positions(keepinds,:) = zscore([pos_lasso(:,inds(1)), pos_lasso(:,inds(2))]);
% 
% figure('color', 'w')
% showShapeSpaceFigure(model, inten)

