function [dimensionality, dnaimgs, cellimgs, param] = script_modelsetup_newseg_com_nuc(parentdir)


if ~exist('parentdir', 'var') || isempty(parentdir)
    parentdirs = {'/Volumes/Data/Documents/Murphylab/projects/shapeSpace/', ...
                    '/projects/cellorganizer/gj/shapeSpace/', ...
                    '/Users/Greg/Documents/murphylab/projects/shapeSpace/'};


    for i = 1:length(parentdirs)
        if exist(parentdirs{i}, 'dir')
            parentdir = parentdirs{i};
            break
        end
    end
end


addpath(genpath([parentdir filesep 'code']));

expdir = [parentdir filesep 'diffeo_tub'];
addpath(genpath([expdir filesep 'code']));


diffeodir = [expdir filesep 'diffeomorphic_newesg_com_nuc'];
if ~exist(diffeodir, 'dir')
    mkdir(diffeodir)
end
cd(diffeodir)

segdir = [diffeodir filesep 'seg'];
if ~exist(segdir, 'dir')
    mkdir(segdir)
end

settingsfile = [diffeodir filesep 'modelSettings.mat'];


if exist(settingsfile, 'file')
    disp(['Settings file found. Loading from, ' settingsfile '.'])
    load(settingsfile)
    return
end


tempDir = [diffeodir filesep 'temp'];

imgparentdirs = getSubDirs([expdir '/images/Hela/3D/']);

resolution = [0.049, 0.049, 0.2000];
downsampling = [resolution(end)/resolution(1),resolution(end)/resolution(2),1]*2.5;

c = 1;
for i = 1:length(imgparentdirs)
    [subdirs, subdirname] = getSubDirs(imgparentdirs{i}, 'cell*');
    for j = 1:length(subdirs)
        c
        disp(['Segmenting ' num2str(c) ': ' subdirs{j}])
        
        savefile = [segdir filesep 'cell' num2str(c) '.mat'];
        if ~exist(savefile, 'file')
            
            dna = [];
            cell = [];
            
            try
                
                [dna, cell, dna_sub, cell_sub] = imfunc_seg_hela(subdirs{j}, downsampling);

                imcell = zeros(size(cell));
                imcell(cell>0) = 1;
                imcell(dna>0) = 2;

                imwrite([im2projection_RGB(imcell), im2projection_RGB(imcell, struct('method', 'max'))], [segdir filesep 'cell' num2str(c) '.tif'], 'tif');
                
                imwrite([repmat(mat2gray(im2projection(dna_sub)),[1,1,3]), repmat(mat2gray(im2projection(cell_sub)),[1,1,3]),  im2projection_RGB(imcell), im2projection_RGB(imcell, struct('method', 'max'))], [segdir filesep 'cell_all' num2str(c) '.tif'], 'tif');
                
            catch
                disp([num2str(c) ' failed'])
            end
            save(savefile, 'cell', 'dna')
        end

        dnaimgs{c} = @() imfunc_load_hela(savefile, 'dna');
        cellimgs{c} = @() imfunc_load_hela(savefile, 'cell');
        imgparentdir{c} = imgparentdirs{i};
        imgdir{c} = subdirs{j};
        c = c+1;
        
    end
end

skipimgs = [106, 109, 110, 111, 113, 115, 116, 117, 149, 203, 204, 205, ...
    206:252, 306, 314, 401, 402, 405, 429, 434, 442, 443];

for i = 1:length(skipimgs)
    dnaimgs{skipimgs(i)} = @() [];
    cellimgs{skipimgs(i)} = @() [];
end

param.model.diffeomorphic.convolution_method = 'convnfft';

dimensionality = '3D';
param.model.name = 'tub and nuc diffeomorphic cell shape model';
param.model.id = 'num2str(now)';
param.model.resolution = [1,1,1];
param.model.filename = 'cellModel.mat';
param.nucleus.type = 'diffeomorphic';
param.cell.type = 'diffeomorphic';
param.train.flag = 'framework';
param.verbose = true;
param.debug = true;
param.model.downsampling = [1,1,1];


param.preprocess = true;

% param.masks = maskimgs;

param.model.diffeomorphic.number_windows = 1;
param.saveIntermediateResults = false;

param.model.diffeomorphic.convergence_absolute_error_difference = 5;
param.model.diffeomorphic.convergence_absolute_error = 1;

param.model.diffeomorphic.scale_z_kernel = true;
% param.model.diffeomorphic.relative_filter_radius = 0.05;

param.model.diffeomorphic.maximum_deformation_per_step = [1,1,1]/10;

param.model.diffeomorphic.embedding.weighted = true;

save(settingsfile);