
parentdirs = {'/Volumes/Data/Documents/Murphylab/projects/shapeSpace/', ...
                '/projects/cellorganizer/gj/shapeSpace/', ...
                '/Users/Greg/Documents/murphylab/projects/shapeSpace/'};
            
            
for i = 1:length(parentdirs)
    if exist(parentdirs{i}, 'dir')
        parentdir = parentdirs{i};
        break
    end
end

addpath(genpath([parentdir filesep 'code']));

expdir = [parentdir filesep 'diffeo_tub'];
addpath(genpath([expdir filesep 'code']));

diffeodir = [expdir filesep 'diffeomorphic_newesg'];
if ~exist(diffeodir, 'dir')
    mkdir(diffeodir)
end
disp(diffeodir);
cd(diffeodir)

segdir = [diffeodir filesep 'seg'];
if ~exist(segdir, 'dir')
    mkdir(segdir)
end

resultsdir = [diffeodir filesep 'results'];
if ~exist(resultsdir, 'dir')
    mkdir(resultsdir)
end

saveparent = [resultsdir filesep mfilename];

if ~exist(saveparent, 'dir')
    mkdir(saveparent)
end

distances_cell_file = [saveparent filesep 'distances_cell.mat'];
distances_nuc_file = [saveparent filesep 'distances_nuc.mat'];


if ~exist(distances_cell_file, 'file')
    load('cellModel_cell.mat')
    distances_incomplete_cell = model.cellShapeModel.distances_incomplete;
    clear model
    
    save(distances_cell_file, 'distances_incomplete_cell')
else
    load(distances_cell_file)
end

if ~exist(distances_nuc_file, 'file')
    load('cellModel_nuc.mat')
    distances_incomplete_nuc = model.cellShapeModel.distances_incomplete;
    clear model
    
    save(distances_nuc_file, 'distances_incomplete_nuc')
else
    load(distances_nuc_file)
end

[ keepinds_cell ] = get_complete_distance_matrix( distances_incomplete_cell);
[ keepinds_nuc ] = get_complete_distance_matrix( distances_incomplete_nuc);

keepinds = intersect(keepinds_cell, keepinds_nuc);

distances_complete_cell = distances_incomplete_cell(keepinds, keepinds);
distances_complete_nuc = distances_incomplete_nuc(keepinds, keepinds);

embedding_dat_file = [saveparent filesep 'embeddingdat.mat'];
if ~exist(embedding_dat_file, 'file')
    [dim_struct_euc_cell, dim_struct_orig_cell] = compute_embedding_dimensionality(distances_complete_cell);
    [dim_struct_euc_nuc, dim_struct_orig_nuc] = compute_embedding_dimensionality(distances_complete_nuc);
    save(embedding_dat_file, 'dim_struct_euc_cell', 'dim_struct_orig_cell', 'dim_struct_euc_nuc', 'dim_struct_orig_nuc')
else
    load(embedding_dat_file)
end
    
p_cell = dim_struct_orig_cell.pos;
p_nuc = dim_struct_orig_nuc.pos;

obj_condition = ones(size(p_nuc,1),1);

ucond = 1;

niter = 500;

alldatfile = [saveparent filesep 'alldat.mat'];
if ~exist(alldatfile, 'file')
    savedir = [saveparent filesep 'c' num2str(ucond)];    
    cvinds = 1:size(p_cell);
    
    [pval_nuc, pval_nuc_all, pval_cell, pval_cell_all] = test_cell_nuc_pred_linreg(p_cell, p_nuc, cvinds, niter, savedir);
    

else
    load(alldatfile)
end

% param.imfunc = @(x) im2projection_RGB(model.cellShapeModel.imfunc(x), p2)

param.subsize = 1000;

cell_shape_figure = [saveparent filesep 'shapeSpace_cell.tif'];
cell_shape_figure_blk = [saveparent filesep 'shapeSpace_cell_blk.tif'];
if ~exist(cell_shape_figure, 'file')
    load('cellModel_cell.mat')
    model.cellShapeModel.positions = nan(size(model.cellShapeModel.positions,1), size(p_cell,2));
    model.cellShapeModel.positions(keepinds,:) = p_cell;
    colors = grayrange2rgb(pval_nuc);
    param.imfunc = @(x) (im2projection_RGB(model.cellShapeModel.imfunc(x),struct('scale_inten', 2, 'justz', true, 'cm', @(y) colormap(colors(x,:)))));

    figure, 
    showShapeSpaceFigure(model, pval_nuc, param)
    saveas(gcf, cell_shape_figure, 'tif')
    
    img = ml_readimage(cell_shape_figure);
    whtpix = all(img == 255, 3);
    blkpix = all(img == 0,3);

    img(repmat(whtpix,[1,1,3])) = 0;
    img(repmat(blkpix,[1,1,3])) = 255;

    imwrite(img, cell_shape_figure_blk, 'tif')
end
    

param.subsize = 1000;
nuc_shape_figure = [saveparent filesep 'shapeSpace_nuc.tif'];
nuc_shape_figure_blk = [saveparent filesep 'shapeSpace_nuc_blk.tif'];
if ~exist(nuc_shape_figure, 'file')
    load('cellModel_nuc.mat')
    model.cellShapeModel.positions = nan(size(model.cellShapeModel.positions,1), size(p_nuc,2));
    model.cellShapeModel.positions(keepinds,:) = p_nuc;
    
    colors = grayrange2rgb(pval_cell);
    param.imfunc = @(x) (im2projection_RGB(model.cellShapeModel.imfunc(x),struct('scale_inten', 4, 'justz', true, 'cm', @(y) colormap(colors(x,:)))));

    figure, 
    showShapeSpaceFigure(model, pval_cell, param)
    saveas(gcf, nuc_shape_figure, 'tif')
    
    img = ml_readimage(nuc_shape_figure);
    whtpix = all(img == 255, 3);
    blkpix = all(img == 0,3);

    img(repmat(whtpix,[1,1,3])) = 0;
    img(repmat(blkpix,[1,1,3])) = 255;

    imwrite(img, nuc_shape_figure_blk, 'tif')
end




mse_nuc = mean(pred_err_nuc);
mse_cell = mean(pred_err_cell);

p_nuc_err = mean(repmat(pred_err_nuc, 1, niter) >= [pred_err_nuc_perm{:}],2);
p_nuc_mse = mean(repmat(mse_nuc, 1, niter) >= mean([pred_err_nuc_perm{:}],1),2);
p_cell_err = mean(repmat(pred_err_cell, 1, niter) >= [pred_err_cell_perm{:}],2);
p_cell_mse = mean(repmat(mse_cell, 1, niter) >= mean([pred_err_cell_perm{:}],1),2);


t = table(ucond', mse_nuc', p_nuc_mse', mse_cell', p_cell_mse');
t.Properties.VariableNames = {'Condition', 'NuclerShapeMSE', 'NuclerShapeMSEp', 'CellShapeMSE', 'CellShapeMSEp'};

[mse_nuc, mse_cell]

figure('color', 'w'), 
subplot(1,2, 1)
bar(mse_nuc')
xlabel('condition')
ylabel('mean squared prediction error')

subplot(1,2, 2), bar(mse_cell')
xlabel('condition')


% param.subsize = 6000;
param.synthorder = keepinds;

err = inf(size(p_cell,1),1);
err(keepinds) = pred_err_nuc;


