
system('hostname');


parentdirs = {'/Volumes/Data/Documents/Murphylab/projects/shapeSpace/', ...
                '/projects/cellorganizer/gj/shapeSpace/', ...
                '/Users/Greg/Documents/murphylab/projects/shapeSpace/'};
            
            
for i = 1:length(parentdirs)
    if exist(parentdirs{i}, 'dir')
        parentdir = parentdirs{i};
        break
    end
end

codefiles_save = [parentdir filesep 'codefiles' num2str(i) '.mat'];

expdir = [parentdir filesep 'diffeo_tub'];

% if ~exist(codefiles_save, 'file')
%     disp('building one time file structure')
    code_files = genpath([parentdir filesep 'code']);
    diffeo_files = genpath([expdir filesep 'code']);
%     save(codefiles_save, 'code_files', 'diffeo_files')
% else
%     load(codefiles_save)
% end

addpath(code_files)
addpath(diffeo_files);

if ~exist('param_in', 'var')
    param_in = [];
end

param_in = ml_initparam(param_in, struct('skipsynth', false));

diffeodir = [expdir filesep 'diffeomorphic_newesg'];
if ~exist(diffeodir, 'dir')
    mkdir(diffeodir)
end
disp(diffeodir);
cd(diffeodir)

segdir = [diffeodir filesep 'seg'];
if ~exist(segdir, 'dir')
    mkdir(segdir)
end

resultsdir = [diffeodir filesep 'results'];
if ~exist(resultsdir, 'dir')
    mkdir(resultsdir)
end

saveparent = [resultsdir filesep mfilename];

if ~exist(saveparent, 'dir')
    mkdir(saveparent)
end

if ~exist('build', 'var')
    build = false;
end

distances_cell_file = [saveparent filesep 'distances_cell.mat'];
distances_nuc_file = [saveparent filesep 'distances_nuc.mat'];


if ~exist(distances_cell_file, 'file')
    load('cellModel_cell.mat')
    distances_incomplete_cell = model.cellShapeModel.distances_incomplete;
    clear model
    
    save(distances_cell_file, 'distances_incomplete_cell')
else
    load(distances_cell_file)
end

if ~exist(distances_nuc_file, 'file')
    load('cellModel_nuc.mat')
    distances_incomplete_nuc = model.cellShapeModel.distances_incomplete;
    clear model
    
    save(distances_nuc_file, 'distances_incomplete_nuc')
else
    load(distances_nuc_file)
end

[ keepinds_cell ] = get_complete_distance_matrix( distances_incomplete_cell);
[ keepinds_nuc ] = get_complete_distance_matrix( distances_incomplete_nuc);

keepinds = intersect(keepinds_cell, keepinds_nuc);

distances_complete_cell = distances_incomplete_cell(keepinds, keepinds);
distances_complete_nuc = distances_incomplete_nuc(keepinds, keepinds);

embedding_dat_file = [saveparent filesep 'embeddingdat.mat'];
if ~exist(embedding_dat_file, 'file')
    [dim_struct_euc_cell, dim_struct_orig_cell, ~, positions_cell] = compute_embedding_dimensionality(distances_complete_cell);
    [dim_struct_euc_nuc, dim_struct_orig_nuc, ~, positions_nuc] = compute_embedding_dimensionality(distances_complete_nuc);
    save(embedding_dat_file, 'dim_struct_euc_cell', 'dim_struct_orig_cell', 'dim_struct_euc_nuc', 'dim_struct_orig_nuc', 'positions_cell', 'positions_nuc')
else
    load(embedding_dat_file)
end
    
[cellAngles, cellOffsets] = modeldir2angles_offsets(diffeodir);
cellAngles = cellAngles(keepinds,:);
cellOffsets = cellOffsets(keepinds,:);



niter = 250;

methods = {'linreg', 'gauss'};

cvinds = 1:size(positions_cell{1},1);

param.split_on_sig = false;

alldatfile = [saveparent filesep 'alldat.mat'];
if ~exist(alldatfile, 'file')

    for j = 1:length(methods)
        method_dat_file{j} = [saveparent filesep 'alldat' methods{j} '.mat'];

        for i = 1:length(positions_cell)

            param.method = methods{j};

            savedir = [saveparent filesep param.method filesep 'd' num2str(i)];
            param.name = 'nuc2cell';
            savedir_iter = [savedir filesep param.name];
            param.normfunc = @(x) x;
            nuc2cell{i} = kernel_pred_iter(positions_nuc{i}, positions_cell{i}, cvinds, niter, savedir_iter, param);

% 
%             param.name = 'nuc2pos';
%             savedir_iter = [savedir filesep param.name];
%             param.normfunc = @(x) x;
%             nuc2pos{i} = kernel_pred_iter(positions_nuc{i}, cellOffsets, cvinds, niter, savedir_iter, param);
% 
% 
%             param.name = 'nuc2angle';
%             savedir_iter = [savedir filesep param.name];
%             param.normfunc = @(x) x./repmat(cellfun(@norm, mat2cell(x, [ones(size(x,1),1)], 2)),1,2);
%             nuc2angle{i} = kernel_pred_iter(positions_nuc{i}, cellAngles, cvinds, niter, savedir_iter, param);

            param.name = 'cell2nuc';
            savedir_iter = [savedir filesep param.name];
            param.normfunc = @(x) x;
            cell2nuc{i} = kernel_pred_iter(positions_cell{i}, positions_nuc{i}, cvinds, niter, savedir_iter, param);
% 
%             param.name = 'cell2pos';
%             savedir_iter = [savedir filesep param.name];
%             param.normfunc = @(x) x;
%             cell2pos{i} = kernel_pred_iter(positions_cell{i}, -cellOffsets, cvinds, niter, savedir_iter, param);
% 
%             param.name = 'cell2angle';
%             savedir_iter = [savedir filesep param.name];
%             param.normfunc = @(x) x./repmat(cellfun(@norm, mat2cell(x, [ones(size(x,1),1)], 2)),1,2);
%             cell2angle{i} = kernel_pred_iter(positions_cell{i}, -cellAngles, cvinds, niter, savedir_iter, param);
% 
%             param.name = 'nuccell2pos';
%             savedir_iter = [savedir filesep param.name];
%             param.normfunc = @(x) x;
%             nuccell2pos{i} = kernel_pred_iter([positions_nuc{i} positions_cell{i}], cellOffsets, cvinds, niter, savedir_iter, param);
% 
%             param.name = 'nuccell2angle';
%             savedir_iter = [savedir filesep param.name];
%             param.normfunc = @(x) x;
%             nuccell2angle{i} = kernel_pred_iter([positions_nuc{i} positions_cell{i}], cellAngles, cvinds, niter, savedir_iter, param);

            %do 
            %pred cell shape from nuc shape
            %pred cell shape and orienation from nuc shape
            %pred cell orientation from nuc shape

            %pred nuc shape form cell shape
            %pred nuc shape and orientation from cell shape
            %pred nuc orientation from cell shape

        end  
        save(method_dat_file{j}, 'nuc2cell', 'cell2nuc');%,  'nuc2pos', 'nuc2angle', 'cell2nuc', 'cell2pos', 'cell2angle', 'nuccell2pos', 'nuccell2angle', 'keepinds')
    end

    dat = [];
    c = 1;
    for i = 1:length(method_dat_file)
        if exist(method_dat_file{i}, 'file')
            dat{c} = load(method_dat_file{i});
            c = c+1;
        end
    end


%     i = 7; j = 1;
% 
%     for j = 1:length(methods)
%         param.method = methods{j};
%         savedir = [saveparent filesep param.method filesep 'd' num2str(i)];
% 
%         param.split_on_sig = true;
%         param.name = 'cell2nuc';
%         savedir_iter = [savedir filesep param.name];
%         param.normfunc = @(x) x;
%         dat{j}.cell2nuc{i} = kernel_pred_iter(positions_cell{i}, positions_nuc{i}, cvinds, niter, savedir_iter, param);
% 
%         param.name = 'nuc2cell';
%         savedir_iter = [savedir filesep param.name];
%         param.normfunc = @(x) x;
%         dat{j}.nuc2cell{i} = kernel_pred_iter(positions_nuc{i}, positions_cell{i}, cvinds, niter, savedir_iter, param);
%     end

    save(alldatfile, 'dat')

else
    load(alldatfile)
end

save_pred_dat(dat{2}.cell2nuc(7), dat{2}.nuc2cell(7), dat{2}.cell2nuc{7}.y_pval, dat{2}.nuc2cell{7}.y_pval, {'3T3'}, niter, [], saveparent)


cell2nuc_pval2 = pred_pval(positions_nuc{7}, dat{2}.cell2nuc{7}.y_pred);
nuc2cell_pval2 = pred_pval(positions_cell{7}, dat{2}.nuc2cell{7}.y_pred);



pred_img_dir = [saveparent filesep 'synth_imgs2'];

if ~exist(pred_img_dir, 'dir')
    mkdir(pred_img_dir)
end


cellmodel = load('cellModel_cell.mat');
nucmodel = load('cellModel_nuc.mat');

embedding_file = [saveparent filesep 'embedding_synth.mat'];

pos_cell = nan(size(cellmodel.model.cellShapeModel.positions,1), 7);
pos_nuc = nan(size(nucmodel.model.cellShapeModel.positions,1), 7);

if ~exist(embedding_file, 'file')
    options_cell.positions = nan(size(cellmodel.model.cellShapeModel.positions,1), 7);
    options_cell.positions(keepinds,:) = positions_cell{7};
    
    [ embedding_cell ] = embed_distance_matrix( options_cell.positions, options_cell );
    
    options_nuc.positions = nan(size(nucmodel.model.cellShapeModel.positions,1), 7);
    options_nuc.positions(keepinds,:) = positions_nuc{7};
    
    [ embedding_nucleus ] = embed_distance_matrix( options_nuc.positions, options_nuc );
    
    save(embedding_file, 'embedding_cell', 'embedding_nucleus')
else
    load(embedding_file)
end

cellmodel.model.cellShapeModel.positions = embedding_cell.positions;
cellmodel.model.cellShapeModel.convex_hull = embedding_cell.convex_hull;
cellmodel.model.cellShapeModel.tessellation = embedding_cell.tessellation;
cellmodel.model.nuclearShapeModel = cellmodel.model.cellShapeModel;


nucmodel.model.cellShapeModel.positions = embedding_nucleus.positions;
nucmodel.model.cellShapeModel.convex_hull = embedding_nucleus.convex_hull;
nucmodel.model.cellShapeModel.tessellation = embedding_nucleus.tessellation;
nucmodel.model.nuclearShapeModel = nucmodel.model.cellShapeModel;


[nuc2cell_framefiles_pred, nuc2cell_framefiles_perm ] = shape_generator_pred_struct(cellmodel.model, dat{2}.nuc2cell{7}, [saveparent filesep 'synth_nuc2cell']);
[cell2nuc_framefiles_pred, cell2nuc_framefiles_perm ] = shape_generator_pred_struct(nucmodel.model, dat{2}.cell2nuc{7}, [saveparent filesep 'synth_cell2nuc']);

dice_sig2_file = [saveparent filesep 'dice_sig2.mat'];
if ~exist(dice_sig2_file, 'file')
    [nuc2cell_dice_pval2, dice_pred_cell, dice_dists_cell] = dice_sig2(cellmodel.model, keepinds, nuc2cell_framefiles_pred);
    [cell2nuc_dice_pval2, dice_pred_nuc, dice_dists_nuc] = dice_sig2(nucmodel.model, keepinds, cell2nuc_framefiles_pred);
    
    save(dice_sig2_file, 'nuc2cell_dice_pval2', 'dice_pred_cell', 'dice_dists_cell', ...
                            'cell2nuc_dice_pval2', 'dice_pred_nuc', 'dice_dists_nuc')
else
    load(dice_sig2_file)
end

[nuc2cell_dice_pval] = dice_sig(cellmodel.model, keepinds, nuc2cell_framefiles_pred, nuc2cell_framefiles_perm);
[cell2nuc_dice_pval] = dice_sig(nucmodel.model, keepinds, cell2nuc_framefiles_pred, cell2nuc_framefiles_perm);



cellsize = size(cellmodel.model.cellShapeModel.imfunc(1));
nucsize = size(nucmodel.model.cellShapeModel.imfunc(1));

imparam.justz = true;
imparam.colors = [1,1,1];

dists = pdist2([0,0; 0,1;1,0;1,1], [dat{2}.cell2nuc{7}.y_pval, dat{2}.nuc2cell{7}.y_pval]);
[~,min_inds] = min(dists,[],2);

cell_pvals = dat{2}.cell2nuc{7}.y_pval(min_inds);
nuc_pvals = dat{2}.nuc2cell{7}.y_pval(min_inds);

for j = 1:length(min_inds)

    synthfile = [ pred_img_dir filesep 'shape' num2str(j) '.mat'];

    synthfile_temp = [synthfile '.tmp'];
    if ~exist(synthfile, 'file') && ~exist(synthfile_temp, 'file') && ~param_in.skipsynth
        system(['touch ' synthfile_temp]);

%         disp(['pred:' num2str(i) filesep num2str(ndat)])

        synthparam.position = dat{2}.nuc2cell{7}.y_pred(min_inds(j),:); 
        synthparam.tempdir = [pred_img_dir filesep 'temp' filesep 'cell' num2str(j)];
        
        if ~exist(synthparam.tempdir, 'dir')
            mkdir(synthparam.tempdir)
        end
        cd(synthparam.tempdir)
        
        cell_pred = model2img({cellmodel.model}, synthparam);
        

        synthparam.position = dat{2}.cell2nuc{7}.y_pred(min_inds(j),:);
        synthparam.tempdir = [pred_img_dir filesep 'temp' filesep 'nuc' num2str(j)];

        if ~exist(synthparam.tempdir, 'dir')
            mkdir(synthparam.tempdir)
        end
        cd(synthparam.tempdir)
        
        nuc_pred = model2img({nucmodel.model}, synthparam);

        save(synthfile, 'cell_pred', 'nuc_pred')

        system(['rm ' synthfile_temp])
    end
    
    if exist(synthfile, 'file')
        load(synthfile)
        if ~isempty(cell_pred)
            
            
            im_pred = cropimg(cell_pred{1});
            predsize = size(im_pred);
            diffsize = cellsize - predsize;
            
            add_dims = diffsize./2 ~= round(diffsize./2);

            im_pred = padarray(im_pred, floor(diffsize./2), 'pre');
            im_pred = padarray(im_pred, floor(diffsize./2)+add_dims, 'post');
           
            im_true = im2projection_RGB(cellmodel.model.cellShapeModel.imfunc(keepinds(min_inds(j))), imparam);
            im_pred = im2projection_RGB(im_pred, imparam);
            
            img = [im_true im_pred].*2;
            
            imwrite(img,[ pred_img_dir filesep 'cell' num2str(j) '.png'], 'png')
            img(repmat(all(img == 0,3),[1,1,3])) = 255;
            imwrite(img,[ pred_img_dir filesep 'cell' num2str(j) 'white.png'], 'png')
        end
        
        if ~isempty(nuc_pred)
          
            im_pred = cropimg(nuc_pred{1});
            predsize = size(im_pred);
            diffsize = nucsize - predsize;
            
            add_dims = diffsize./2 ~= round(diffsize./2);

            im_pred = padarray(im_pred, floor(diffsize./2), 'pre');
            im_pred = padarray(im_pred, floor(diffsize./2)+add_dims, 'post');
           
            im_true = im2projection_RGB(nucmodel.model.cellShapeModel.imfunc(keepinds(min_inds(j))), imparam);
            im_pred = im2projection_RGB(im_pred, imparam);
            
            img = [im_true im_pred].*2;
            
            imwrite(img,[ pred_img_dir filesep 'nuc' num2str(j) '.png'], 'png')
            img(repmat(all(img == 0,3),[1,1,3])) = 255;
            imwrite(img,[ pred_img_dir filesep 'nuc' num2str(j) 'white.png'], 'png')
            
        end
        
    end
    
end


pred_img_dir = [saveparent filesep 'synth_imgs'];

% for j = 1:size(positions_cell{7},1)
% 
%     disp([num2str(j) filesep num2str(size(positions_cell{7},1))]);
%     
%     synthfile = [ pred_img_dir filesep 'shape' num2str(j) '.mat'];
% 
%     synthfile_temp = [synthfile '.tmp'];
%     if ~exist(synthfile, 'file') && ~exist(synthfile_temp, 'file') && ~param_in.build
%         system(['touch ' synthfile_temp]);
% 
% %         disp(['pred:' num2str(i) filesep num2str(ndat)])
% 
%         synthparam.position = dat{2}.nuc2cell{7}.y_pred(j,:); 
%         synthparam.tempdir = [pred_img_dir filesep 'temp' filesep 'cell' num2str(j)];
%         
%         if ~exist(synthparam.tempdir, 'dir')
%             mkdir(synthparam.tempdir)
%         end
%         cd(synthparam.tempdir)
%         
%         cell_pred = model2img({cellmodel.model}, synthparam);
% 
%         synthparam.position = dat{2}.cell2nuc{7}.y_pred(j,:);
%         synthparam.tempdir = [pred_img_dir filesep 'temp' filesep 'nuc' num2str(j)];
% 
%         if ~exist(synthparam.tempdir, 'dir')
%             mkdir(synthparam.tempdir)
%         end
%         cd(synthparam.tempdir)
%         
%         nuc_pred = model2img({nucmodel.model}, synthparam);
% 
%         save(synthfile, 'cell_pred', 'nuc_pred')
% 
%         system(['rm ' synthfile_temp])
%     end
%     
%     if exist(synthfile, 'file')
%         load(synthfile)
%         if ~isempty(cell_pred)
%             
%             
%             im_pred = cropimg(cell_pred{1});
%             predsize = size(im_pred);
%             diffsize = cellsize - predsize;
%             
%             add_dims = diffsize./2 ~= round(diffsize./2);
% 
%             im_pred = padarray(im_pred, floor(diffsize./2), 'pre');
%             im_pred = padarray(im_pred, floor(diffsize./2)+add_dims, 'post');
%            
%             im_true = im2projection_RGB(cellmodel.model.cellShapeModel.imfunc(keepinds(j)), imparam);
%             im_pred = im2projection_RGB(im_pred, imparam);
%             
%             img = [im_true im_pred].*2;
%             
%             if dat{2}.nuc2cell{7}.y_pval(j) < 0.05
%                 suffix = '_sig';
%             else
%                 suffix = '_not_sig';
%             end
%             
%             imwrite(img,[ pred_img_dir filesep 'cell' num2str(j) suffix '.png'], 'png')
%             img(repmat(all(img == 0,3),[1,1,3])) = 255;
%             imwrite(img,[ pred_img_dir filesep 'cell' num2str(j) suffix 'white.png'], 'png')
%         end
%         
%         if ~isempty(nuc_pred)
%             
%             try
%             im_pred = cropimg(nuc_pred{1});
%             predsize = size(im_pred);
%             diffsize = nucsize - predsize;
%             
%             add_dims = diffsize./2 ~= round(diffsize./2);
% 
%             im_pred = padarray(im_pred, floor(diffsize./2), 'pre');
%             im_pred = padarray(im_pred, floor(diffsize./2)+add_dims, 'post');
%            
%             im_true = im2projection_RGB(nucmodel.model.cellShapeModel.imfunc(keepinds(j)), imparam);
%             im_pred = im2projection_RGB(im_pred, imparam);
%             
%             img = [im_true im_pred].*2;
%             
%             if dat{2}.cell2nuc{7}.y_pval(j) < 0.05
%                 suffix = '_sig';
%             else
%                 suffix = '_not_sig';
%             end
%             
%             imwrite(img,[ pred_img_dir filesep 'nuc' num2str(j) suffix '.png'], 'png')
%             img(repmat(all(img == 0,3),[1,1,3])) = 255;
%             imwrite(img,[ pred_img_dir filesep 'nuc' num2str(j) suffix 'white.png'], 'png')
%             
%             catch
%             end
%         end
%         
%     end
%     
% end





fields = fieldnames(dat{1});
fields(ismember(fields,'keepinds')) = [];

figure('color', 'w')
colors = jet(length(fields));
for i = 1:length(dat)
    subplot(1, length(dat), i)
    hold on
    for j = 1:length(fields)
        plot(1:15,cellfun(@(x) x.y_pval_all, eval(['dat{i}.' fields{j}])), 'color', colors(j,:))
    end
    axis([0,15, 0, 0.55])
    
    title(methods{i})
    xlabel('# dimensions')
    ylabel('p-value')
    legend(fields)
end

colors = jet(length(dat{1}.nuc2cell));
figure('color', 'w')
for i = 1:length(dat)
    for j = 1:length(dat{i}.nuc2cell)
        d(:,j) = dat{i}.nuc2cell{j}.y_pval;

    end
end



% param.imfunc = @(x) im2projection_RGB(model.cellShapeModel.imfunc(x), p2)

load('cellModel_cell.mat')
imfunc_cell = model.cellShapeModel.imfunc;
load('cellModel_nuc.mat')
imfunc_nuc = model.cellShapeModel.imfunc;


param.subsize = 150;

i = 2;
dim_num = 7;

boundpad = 0.25;

cell2nuc = dat{i}.cell2nuc{dim_num};
nuc2cell = dat{i}.nuc2cell{dim_num};

cell2nuc_mse = mean(dat{i}.cell2nuc{dim_num}.y_pred_err);
nuc2cell_mse = mean(dat{i}.nuc2cell{dim_num}.y_pred_err);

positions_nuc_norm = zeros(size(positions_nuc{dim_num}));
positions_nuc_norm(:) = zscore(positions_nuc{dim_num}(:));

positions_cell_norm = zeros(size(positions_cell{dim_num}));
positions_cell_norm(:) = zscore(positions_cell{dim_num}(:));

colors = nan(size(model.cellShapeModel.positions,1),3);
colors(keepinds,:) = grayrange2rgb(dat{i}.cell2nuc{dim_num}.y_pval, 0,1);

model.cellShapeModel.positions = nan(size(model.cellShapeModel.positions,1), 7);
model.cellShapeModel.positions(keepinds,:) = positions_nuc_norm;

param.imfunc = @(x) (im2projection_RGB(permute(cropimg(imfunc_cell(x),0),[2,1,3]),struct('scale_inten', 2, 'justz', true, 'cm', @(y) colormap(colors(x,:)))));


h(1) = figure; 
showShapeSpaceFigure(model,[], param);
% title('cell2nuc, nuc positions')
minpos = min(model.cellShapeModel.positions,[],1);
maxpos = max(model.cellShapeModel.positions,[],1);
axis([minpos(1)-boundpad maxpos(1)+boundpad minpos(2)-boundpad maxpos(2) + boundpad])


param.imfunc = @(x) (im2projection_RGB(permute(cropimg(imfunc_nuc(x),0),[2,1,3]),struct('scale_inten', 2, 'justz', true, 'cm', @(y) colormap(colors(x,:)))));

h(2) = figure;
showShapeSpaceFigure(model,[], param);

axis([minpos(1)-boundpad maxpos(1)+boundpad minpos(2)-boundpad maxpos(2) + boundpad])
% title('cell2nuc, nuc positions')

colors = nan(size(model.cellShapeModel.positions,1),3);
colors(keepinds,:) = grayrange2rgb(dat{i}.nuc2cell{dim_num}.y_pval, 0,1);

model.cellShapeModel.positions = nan(size(model.cellShapeModel.positions,1), 7);
model.cellShapeModel.positions(keepinds,:) = positions_cell_norm;

param.imfunc = @(x) (im2projection_RGB(permute(cropimg(imfunc_cell(x),0),[2,1,3]),struct('scale_inten', 2, 'justz', true, 'cm', @(y) colormap(colors(x,:)))));

h(3) = figure;
showShapeSpaceFigure(model,[], param);

minpos = min(model.cellShapeModel.positions,[],1);
maxpos = max(model.cellShapeModel.positions,[],1);
axis([minpos(1)-boundpad maxpos(1)+boundpad minpos(2)-boundpad maxpos(2) + boundpad])
% title('nuc2cell, cell positions')

param.imfunc = @(x) (im2projection_RGB(permute(cropimg(imfunc_nuc(x),0),[2,1,3]),struct('scale_inten', 2, 'justz', true, 'cm', @(y) colormap(colors(x,:)))));

h(4) = figure;
showShapeSpaceFigure(model,[], param);
axis([minpos(1)-boundpad maxpos(1)+boundpad minpos(2)-boundpad maxpos(2) + boundpad])
% title('nuc2cell, cell positions')


h(5) = figure;
hold on, scatter(0,0,'r')
hold on, scatter(0,0,'b')
ax = colorbar;
set(ax, 'ylim', [0,1])

figdir = [saveparent filesep 'cell_vs_nuc'];
if ~exist(figdir, 'dir')
    mkdir(figdir)
end


for i = 1:length(h)
   
    set(get(h(i), 'CurrentAxes'), 'FontSize', 8)
    set(get(h(i), 'CurrentAxes'),'position',[0.05 0.05 .9 .9],'units','normalized')
%     set(get(h(i), 'CurrentAxes'), 'Axes', 'tight')
    set(h(i), 'PaperUnits', 'Inches')
    set(h(i), 'PaperSize', [3.45, 3.45])
end
set(get(h(5), 'CurrentAxes'),'position',[0.05 0.05 .85 .9],'units','normalized')

print([figdir filesep 'cell2nuc_nucpos_cells.tif'], h(1), '-dtiffn', '-r600')
print([figdir filesep 'cell2nuc_nucpos_nuc.tif'], h(2), '-dtiffn', '-r600')
print([figdir filesep 'nuc2cell_cellpos_cells.tif'], h(3), '-dtiffn', '-r600')
print([figdir filesep 'nuc2cell_cellpos_nuc.tif'], h(4), '-dtiffn', '-r600')

print([figdir filesep 'colormap.tif'], h(5), '-dtiffn', '-r600')


 load('cellModel_nuc.mat')
model.cellShapeModel.positions = nan(size(model.cellShapeModel.positions,1), 7);
model.cellShapeModel.positions(keepinds,:) = positions_cell{7};

colors = nan(size(model.cellShapeModel.positions,1),3);
colors(keepinds,:) = grayrange2rgb(dat{i}.nuc2cell{7}.y_pval, 0,1);
param.imfunc = @(x) (im2projection_RGB(model.cellShapeModel.imfunc(x),struct('scale_inten', 2, 'justz', true, 'cm', @(y) colormap(colors(x,:)))));
figure, 
showShapeSpaceFigure(model,[], param)




figdir = [saveparent filesep 'pval figs'];
if ~exist(figdir, 'dir')
    mkdir(figdir)
end


for i = 1:length(dat)
    for j = 1:length(fields)
        
        figure('color', 'w'), 

        subplot(1,2,1)
        scatter(cell2nuc{i,j}.y_pred_err, nuc2cell{i,j}.y_pred_err, '.')
        xlabel('pred nuc err'), ylabel('pred cell err')

        subplot(1,2,2)
        scatter(cell2nuc{i,j}.y_pval, nuc2cell{i,j}.y_pval, '.')
        xlabel('pval nuc'), ylabel('pval cell')

    end
end


for i = 1:length(positions_cell)
    p_cell = positions_cell{i};
    p_nuc = positions_nuc{i};
    
    cell_shape_figure = [saveparent filesep 'shapeSpace_cell_' num2str(i) '.tif'];
    cell_shape_figure_blk = [saveparent filesep 'shapeSpace_cell_blk_' num2str(i) '.tif'];
    if ~exist(cell_shape_figure, 'file')
        load('cellModel_cell.mat')
        model.cellShapeModel.positions = nan(size(model.cellShapeModel.positions,1), size(p_cell,2));
        model.cellShapeModel.positions(keepinds,:) = p_cell;
        colors = grayrange2rgb(pval_nuc{i});
        param.imfunc = @(x) (im2projection_RGB(model.cellShapeModel.imfunc(x),struct('scale_inten', 2, 'justz', true, 'cm', @(y) colormap(colors(x,:)))));

        figure, 
        showShapeSpaceFigure(model, pval_nuc{i}, param)
        saveas(gcf, cell_shape_figure, 'tif')

        img = ml_readimage(cell_shape_figure);
        whtpix = all(img == 255, 3);
        blkpix = all(img == 0,3);

        img(repmat(whtpix,[1,1,3])) = 0;
        img(repmat(blkpix,[1,1,3])) = 255;

        imwrite(img, cell_shape_figure_blk, 'tif')
    end


    param.subsize = 1000;
    nuc_shape_figure = [saveparent filesep 'shapeSpace_nuc_' num2str(i) '.tif'];
    nuc_shape_figure_blk = [saveparent filesep 'shapeSpace_nuc_blk_' num2str(i) '.tif'];
    if ~exist(nuc_shape_figure, 'file')
        load('cellModel_nuc.mat')
        model.cellShapeModel.positions = nan(size(model.cellShapeModel.positions,1), size(p_nuc,2));
        model.cellShapeModel.positions(keepinds,:) = p_nuc;

        colors = grayrange2rgb(pval_cell{i});
        param.imfunc = @(x) (im2projection_RGB(model.cellShapeModel.imfunc(x),struct('scale_inten', 4, 'justz', true, 'cm', @(y) colormap(colors(x,:)))));

        figure, 
        showShapeSpaceFigure(model, pval_cell{i}, param)
        saveas(gcf, nuc_shape_figure, 'tif')

        img = ml_readimage(nuc_shape_figure);
        whtpix = all(img == 255, 3);
        blkpix = all(img == 0,3);

        img(repmat(whtpix,[1,1,3])) = 0;
        img(repmat(blkpix,[1,1,3])) = 255;

        imwrite(img, nuc_shape_figure_blk, 'tif')
    end
end




mse_nuc = mean(pred_err_nuc);
mse_cell = mean(pred_err_cell);

p_nuc_err = mean(repmat(pred_err_nuc, 1, niter) >= [pred_err_nuc_perm{:}],2);
p_nuc_mse = mean(repmat(mse_nuc, 1, niter) >= mean([pred_err_nuc_perm{:}],1),2);
p_cell_err = mean(repmat(pred_err_cell, 1, niter) >= [pred_err_cell_perm{:}],2);
p_cell_mse = mean(repmat(mse_cell, 1, niter) >= mean([pred_err_cell_perm{:}],1),2);


t = table(ucond', mse_nuc', p_nuc_mse', mse_cell', p_cell_mse');
t.Properties.VariableNames = {'Condition', 'NuclerShapeMSE', 'NuclerShapeMSEp', 'CellShapeMSE', 'CellShapeMSEp'};

[mse_nuc, mse_cell]

figure('color', 'w'), 
subplot(1,2, 1)
bar(mse_nuc')
xlabel('condition')
ylabel('mean squared prediction error')

subplot(1,2, 2), bar(mse_cell')
xlabel('condition')


% param.subsize = 6000;
param.synthorder = keepinds;

err = inf(size(p_cell,1),1);
err(keepinds) = pred_err_nuc;


